@@ -1,5 +1,7 @@
-#include <errno.h>
 #include "dtmf.h"
+
+#include <errno.h>
+
 #include "bencode.h"
 #include "control_ng.h"
 #include "media_socket.h"
@@ -13,6 +15,65 @@
 
 static socket_t dtmf_log_sock;
 
+
+static void dtmf_trigger_block_action(struct call_media *, struct call_monologue *);
+static void dtmf_trigger_block_digit(struct call_media *, struct call_monologue *);
+static void dtmf_trigger_unblock_action(struct call_media *, struct call_monologue *);
+static void dtmf_trigger_start_rec(struct call_media *, struct call_monologue *);
+static void dtmf_trigger_stop_rec(struct call_media *, struct call_monologue *);
+static void dtmf_trigger_start_stop_rec(struct call_media *, struct call_monologue *);
+static void dtmf_trigger_pause_rec(struct call_media *, struct call_monologue *);
+static void dtmf_trigger_pause_resume_rec(struct call_media *, struct call_monologue *);
+static void dtmf_trigger_start_pause_resume_rec(struct call_media *, struct call_monologue *);
+
+struct dtmf_trigger_action dtmf_trigger_actions[__NUM_DTMF_TRIGGERS] = {
+	[DTMF_TRIGGER_BLOCK] = {
+		.matched = dtmf_trigger_block_action,
+		.repeatable = false,
+		.digit = dtmf_trigger_block_digit,
+	},
+	[DTMF_TRIGGER_UNBLOCK] = {
+		.matched = dtmf_trigger_unblock_action,
+		.repeatable = false,
+	},
+	[DTMF_TRIGGER_START_REC] = {
+		.matched = dtmf_trigger_start_rec,
+		.repeatable = true,
+	},
+	[DTMF_TRIGGER_STOP_REC] = {
+		.matched = dtmf_trigger_stop_rec,
+		.repeatable = true,
+	},
+	[DTMF_TRIGGER_START_STOP_REC] = {
+		.matched = dtmf_trigger_start_stop_rec,
+		.repeatable = true,
+	},
+	[DTMF_TRIGGER_PAUSE_REC] = {
+		.matched = dtmf_trigger_pause_rec,
+		.repeatable = true,
+	},
+	[DTMF_TRIGGER_PAUSE_RESUME_REC] = {
+		.matched = dtmf_trigger_pause_resume_rec,
+		.repeatable = true,
+	},
+	[DTMF_TRIGGER_START_PAUSE_RESUME_REC] = {
+		.matched = dtmf_trigger_start_pause_resume_rec,
+		.repeatable = true,
+	},
+};
+
+const char *dtmf_trigger_types[__NUM_DTMF_TRIGGERS] = {
+	[DTMF_TRIGGER_BLOCK] = "block DTMF",
+	[DTMF_TRIGGER_UNBLOCK] = "unblock DTMF",
+	[DTMF_TRIGGER_START_REC] = "start recording",
+	[DTMF_TRIGGER_STOP_REC] = "stop recording",
+	[DTMF_TRIGGER_START_STOP_REC] = "start/stop recording",
+	[DTMF_TRIGGER_PAUSE_REC] = "pause recording",
+	[DTMF_TRIGGER_PAUSE_RESUME_REC] = "pause/resume recording",
+	[DTMF_TRIGGER_START_PAUSE_RESUME_REC] = "start/pause/resume recording",
+};
+
+
 bool dtmf_init(void) {
 	ilog(LOG_DEBUG, "log dtmf over ng %d", rtpe_config.dtmf_via_ng);
 	ilog(LOG_DEBUG, "no log injected dtmf %d", rtpe_config.dtmf_no_log_injects);
@@ -42,7 +103,7 @@ char dtmf_code_to_char(int code) {
 static void dtmf_bencode_and_notify(struct call_media *media, unsigned int event, unsigned int volume,
 		unsigned int duration, const endpoint_t *fsin, int clockrate)
 {
-	struct call *call = media->call;
+	call_t *call = media->call;
 	struct call_monologue *ml = media->monologue;
 
 	bencode_buffer_t bencbuf;
@@ -58,13 +119,15 @@ static void dtmf_bencode_and_notify(struct call_media *media, unsigned int event
 
 	bencode_dictionary_add_string_len(data, "callid", call->callid.s, call->callid.len);
 	bencode_dictionary_add_string_len(data, "source_tag", ml->tag.s, ml->tag.len);
-
-	GList *tag_values = g_hash_table_get_values(call->tags);
-	for (GList *tag_it = tag_values; tag_it; tag_it = tag_it->next) {
-		struct call_monologue *tml = tag_it->data;
-		bencode_list_add_str(tags, &tml->tag);
+	if (ml->label.s) {
+		bencode_dictionary_add_string_len(data, "source_label", ml->label.s, ml->label.len);
 	}
-	g_list_free(tag_values);
+
+	tags_ht_iter iter;
+	t_hash_table_iter_init(&iter, call->tags);
+	struct call_monologue *tml;
+	while (t_hash_table_iter_next(&iter, NULL, &tml))
+		bencode_list_add_str(tags, &tml->tag);
 
 	bencode_dictionary_add_string(data, "type", "DTMF");
 	bencode_dictionary_add_string(data, "source_ip", sockaddr_print_buf(&fsin->address));
@@ -82,7 +145,7 @@ static GString *dtmf_json_print(struct call_media *media, unsigned int event, un
 		unsigned int duration,
 		const endpoint_t *fsin, int clockrate)
 {
-	struct call *call = media->call;
+	call_t *call = media->call;
 	struct call_monologue *ml = media->monologue;
 
 	GString *buf = g_string_new("");
@@ -93,21 +156,23 @@ static GString *dtmf_json_print(struct call_media *media, unsigned int event, un
 	g_string_append_printf(buf, "{"
 			"\"callid\":\"" STR_FORMAT "\","
 			"\"source_tag\":\"" STR_FORMAT "\","
+			"\"source_label\":\"" STR_FORMAT "\","
 			"\"tags\":[",
 			STR_FMT(&call->callid),
-			STR_FMT(&ml->tag));
+			STR_FMT(&ml->tag),
+			STR_FMT(ml->label.s ? &ml->label : &STR_EMPTY));
 
-	GList *tag_values = g_hash_table_get_values(call->tags);
+	tags_ht_iter iter;
+	t_hash_table_iter_init(&iter, call->tags);
 	int i = 0;
-	for (GList *tag_it = tag_values; tag_it; tag_it = tag_it->next) {
-		struct call_monologue *tml = tag_it->data;
+	struct call_monologue *tml;
+	while (t_hash_table_iter_next(&iter, NULL, &tml)) {
 		if (i != 0)
 			g_string_append(buf, ",");
 		g_string_append_printf(buf, "\"" STR_FORMAT "\"",
 				STR_FMT(&tml->tag));
 		i++;
 	}
-	g_list_free(tag_values);
 
 	g_string_append_printf(buf, "],"
 			"\"type\":\"DTMF\",\"timestamp\":%lu,\"source_ip\":\"%s\","
@@ -121,7 +186,7 @@ static GString *dtmf_json_print(struct call_media *media, unsigned int event, un
 	return buf;
 }
 
-bool dtmf_do_logging(const struct call *c, bool injected) {
+bool dtmf_do_logging(const call_t *c, bool injected) {
 	if (injected && rtpe_config.dtmf_no_log_injects)
 		return false;
 	if (_log_facility_dtmf)
@@ -147,7 +212,7 @@ static void dtmf_end_event(struct call_media *media, unsigned int event, unsigne
 	if (!injected) {
 		struct dtmf_event *ev = g_slice_alloc0(sizeof(*ev));
 		*ev = (struct dtmf_event) { .code = 0, .ts = ts, .volume = 0 };
-		g_queue_push_tail(&media->dtmf_recv, ev);
+		t_queue_push_tail(&media->dtmf_recv, ev);
 	}
 
 	// only add to send list if injected, a delayed send, or not being blocked
@@ -155,7 +220,7 @@ static void dtmf_end_event(struct call_media *media, unsigned int event, unsigne
 		struct dtmf_event *ev = g_slice_alloc0(sizeof(*ev));
 		*ev = (struct dtmf_event) { .code = 0, .ts = ts + media->monologue->dtmf_delay * clockrate / 1000,
 			.volume = 0, .block_dtmf = media->monologue->block_dtmf };
-		g_queue_push_tail(&media->dtmf_send, ev);
+		t_queue_push_tail(&media->dtmf_send, ev);
 	}
 
 	if (!dtmf_do_logging(media->call, injected))
@@ -183,35 +248,109 @@ static void dtmf_end_event(struct call_media *media, unsigned int event, unsigne
 	g_string_free(buf, TRUE);
 }
 
-static void dtmf_trigger_set(struct call *c, void *mlp) {
-	struct call_monologue *ml = mlp;
+static struct dtmf_trigger_state *dtmf_get_trigger_state(struct call_monologue *ml, enum dtmf_trigger_type type)
+{
+	// Look up entry in ->dtmf_triger_state. If trigger is set already, its index
+	// is stored in dtmf_trigger_index. If it isn't, grab a new entry.
+	// The index must be less then num_triggers and the type of the entry pointed
+	// to by the index must match the requested type. Everything else is invalid
+	// and requires a new entry.
+	// This keeps all set triggers at the front of the list and doesn't pollute
+	// the list with unset entries, while still allowing quick lookup.
+	// trigger_state[trigger_index[type]].type == type
+	// trigger_index[trigger_state[idx].type] == idx
+
+	unsigned int idx = ml->dtmf_trigger_index[type];
+	if (idx >= ml->num_dtmf_triggers)
+		return NULL;
+	struct dtmf_trigger_state *state = &ml->dtmf_trigger_state[idx];
+	if (state->type != type)
+		return NULL;
+	return state;
+}
+
+void dtmf_trigger_set(struct call_monologue *ml, enum dtmf_trigger_type trigger_type,
+		const str *s, bool inactive)
+{
+
+	struct dtmf_trigger_state *state = dtmf_get_trigger_state(ml, trigger_type);
+
+	if (!state) {
+		// Trigger doesn't exist yet. Do we actually want to set a trigger?
+		if (s->len == 0)
+			return; // nothing to do
+
+		// fill in a new entry
+		assert(ml->num_dtmf_triggers < __NUM_DTMF_TRIGGERS);
+		state = &ml->dtmf_trigger_state[ml->num_dtmf_triggers];
+		ml->dtmf_trigger_index[trigger_type] = ml->num_dtmf_triggers;
+		state->type = trigger_type;
+		ml->num_dtmf_triggers++;
+
+		// Trigger is set below
+	}
+	else {
+		// Trigger is already set. Do we want to delete it?
+		if (s->len == 0) {
+			// Shift down remaining items and adjust indexes
+			unsigned int idx = state - ml->dtmf_trigger_state;
+			for (unsigned int i = idx; i < ml->num_dtmf_triggers - 1; i++) {
+				assert(ml->dtmf_trigger_index[ml->dtmf_trigger_state[i].type] == i);
+				assert(ml->dtmf_trigger_index[ml->dtmf_trigger_state[i + 1].type] == i + 1);
+				ml->dtmf_trigger_state[i] = ml->dtmf_trigger_state[i + 1];
+				ml->dtmf_trigger_index[ml->dtmf_trigger_state[i].type] = i;
+			}
+			ml->num_dtmf_triggers--;
+			return;
+		}
+
+		// Replace existing trigger below
+	}
+
+	ilog(LOG_DEBUG, "Setting DTMF trigger '%s' (at idx %u) to '" STR_FORMAT "'",
+			dtmf_trigger_types[trigger_type],
+			(unsigned int) (state - ml->dtmf_trigger_state), STR_FMT(s));
+
+	call_str_cpy(ml->call, &state->trigger, s);
+	state->matched = 0;
+	state->inactive = inactive;
+}
+
+static void dtmf_trigger_set_block(call_t *c, codec_timer_callback_arg_t a) {
+	struct call_monologue *ml = a.ml;
 
 	rwlock_lock_w(&c->master_lock);
 
-	ilog(LOG_INFO, "Setting DTMF block mode to %i and setting new trigger to '" STR_FORMAT "'",
-			ml->block_dtmf_trigger, STR_FMT(&ml->dtmf_trigger_end));
+	struct dtmf_trigger_state *end_trigger = dtmf_get_trigger_state(ml, DTMF_TRIGGER_UNBLOCK);
+
+	if (end_trigger)
+		ilog(LOG_INFO, "Setting DTMF block mode to %i and enabling end trigger '" STR_FORMAT "'",
+				ml->block_dtmf_trigger, STR_FMT(&end_trigger->trigger));
+	else
+		ilog(LOG_INFO, "Setting DTMF block mode to %i",
+				ml->block_dtmf_trigger);
 
 	ml->block_dtmf = ml->block_dtmf_trigger;
 
-	// switch trigger to end trigger
-	ml->block_dtmf_trigger = ml->block_dtmf_trigger_end;
-	ml->dtmf_trigger = ml->dtmf_trigger_end;
-	ml->dtmf_trigger_end = STR_NULL;
-	ml->dtmf_trigger_digits *= -1; // negative means it's active
+	// enable end trigger
+	if (end_trigger) {
+		end_trigger->inactive = false;
+		ml->dtmf_trigger_digits *= -1; // negative means it's active
+	}
 
 	codec_update_all_handlers(ml);
 
 	rwlock_unlock_w(&c->master_lock);
 }
-static void dtmf_trigger_unset(struct call *c, void *mlp) {
-	struct call_monologue *ml = mlp;
+static void dtmf_trigger_unset_block(call_t *c, codec_timer_callback_arg_t a) {
+	struct call_monologue *ml = a.ml;
 
 	ilog(LOG_INFO, "Setting DTMF block mode to %i", ml->block_dtmf_trigger_end);
 
 	rwlock_lock_w(&c->master_lock);
 
 	ml->block_dtmf = ml->block_dtmf_trigger_end;
-	ml->dtmf_trigger = STR_NULL;
+	dtmf_trigger_set(ml, DTMF_TRIGGER_BLOCK, NULL, false);
 
 	codec_update_all_handlers(ml);
 
@@ -219,84 +358,134 @@ static void dtmf_trigger_unset(struct call *c, void *mlp) {
 }
 
 // dtmf_lock must be held
-static void dtmf_check_trigger(struct call_media *media, char event, uint64_t ts, int clockrate) {
-	struct call_monologue *ml = media->monologue;
+static void dtmf_trigger_block_digit(struct call_media *media, struct call_monologue *ml) {
+	if (ml->dtmf_trigger_digits >= 0)
+		return;
 
+	// end trigger is active
+	ml->dtmf_trigger_digits++;
+	if (ml->dtmf_trigger_digits == 0) {
+		// got all digits
+		codec_timer_callback(ml->call, dtmf_trigger_unset_block, ml, 0);
+	}
+}
+
+// dtmf_lock must be held
+static void dtmf_trigger_block_action(struct call_media *media, struct call_monologue *ml) {
+	ilog(LOG_INFO, "DTMF trigger matched, setting block mode to %i",
+			ml->block_dtmf_trigger);
+
+	// We only hold a read-lock on the call here and cannot switch to a write-lock
+	// easily, which is needed to reset the codec handlers. Therefore we do this
+	// asynchronously:
+	codec_timer_callback(ml->call, dtmf_trigger_set_block, ml, 0);
+
+	// set up unblock triggers
+	if (ml->block_dtmf_trigger_end_ms)
+		codec_timer_callback(ml->call, dtmf_trigger_unset_block, ml,
+				ml->block_dtmf_trigger_end_ms * 1000);
+}
+
+// dtmf_lock must be held
+static void dtmf_trigger_unblock_action(struct call_media *media, struct call_monologue *ml) {
+	ilog(LOG_INFO, "DTMF trigger matched, setting block mode to %i",
+			ml->block_dtmf_trigger);
+
+	// We only hold a read-lock on the call here and cannot switch to a write-lock
+	// easily, which is needed to reset the codec handlers. Therefore we do this
+	// asynchronously:
+	codec_timer_callback(ml->call, dtmf_trigger_unset_block, ml, 0);
+}
+
+// dtmf_lock must be held
+static bool dtmf_check_1_trigger(struct call_media *media, struct call_monologue *ml,
+		char event, uint64_t ts, int clockrate, unsigned int i)
+{
+	struct dtmf_trigger_state *state = &ml->dtmf_trigger_state[i];
+	struct dtmf_trigger_action *action = &dtmf_trigger_actions[state->type];
+
+	if (state->matched >= state->trigger.len) // is the trigger done already?
+		return false;
+
+	if (action->digit)
+		action->digit(media, ml);
+
+	// is the new event a match?
+	if (state->trigger.s[state->matched] == event) {
+		state->matched++;
+		if (state->matched == state->trigger.len) {
+			// trigger is finished
+			state->matched = 0; // reset
+
+			ilog(LOG_INFO, "DTMF VSC '%s' ('" STR_FORMAT "') triggered",
+					dtmf_trigger_types[state->type], STR_FMT(&state->trigger));
+
+			action->matched(media, ml);
+
+			if (!action->repeatable)
+				dtmf_trigger_set(ml, state->type, NULL, false);
+
+			return true;
+		}
+		return false;
+	}
+
+	// can we do a partial match?
+	for (size_t off = 1; off < state->matched; off++) {
+		// look for repeating prefix: trigger "ABCABD", matched 5, prefix at offset 3: [AB]C[AB]
+		if (memcmp(state->trigger.s + off, state->trigger.s, state->matched - off))
+			continue;
+		// is the new event a match?
+		unsigned int next_match_idx = state->trigger.len - off;
+		if (state->trigger.s[next_match_idx] == event) {
+			// got a partial match
+			state->matched = next_match_idx;
+			return false;
+		}
+	}
+	// no partial match... reset completely
+	if (event == state->trigger.s[0])
+		state->matched = 1;
+	else
+		state->matched = 0;
+
+	return false;
+}
+
+// dtmf_lock must be held
+static void dtmf_check_trigger(struct call_media *media, char event, uint64_t ts, int clockrate) {
 	if (!clockrate)
 		clockrate = 8000;
 
-	if (!ml->dtmf_trigger.len) // do we have a trigger?
-		return;
-	if (ml->dtmf_trigger_match >= ml->dtmf_trigger.len) // is the trigger done already?
-		return;
+	struct call_monologue *ml = media->monologue;
+
+	if (!ml->num_dtmf_triggers)
+		return; // nothing to do
 
 	// check delay from previous event
-	// dtmf_lock already held
-	struct dtmf_event *last_ev = g_queue_peek_tail(&media->dtmf_recv);
+	bool reset = false;
+	struct dtmf_event *last_ev = t_queue_peek_tail(&media->dtmf_recv);
 	if (last_ev) {
 		uint32_t ts_diff = ts - last_ev->ts;
 		uint64_t ts_diff_ms = (uint64_t) ts_diff * 1000 / clockrate;
 		if (ts_diff_ms > rtpe_config.dtmf_digit_delay) {
 			// delay too long: restart event trigger
-			ml->dtmf_trigger_match = 0;
+			reset = true;
 		}
 	}
 
-	if (ml->dtmf_trigger_digits < 0) {
-		// end trigger is active
-		ml->dtmf_trigger_digits++;
-		if (ml->dtmf_trigger_digits == 0) {
-			// got all digits
-			codec_timer_callback(ml->call, dtmf_trigger_unset, ml, 0);
-		}
+	for (unsigned int i = 0; i < ml->num_dtmf_triggers; i++) {
+		if (reset)
+			ml->dtmf_trigger_state[i].matched = 0;
+
+		if (dtmf_check_1_trigger(media, ml, event, ts, clockrate, i))
+			break; // triggers should be unique, so only act on one
 	}
-
-	// is the new event a match?
-	if (ml->dtmf_trigger.s[ml->dtmf_trigger_match] == event) {
-		ml->dtmf_trigger_match++;
-		if (ml->dtmf_trigger_match == ml->dtmf_trigger.len) {
-			// trigger is finished
-			ml->dtmf_trigger_match = 0; // reset
-
-			ilog(LOG_INFO, "DTMF trigger ('" STR_FORMAT "') matched, setting block mode to %i",
-					STR_FMT(&ml->dtmf_trigger), ml->block_dtmf_trigger);
-
-			// We only hold a read-lock on the call here and cannot switch to a write-lock
-			// easily, which is needed to reset the codec handlers. Therefore we do this
-			// asynchronously:
-			codec_timer_callback(ml->call, dtmf_trigger_set, ml, 0);
-
-			// set up unblock triggers
-			if (ml->block_dtmf_trigger_end_ms)
-				codec_timer_callback(ml->call, dtmf_trigger_unset, ml,
-						ml->block_dtmf_trigger_end_ms * 1000);
-		}
-		return;
-	}
-
-	// can we do a partial match?
-	for (size_t off = 1; off < ml->dtmf_trigger_match; off++) {
-		// look for repeating prefix: trigger "ABCABD", matched 5, prefix at offset 3: [AB]C[AB]
-		if (memcmp(ml->dtmf_trigger.s + off, ml->dtmf_trigger.s, ml->dtmf_trigger_match - off))
-			continue;
-		// is the new event a match?
-		unsigned int next_match_idx = ml->dtmf_trigger.len - off;
-		if (ml->dtmf_trigger.s[next_match_idx] == event) {
-			// got a partial match
-			ml->dtmf_trigger_match = next_match_idx;
-			return;
-		}
-	}
-	// no partial match... reset completely
-	if (event == ml->dtmf_trigger.s[0])
-		ml->dtmf_trigger_match = 1;
-	else
-		ml->dtmf_trigger_match = 0;
 }
 
 // media->dtmf_lock must be held
 static void dtmf_code_event(struct call_media *media, char event, uint64_t ts, int clockrate, int volume, bool injected) {
-	struct dtmf_event *ev = g_queue_peek_tail(&media->dtmf_recv);
+	struct dtmf_event *ev = t_queue_peek_tail(&media->dtmf_recv);
 	if (ev && ev->code == event)
 		return;
 
@@ -311,7 +500,7 @@ static void dtmf_code_event(struct call_media *media, char event, uint64_t ts, i
 		ev = g_slice_alloc0(sizeof(*ev));
 		*ev = (struct dtmf_event) { .code = event, .ts = ts, .volume = volume,
 			.rand_code = '0' + (ssl_random() % 10), .index = media->dtmf_count };
-		g_queue_push_tail(&media->dtmf_recv, ev);
+		t_queue_push_tail(&media->dtmf_recv, ev);
 	}
 
 	// only add to send list if injected, a delayed send, or not being blocked
@@ -320,14 +509,14 @@ static void dtmf_code_event(struct call_media *media, char event, uint64_t ts, i
 		*ev = (struct dtmf_event) { .code = event, .ts = ts + media->monologue->dtmf_delay * clockrate / 1000,
 			.volume = volume,
 			.block_dtmf = media->monologue->block_dtmf };
-		g_queue_push_tail(&media->dtmf_send, ev);
+		t_queue_push_tail(&media->dtmf_send, ev);
 	}
 
 	media->dtmf_count++;
 }
 
 
-struct dtmf_event *is_in_dtmf_event(GQueue *events, uint32_t this_ts, int clockrate, unsigned int head,
+struct dtmf_event *is_in_dtmf_event(dtmf_event_q *events, uint32_t this_ts, int clockrate, unsigned int head,
 		unsigned int trail)
 {
 	if (!clockrate)
@@ -339,7 +528,7 @@ struct dtmf_event *is_in_dtmf_event(GQueue *events, uint32_t this_ts, int clockr
 	uint32_t end_ts = this_ts - trail * clockrate / 1000;
 
 	// go backwards through our list of DTMF events
-	for (GList *l = events->tail; l; l = l->prev) {
+	for (__auto_type l = events->tail; l; l = l->prev) {
 		struct dtmf_event *ev = l->data;
 		uint32_t ts = ev->ts; // truncate to 32 bits
 		if (ev->code) {
@@ -442,15 +631,17 @@ void dtmf_dsp_event(const struct dtmf_event *new_event, struct dtmf_event *cur_e
 	}
 }
 
-void dtmf_event_free(void *e) {
-	g_slice_free1(sizeof(struct dtmf_event), e);
+void dtmf_event_free(struct dtmf_event *e) {
+	g_slice_free1(sizeof(*e), e);
 }
 
 // returns: 0 = no DTMF. 1 = DTMF start event. 2 = DTMF in progress. 3 = DTMF end event.
-int dtmf_event_payload(str *buf, uint64_t *pts, uint64_t duration, struct dtmf_event *cur_event, GQueue *events) {
+int dtmf_event_payload(str *buf, uint64_t *pts, uint64_t duration, struct dtmf_event *cur_event,
+		dtmf_event_q *events)
+{
 	// do we have a relevant state change?
 	struct dtmf_event prev_event = *cur_event;
-	struct dtmf_event *ev = g_queue_peek_head(events);
+	struct dtmf_event *ev = t_queue_peek_head(events);
 	while (events->length) {
 		ilog(LOG_DEBUG, "Next DTMF event starts at %" PRIu64 ". PTS now %" PRIu64, ev->ts, *pts);
 		if (ev->ts > *pts)
@@ -458,15 +649,17 @@ int dtmf_event_payload(str *buf, uint64_t *pts, uint64_t duration, struct dtmf_e
 
 		ilog(LOG_DEBUG, "DTMF state change at %" PRIu64 ": %i -> %i, duration %" PRIu64, ev->ts,
 				cur_event->code, ev->code, duration);
-		g_queue_pop_head(events);
+		t_queue_pop_head(events);
 		*cur_event = *ev;
 		dtmf_event_free(ev);
-		ev = g_queue_peek_head(events);
+		ev = t_queue_peek_head(events);
 		if (ev && ev->code == 0 && cur_event->ts < *pts) {
 			// if the start event ts was before *pts we need
 			// to adjust the end event_ts to ensure we're not shortening
 			// the event
-			ilog(LOG_DEBUG, "Delayed send of DTMF, adjusting end event_ts by %lu - %lu = %lu", *pts, cur_event->ts, *pts - cur_event->ts);
+			ilog(LOG_DEBUG, "Delayed send of DTMF, adjusting end event_ts by "
+					"%" PRIu64 " - %" PRIu64 " = %" PRIu64,
+					*pts, cur_event->ts, *pts - cur_event->ts);
 			ev->ts += *pts - cur_event->ts;
 		}
 		cur_event->ts = *pts; // canonicalise start TS
@@ -526,9 +719,9 @@ static const char *dtmf_inject_pcm(struct call_media *media, struct call_media *
 		struct codec_ssrc_handler *csh,
 		int code, int volume, int duration, int pause)
 {
-	struct call *call = monologue->call;
+	call_t *call = monologue->call;
 
-	for (GList *l = ps->rtp_sinks.head; l; l = l->next) {
+	for (__auto_type l = ps->rtp_sinks.head; l; l = l->next) {
 		struct sink_handler *sh = l->data;
 		struct packet_stream *sink_ps = sh->sink;
 		struct call_monologue *sink_ml = sink_ps->media->monologue;
@@ -697,7 +890,7 @@ const char *dtmf_inject(struct call_media *media, int code, int volume, int dura
 #endif
 
 
-enum block_dtmf_mode dtmf_get_block_mode(struct call *call, struct call_monologue *ml) {
+enum block_dtmf_mode dtmf_get_block_mode(call_t *call, struct call_monologue *ml) {
 	if (!call) {
 		if (!ml)
 			return BLOCK_DTMF_OFF;
@@ -722,3 +915,82 @@ bool is_dtmf_replace_mode(enum block_dtmf_mode mode) {
 		return true;
 	return false;
 }
+
+static void dtmf_trigger_do_start_rec(call_t *c, codec_timer_callback_arg_t a) {
+	rwlock_lock_w(&c->master_lock);
+	recording_start(c);
+	rwlock_unlock_w(&c->master_lock);
+}
+
+// dtmf_lock must be held
+static void dtmf_trigger_start_rec(struct call_media *media, struct call_monologue *ml) {
+	codec_timer_callback(ml->call, dtmf_trigger_do_start_rec, ml, 0);
+}
+
+static void dtmf_trigger_do_stop_rec(call_t *c, codec_timer_callback_arg_t a) {
+	rwlock_lock_w(&c->master_lock);
+	recording_stop(c);
+	rwlock_unlock_w(&c->master_lock);
+}
+
+// dtmf_lock must be held
+static void dtmf_trigger_stop_rec(struct call_media *media, struct call_monologue *ml) {
+	codec_timer_callback(ml->call, dtmf_trigger_do_stop_rec, ml, 0);
+}
+
+static void dtmf_trigger_do_start_stop_rec(call_t *c, codec_timer_callback_arg_t a) {
+	rwlock_lock_w(&c->master_lock);
+	if (c->recording)
+		recording_stop(c);
+	else
+		recording_start(c);
+	rwlock_unlock_w(&c->master_lock);
+}
+
+// dtmf_lock must be held
+static void dtmf_trigger_start_stop_rec(struct call_media *media, struct call_monologue *ml) {
+	codec_timer_callback(ml->call, dtmf_trigger_do_start_stop_rec, ml, 0);
+}
+
+static void dtmf_trigger_do_pause_rec(call_t *c, codec_timer_callback_arg_t a) {
+	rwlock_lock_w(&c->master_lock);
+	recording_pause(c);
+	rwlock_unlock_w(&c->master_lock);
+}
+
+// dtmf_lock must be held
+static void dtmf_trigger_pause_rec(struct call_media *media, struct call_monologue *ml) {
+	codec_timer_callback(ml->call, dtmf_trigger_do_pause_rec, ml, 0);
+}
+
+static void dtmf_trigger_do_pause_resume_rec(call_t *c, codec_timer_callback_arg_t a) {
+	rwlock_lock_w(&c->master_lock);
+	if (!c->recording) {
+		rwlock_unlock_w(&c->master_lock);
+		return;
+	}
+	if (CALL_SET(c, RECORDING_ON))
+		recording_pause(c);
+	else
+		recording_start(c);
+	rwlock_unlock_w(&c->master_lock);
+}
+
+// dtmf_lock must be held
+static void dtmf_trigger_pause_resume_rec(struct call_media *media, struct call_monologue *ml) {
+	codec_timer_callback(ml->call, dtmf_trigger_do_pause_resume_rec, ml, 0);
+}
+
+static void dtmf_trigger_do_start_pause_resume_rec(call_t *c, codec_timer_callback_arg_t a) {
+	rwlock_lock_w(&c->master_lock);
+	if (CALL_SET(c, RECORDING_ON))
+		recording_pause(c);
+	else
+		recording_start(c);
+	rwlock_unlock_w(&c->master_lock);
+}
+
+// dtmf_lock must be held
+static void dtmf_trigger_start_pause_resume_rec(struct call_media *media, struct call_monologue *ml) {
+	codec_timer_callback(ml->call, dtmf_trigger_do_start_pause_resume_rec, ml, 0);
+}
