@@ -1,7 +1,9 @@
 #include "ice.h"
+
 #include <glib.h>
 #include <sys/time.h>
 #include <unistd.h>
+
 #include "str.h"
 #include "call.h"
 #include "helpers.h"
@@ -13,8 +15,6 @@
 #include "timerthread.h"
 #include "call_interfaces.h"
 
-
-
 #if __DEBUG
 #define ICE_DEBUG 1
 #else
@@ -27,39 +27,33 @@
 #define __DBG(x...) ilogs(internals, LOG_DEBUG, x)
 #endif
 
-
-
-
 #define PAIR_FORMAT STR_FORMAT_M ":" STR_FORMAT_M ":%lu"
 #define PAIR_FMT(p) 								\
 			STR_FMT_M(&(p)->local_intf->ice_foundation),		\
 			STR_FMT_M(&(p)->remote_candidate->foundation),		\
 			(p)->remote_candidate->component_id
 
-
-
-
 struct fragment_key {
 	str call_id;
 	str from_tag;
 };
 struct sdp_fragment {
-	struct ng_buffer *ngbuf;
+	ng_buffer *ngbuf;
 	struct timeval received;
-	GQueue streams;
-	struct sdp_ng_flags flags;
+	sdp_streams_q streams;
+	sdp_ng_flags flags;
 };
 
 
 
 static void __ice_agent_free(void *p);
-static void create_random_ice_string(struct call *call, str *s, int len);
+static void create_random_ice_string(call_t *call, str *s, int len);
 static void __do_ice_checks(struct ice_agent *ag);
 static struct ice_candidate_pair *__pair_lookup(struct ice_agent *, struct ice_candidate *cand,
 		const struct local_intf *ifa);
 static void __recalc_pair_prios(struct ice_agent *ag);
 static void __role_change(struct ice_agent *ag, int new_controlling);
-static void __get_complete_components(GQueue *out, struct ice_agent *ag, GTree *t, unsigned int);
+static void __get_complete_components(candidate_pair_q *out, struct ice_agent *ag, GTree *t, unsigned int);
 static void __agent_schedule(struct ice_agent *ag, unsigned long);
 static void __agent_schedule_abs(struct ice_agent *ag, const struct timeval *tv);
 static void __agent_deschedule(struct ice_agent *ag);
@@ -93,13 +87,22 @@ const char * const ice_type_strings[] = {
 
 
 
+static unsigned int frag_key_hash(const struct fragment_key *A);
+static int frag_key_eq(const struct fragment_key *A, const struct fragment_key *B);
+static void fragment_key_free(struct fragment_key *);
+
+TYPED_GQUEUE(fragment, struct sdp_fragment)
+TYPED_GHASHTABLE(fragments_ht, struct fragment_key, fragment_q, frag_key_hash, frag_key_eq,
+		fragment_key_free, NULL)
+TYPED_GHASHTABLE_LOOKUP_INSERT(fragments_ht, fragment_key_free, fragment_q_new)
+
 static mutex_t sdp_fragments_lock;
-static GHashTable *sdp_fragments;
+static fragments_ht sdp_fragments;
 
 
 
-static void ice_update_media_streams(struct call_monologue *ml, GQueue *streams) {
-	for (GList *l = streams->head; l; l = l->next) {
+static void ice_update_media_streams(struct call_monologue *ml, sdp_streams_q *streams) {
+	for (__auto_type l = streams->head; l; l = l->next) {
 		struct stream_params *sp = l->data;
 		struct call_media *media = NULL;
 
@@ -130,30 +133,26 @@ static void ice_update_media_streams(struct call_monologue *ml, GQueue *streams)
 }
 
 
-static unsigned int frag_key_hash(const void *A) {
-	const struct fragment_key *a = A;
+static unsigned int frag_key_hash(const struct fragment_key *a) {
 	return str_hash(&a->call_id) ^ str_hash(&a->from_tag);
 }
-static int frag_key_eq(const void *A, const void *B) {
-	const struct fragment_key *a = A;
-	const struct fragment_key *b = B;
+static int frag_key_eq(const struct fragment_key *a, const struct fragment_key *b) {
 	return str_equal(&a->call_id, &b->call_id)
 		&& str_equal(&a->from_tag, &b->from_tag);
 }
 
 static void fragment_free(struct sdp_fragment *frag) {
-	sdp_streams_free(&frag->streams);
+	sdp_streams_clear(&frag->streams);
 	call_ng_free_flags(&frag->flags);
 	obj_put(frag->ngbuf);
 	g_slice_free1(sizeof(*frag), frag);
 }
-static void fragment_key_free(void *p) {
-	struct fragment_key *k = p;
+static void fragment_key_free(struct fragment_key *k) {
 	g_free(k->call_id.s);
 	g_free(k->from_tag.s);
 	g_slice_free1(sizeof(*k), k);
 }
-static void queue_sdp_fragment(struct ng_buffer *ngbuf, GQueue *streams, struct sdp_ng_flags *flags) {
+static void queue_sdp_fragment(ng_buffer *ngbuf, sdp_streams_q *streams, sdp_ng_flags *flags) {
 	ilog(LOG_DEBUG, "Queuing up SDP fragment for " STR_FORMAT_M "/" STR_FORMAT_M,
 			STR_FMT_M(&flags->call_id), STR_FMT_M(&flags->from_tag));
 
@@ -166,16 +165,16 @@ static void queue_sdp_fragment(struct ng_buffer *ngbuf, GQueue *streams, struct
 	frag->ngbuf = obj_get(ngbuf);
 	frag->streams = *streams;
 	frag->flags = *flags;
-	g_queue_init(streams);
+	t_queue_init(streams);
 	ZERO(*flags);
 
 	mutex_lock(&sdp_fragments_lock);
-	GQueue *frags = g_hash_table_lookup_queue_new(sdp_fragments, k, fragment_key_free);
-	g_queue_push_tail(frags, frag);
+	fragment_q *frags = fragments_ht_lookup_insert(sdp_fragments, k);
+	t_queue_push_tail(frags, frag);
 	mutex_unlock(&sdp_fragments_lock);
 }
-bool trickle_ice_update(struct ng_buffer *ngbuf, struct call *call, struct sdp_ng_flags *flags,
-		GQueue *streams)
+bool trickle_ice_update(ng_buffer *ngbuf, call_t *call, sdp_ng_flags *flags,
+		sdp_streams_q *streams)
 {
 	if (!flags->fragment)
 		return false;
@@ -201,11 +200,11 @@ void dequeue_sdp_fragments(struct call_monologue *monologue) {
 	k.call_id = monologue->call->callid;
 	k.from_tag = monologue->tag;
 
-	GQueue *frags = NULL;
+	fragment_q *frags = NULL;
 
 	{
 		LOCK(&sdp_fragments_lock);
-		g_hash_table_steal_extended(sdp_fragments, &k, NULL, (void **) &frags);
+		t_hash_table_steal_extended(sdp_fragments, &k, NULL, &frags);
 		if (!frags)
 			return;
 
@@ -213,7 +212,7 @@ void dequeue_sdp_fragments(struct call_monologue *monologue) {
 	}
 
 	struct sdp_fragment *frag;
-	while ((frag = g_queue_pop_head(frags))) {
+	while ((frag = t_queue_pop_head(frags))) {
 		if (timeval_diff(&rtpe_now, &frag->received) > MAX_FRAG_AGE)
 			goto next;
 
@@ -226,30 +225,28 @@ next:
 		fragment_free(frag);
 	}
 
-	g_queue_free(frags);
+	t_queue_free(frags);
 }
-static gboolean fragment_check_cleanup(void *k, void *v, void *p) {
+static gboolean fragment_check_cleanup(struct fragment_key *key, fragment_q *frags, void *p) {
 	bool all = GPOINTER_TO_INT(p);
-	struct fragment_key *key = k;
-	GQueue *frags = v;
 	if (!key || !frags)
 		return TRUE;
 	while (frags->length) {
 		struct sdp_fragment *frag = frags->head->data;
 		if (!all && timeval_diff(&rtpe_now, &frag->received) <= MAX_FRAG_AGE)
 			break;
-		g_queue_pop_head(frags);
+		t_queue_pop_head(frags);
 		fragment_free(frag);
 	}
 	if (!frags->length) {
-		g_queue_free(frags);
+		t_queue_free(frags);
 		return TRUE;
 	}
 	return FALSE;
 }
 static void fragments_cleanup(bool all) {
 	mutex_lock(&sdp_fragments_lock);
-	g_hash_table_foreach_remove(sdp_fragments, fragment_check_cleanup, GINT_TO_POINTER(all));
+	t_hash_table_foreach_remove(sdp_fragments, fragment_check_cleanup, GINT_TO_POINTER(all));
 	mutex_unlock(&sdp_fragments_lock);
 }
 
@@ -296,15 +293,15 @@ static void __do_ice_pair_priority(struct ice_candidate_pair *pair) {
 static void __new_stun_transaction(struct ice_candidate_pair *pair) {
 	struct ice_agent *ag = pair->agent;
 
-	g_hash_table_remove(ag->transaction_hash, pair->stun_transaction);
+	t_hash_table_remove(ag->transaction_hash, pair->stun_transaction);
 	random_string((void *) pair->stun_transaction, sizeof(pair->stun_transaction));
-	g_hash_table_insert(ag->transaction_hash, pair->stun_transaction, pair);
+	t_hash_table_insert(ag->transaction_hash, pair->stun_transaction, pair);
 }
 
 /* agent must be locked */
 static void __all_pairs_list(struct ice_agent *ag) {
-	g_queue_clear(&ag->all_pairs_list);
-	g_tree_get_values(&ag->all_pairs_list, ag->all_pairs);
+	t_queue_clear(&ag->all_pairs_list);
+	g_tree_get_values(&ag->all_pairs_list.q, ag->all_pairs);
 }
 
 static void __tree_coll_callback(void *oo, void *nn) {
@@ -315,7 +312,7 @@ static void __tree_coll_callback(void *oo, void *nn) {
 }
 
 /* agent must be locked */
-static struct ice_candidate_pair *__pair_candidate(struct stream_fd *sfd, struct ice_agent *ag,
+static struct ice_candidate_pair *__pair_candidate(stream_fd *sfd, struct ice_agent *ag,
 		struct ice_candidate *cand)
 {
 	struct ice_candidate_pair *pair;
@@ -334,8 +331,8 @@ static struct ice_candidate_pair *__pair_candidate(struct stream_fd *sfd, struct
 	__do_ice_pair_priority(pair);
 	__new_stun_transaction(pair);
 
-	g_queue_push_tail(&ag->candidate_pairs, pair);
-	g_hash_table_insert(ag->pair_hash, pair, pair);
+	t_queue_push_tail(&ag->candidate_pairs, pair);
+	t_hash_table_insert(ag->pair_hash, pair, pair);
 	g_tree_insert_coll(ag->all_pairs, pair, pair, __tree_coll_callback);
 
 	ilogs(ice, LOG_DEBUG, "Created candidate pair "PAIR_FORMAT" between %s and %s%s%s, type %s", PAIR_FMT(pair),
@@ -346,39 +343,31 @@ static struct ice_candidate_pair *__pair_candidate(struct stream_fd *sfd, struct
 	return pair;
 }
 
-static unsigned int __pair_hash(const void *p) {
-	const struct ice_candidate_pair *pair = p;
+static unsigned int __pair_hash(const struct ice_candidate_pair *pair) {
 	return g_direct_hash(pair->local_intf) ^ g_direct_hash(pair->remote_candidate);
 }
-static int __pair_equal(const void *a, const void *b) {
-	const struct ice_candidate_pair *A = a, *B = b;
+static int __pair_equal(const struct ice_candidate_pair *A, const struct ice_candidate_pair *B) {
 	return A->local_intf == B->local_intf
 		&& A->remote_candidate == B->remote_candidate;
 }
-static unsigned int __cand_hash(const void *p) {
-	const struct ice_candidate *cand = p;
+static unsigned int __cand_hash(const struct ice_candidate *cand) {
 	return endpoint_hash(&cand->endpoint) ^ cand->component_id;
 }
-static int __cand_equal(const void *a, const void *b) {
-	const struct ice_candidate *A = a, *B = b;
+static int __cand_equal(const struct ice_candidate *A, const struct ice_candidate *B) {
 	return endpoint_eq(&A->endpoint, &B->endpoint)
 		&& A->component_id == B->component_id;
 }
-static unsigned int __found_hash(const void *p) {
-	const struct ice_candidate *cand = p;
+static unsigned int __found_hash(const struct ice_candidate *cand) {
 	return str_hash(&cand->foundation) ^ cand->component_id;
 }
-static int __found_equal(const void *a, const void *b) {
-	const struct ice_candidate *A = a, *B = b;
+static int __found_equal(const struct ice_candidate *A, const struct ice_candidate *B) {
 	return str_equal(&A->foundation, &B->foundation)
 		&& A->component_id == B->component_id;
 }
-static unsigned int __trans_hash(const void *p) {
-	const uint32_t *tp = p;
+static unsigned int __trans_hash(const uint32_t *tp) {
 	return tp[0] ^ tp[1] ^ tp[2];
 }
-static int __trans_equal(const void *a, const void *b) {
-	const uint32_t *A = a, *B = b;
+static int __trans_equal(const uint32_t *A, const uint32_t *B) {
 	return A[0] == B[0] && A[1] == B[1] && A[2] == B[2];
 }
 static int __pair_prio_cmp(const void *a, const void *b) {
@@ -401,16 +390,23 @@ static int __pair_prio_cmp(const void *a, const void *b) {
 	return 0;
 }
 
+
+TYPED_GHASHTABLE_IMPL(candidate_ht, __cand_hash, __cand_equal, NULL, NULL)
+TYPED_GHASHTABLE_IMPL(candidate_pair_ht, __pair_hash, __pair_equal, NULL, NULL)
+TYPED_GHASHTABLE_IMPL(foundation_ht, __found_hash, __found_equal, NULL, NULL)
+TYPED_GHASHTABLE_IMPL(priority_ht, g_direct_hash, g_direct_equal, NULL, NULL)
+TYPED_GHASHTABLE_IMPL(transaction_ht, __trans_hash, __trans_equal, NULL, NULL)
+
 static void __ice_agent_initialize(struct ice_agent *ag) {
 	struct call_media *media = ag->media;
-	struct call *call = ag->call;
+	call_t *call = ag->call;
 
-	ag->candidate_hash = g_hash_table_new(__cand_hash, __cand_equal);
-	ag->cand_prio_hash = g_hash_table_new(g_direct_hash, g_direct_equal);
-	ag->pair_hash = g_hash_table_new(__pair_hash, __pair_equal);
-	ag->transaction_hash = g_hash_table_new(__trans_hash, __trans_equal);
-	ag->foundation_hash = g_hash_table_new(__found_hash, __found_equal);
-	ag->agent_flags = 0;
+	ag->candidate_hash = candidate_ht_new();
+	ag->cand_prio_hash = priority_ht_new();
+	ag->pair_hash = candidate_pair_ht_new();
+	ag->transaction_hash = transaction_ht_new();
+	ag->foundation_hash = foundation_ht_new();
+	atomic64_set_na(&ag->agent_flags, 0);
 	bf_copy(&ag->agent_flags, ICE_AGENT_CONTROLLING, &media->media_flags, MEDIA_FLAG_ICE_CONTROLLING);
 	bf_copy(&ag->agent_flags, ICE_AGENT_LITE_SELF, &media->media_flags, MEDIA_FLAG_ICE_LITE_SELF);
 	ag->logical_intf = media->logical_intf;
@@ -423,15 +419,16 @@ static void __ice_agent_initialize(struct ice_agent *ag) {
 	create_random_ice_string(call, &ag->ufrag[1], 8);
 	create_random_ice_string(call, &ag->pwd[1], 26);
 
-	atomic64_set(&ag->last_activity, rtpe_now.tv_sec);
+	atomic64_set_na(&ag->last_activity, rtpe_now.tv_sec);
 }
 
 static struct ice_agent *__ice_agent_new(struct call_media *media) {
 	struct ice_agent *ag;
-	struct call *call = media->call;
+	call_t *call = media->call;
 
 	ag = obj_alloc0("ice_agent", sizeof(*ag), __ice_agent_free);
 	ag->tt_obj.tt = &ice_agents_timer_thread;
+	ag->tt_obj.thread = &ice_agents_timer_thread.threads[0]; // there's only one thread
 	ag->call = obj_get(call);
 	ag->media = media;
 	mutex_init(&ag->lock);
@@ -451,7 +448,7 @@ void ice_agent_init(struct ice_agent **agp, struct call_media *media) {
 		*agp = ag = __ice_agent_new(media);
 }
 
-static int __copy_cand(struct call *call, struct ice_candidate *dst, const struct ice_candidate *src) {
+static int __copy_cand(call_t *call, struct ice_candidate *dst, const struct ice_candidate *src) {
 	int eq = (dst->priority == src->priority);
 	*dst = *src;
 	call_str_cpy(call, &dst->foundation, &src->foundation);
@@ -490,22 +487,21 @@ void ice_restart(struct ice_agent *ag) {
 
 /* called with the call lock held in W, hence agent doesn't need to be locked */
 void ice_update(struct ice_agent *ag, struct stream_params *sp, bool allow_reset) {
-	GList *l, *k;
 	struct ice_candidate *cand, *dup;
 	struct call_media *media;
-	struct call *call;
+	call_t *call;
 	int recalc = 0;
 	unsigned int comps;
 	struct packet_stream *components[MAX_COMPONENTS], *ps;
-	GQueue *candidates;
-	struct stream_fd *sfd;
+	candidate_q *candidates;
+	stream_fd *sfd;
 
 	if (!ag)
 		return;
 
 	log_info_ice_agent(ag);
 
-	atomic64_set(&ag->last_activity, rtpe_now.tv_sec);
+	atomic64_set_na(&ag->last_activity, rtpe_now.tv_sec);
 	media = ag->media;
 	call = media->call;
 
@@ -533,13 +529,13 @@ void ice_update(struct ice_agent *ag, struct stream_params *sp, bool allow_reset
 	/* get our component streams */
 	ZERO(components);
 	comps = 0;
-	for (l = media->streams.head; l; l = l->next)
+	for (__auto_type l = media->streams.head; l; l = l->next)
 		components[comps++] = l->data;
 	if (comps == 2 && (MEDIA_ISSET(media, RTCP_MUX) || !proto_is_rtp(media->protocol)))
 		components[1] = NULL;
 
 	comps = 0;
-	for (l = candidates->head; l; l = l->next) {
+	for (__auto_type l = candidates->head; l; l = l->next) {
 		if (ag->remote_candidates.length >= MAX_ICE_CANDIDATES) {
 			ilogs(ice, LOG_WARNING, "Maxmimum number of ICE candidates exceeded");
 			break;
@@ -555,7 +551,7 @@ void ice_update(struct ice_agent *ag, struct stream_params *sp, bool allow_reset
 		if (ps) /* only count active components */
 			comps = MAX(comps, cand->component_id);
 
-		dup = g_hash_table_lookup(ag->candidate_hash, cand);
+		dup = t_hash_table_lookup(ag->candidate_hash, cand);
 		if (!sp && dup) /* this isn't a real update, so only check pairings */
 			goto pair;
 
@@ -588,7 +584,7 @@ void ice_update(struct ice_agent *ag, struct stream_params *sp, bool allow_reset
 			}
 
 			/* priority and foundation may change */
-			g_hash_table_remove(ag->foundation_hash, dup);
+			t_hash_table_remove(ag->foundation_hash, dup);
 			recalc += __copy_cand(call, dup, cand);
 		}
 		else {
@@ -596,18 +592,18 @@ void ice_update(struct ice_agent *ag, struct stream_params *sp, bool allow_reset
 					STR_FMT_M(&cand->foundation), cand->component_id);
 			dup = g_slice_alloc(sizeof(*dup));
 			__copy_cand(call, dup, cand);
-			g_hash_table_insert(ag->candidate_hash, dup, dup);
-			g_hash_table_insert(ag->cand_prio_hash, GUINT_TO_POINTER(dup->priority), dup);
-			g_queue_push_tail(&ag->remote_candidates, dup);
+			t_hash_table_insert(ag->candidate_hash, dup, dup);
+			t_hash_table_insert(ag->cand_prio_hash, GUINT_TO_POINTER(dup->priority), dup);
+			t_queue_push_tail(&ag->remote_candidates, dup);
 		}
 
-		g_hash_table_insert(ag->foundation_hash, dup, dup);
+		t_hash_table_insert(ag->foundation_hash, dup, dup);
 
 pair:
 		if (!ps)
 			continue;
 
-		for (k = ps->sfds.head; k; k = k->next) {
+		for (__auto_type k = ps->sfds.head; k; k = k->next) {
 			sfd = k->data;
 			/* skip duplicates here also */
 			if (__pair_lookup(ag, dup, sfd->local_intf))
@@ -641,17 +637,17 @@ pair:
 }
 
 
-static void ice_candidate_free(void *p) {
-	g_slice_free1(sizeof(struct ice_candidate), p);
+static void ice_candidate_free(struct ice_candidate *p) {
+	g_slice_free1(sizeof(*p), p);
 }
-void ice_candidates_free(GQueue *q) {
-	g_queue_clear_full(q, ice_candidate_free);
+void ice_candidates_free(candidate_q *q) {
+	t_queue_clear_full(q, ice_candidate_free);
 }
-static void ice_candidate_pair_free(void *p) {
+static void ice_candidate_pair_free(struct ice_candidate_pair *p) {
 	g_slice_free1(sizeof(struct ice_candidate_pair), p);
 }
-static void ice_candidate_pairs_free(GQueue *q) {
-	g_queue_clear_full(q, ice_candidate_pair_free);
+static void ice_candidate_pairs_free(candidate_pair_q *q) {
+	t_queue_clear_full(q, ice_candidate_pair_free);
 }
 
 
@@ -679,14 +675,14 @@ static void __ice_agent_free_components(struct ice_agent *ag) {
 		return;
 	}
 
-	g_queue_clear(&ag->triggered);
-	g_hash_table_destroy(ag->candidate_hash);
-	g_hash_table_destroy(ag->cand_prio_hash);
-	g_hash_table_destroy(ag->pair_hash);
-	g_hash_table_destroy(ag->transaction_hash);
-	g_hash_table_destroy(ag->foundation_hash);
+	t_queue_clear(&ag->triggered);
+	t_hash_table_destroy(ag->candidate_hash);
+	t_hash_table_destroy(ag->cand_prio_hash);
+	t_hash_table_destroy(ag->pair_hash);
+	t_hash_table_destroy(ag->transaction_hash);
+	t_hash_table_destroy(ag->foundation_hash);
 	g_tree_destroy(ag->all_pairs);
-	g_queue_clear(&ag->all_pairs_list);
+	t_queue_clear(&ag->all_pairs_list);
 	g_tree_destroy(ag->nominated_pairs);
 	g_tree_destroy(ag->succeeded_pairs);
 	g_tree_destroy(ag->valid_pairs);
@@ -726,7 +722,9 @@ static void __agent_schedule_abs(struct ice_agent *ag, const struct timeval *tv)
 
 	nxt = *tv;
 
-	mutex_lock(&ice_agents_timer_thread.lock);
+	struct timerthread_thread *tt = ag->tt_obj.thread;
+
+	mutex_lock(&tt->lock);
 	if (ag->tt_obj.last_run.tv_sec) {
 		/* make sure we don't run more often than we should */
 		diff = timeval_diff(&nxt, &ag->tt_obj.last_run);
@@ -734,7 +732,7 @@ static void __agent_schedule_abs(struct ice_agent *ag, const struct timeval *tv)
 			timeval_add_usec(&nxt, TIMER_RUN_INTERVAL * 1000 - diff);
 	}
 	timerthread_obj_schedule_abs_nl(&ag->tt_obj, &nxt);
-	mutex_unlock(&ice_agents_timer_thread.lock);
+	mutex_unlock(&tt->lock);
 }
 static void __agent_deschedule(struct ice_agent *ag) {
 	if (ag)
@@ -743,9 +741,9 @@ static void __agent_deschedule(struct ice_agent *ag) {
 
 void ice_init(void) {
 	random_string((void *) &tie_breaker, sizeof(tie_breaker));
-	timerthread_init(&ice_agents_timer_thread, ice_agents_timer_run);
+	timerthread_init(&ice_agents_timer_thread, 1, ice_agents_timer_run);
 
-	sdp_fragments = g_hash_table_new_full(frag_key_hash, frag_key_eq, fragment_key_free, NULL);
+	sdp_fragments = fragments_ht_new();
 	mutex_init(&sdp_fragments_lock);
 }
 
@@ -753,8 +751,7 @@ void ice_free(void) {
 	timerthread_free(&ice_agents_timer_thread);
 
 	fragments_cleanup(true);
-	g_hash_table_destroy(sdp_fragments);
-	sdp_fragments = NULL;
+	t_hash_table_destroy_ptr(&sdp_fragments);
 	mutex_destroy(&sdp_fragments_lock);
 }
 
@@ -771,7 +768,7 @@ static void __fail_pair(struct ice_candidate_pair *pair) {
 
 /* agent must NOT be locked, but call must be locked in R */
 static void __do_ice_check(struct ice_candidate_pair *pair) {
-	struct stream_fd *sfd = pair->sfd;
+	stream_fd *sfd = pair->sfd;
 	struct ice_agent *ag = pair->agent;
 	uint32_t prio, transact[3];
 
@@ -835,20 +832,19 @@ static int __component_find(const void *a, const void *b) {
 static struct ice_candidate_pair *__get_pair_by_component(GTree *t, unsigned int component) {
 	return g_tree_find_first(t, __component_find, GUINT_TO_POINTER(component));
 }
-static void __get_pairs_by_component(GQueue *out, GTree *t, unsigned int component) {
-	g_tree_find_all(out, t, __component_find, GUINT_TO_POINTER(component));
+static void __get_pairs_by_component(candidate_pair_q *out, GTree *t, unsigned int component) {
+	g_tree_find_all(&out->q, t, __component_find, GUINT_TO_POINTER(component));
 }
 
-static void __get_complete_succeeded_pairs(GQueue *out, struct ice_agent *ag) {
+static void __get_complete_succeeded_pairs(candidate_pair_q *out, struct ice_agent *ag) {
 	__get_complete_components(out, ag, ag->succeeded_pairs, ICE_PAIR_SUCCEEDED);
 }
-static void __get_complete_valid_pairs(GQueue *out, struct ice_agent *ag) {
+static void __get_complete_valid_pairs(candidate_pair_q *out, struct ice_agent *ag) {
 	__get_complete_components(out, ag, ag->valid_pairs, ICE_PAIR_VALID);
 }
 
 static void __nominate_pairs(struct ice_agent *ag) {
-	GQueue complete;
-	GList *l;
+	candidate_pair_q complete;
 	struct ice_candidate_pair *pair;
 
 	ilogs(ice, LOG_DEBUG, "Start nominating ICE pairs");
@@ -858,24 +854,23 @@ static void __nominate_pairs(struct ice_agent *ag) {
 
 	__get_complete_succeeded_pairs(&complete, ag);
 
-	for (l = complete.head; l; l = l->next) {
+	for (__auto_type l = complete.head; l; l = l->next) {
 		pair = l->data;
 		ilogs(ice, LOG_DEBUG, "Nominating ICE pair "PAIR_FORMAT, PAIR_FMT(pair));
 		PAIR_CLEAR(pair, IN_PROGRESS);
 		PAIR_SET2(pair, NOMINATED, TO_USE);
 		pair->retransmits = 0;
 		__new_stun_transaction(pair);
-		g_queue_push_tail(&ag->triggered, pair);
+		t_queue_push_tail(&ag->triggered, pair);
 	}
 
-	g_queue_clear(&complete);
+	t_queue_clear(&complete);
 }
 
 /* call must be locked R or W, agent must not be locked */
 static void __do_ice_checks(struct ice_agent *ag) {
-	GList *l;
 	struct ice_candidate_pair *pair, *highest = NULL, *frozen = NULL, *valid;
-	struct stream_fd *sfd;
+	stream_fd *sfd;
 	GQueue retransmits = G_QUEUE_INIT;
 	struct timeval next_run = {0,0};
 	int have_more = 0;
@@ -888,7 +883,7 @@ static void __do_ice_checks(struct ice_agent *ag) {
 	if (!ag->pwd[0].s)
 		return;
 
-	atomic64_set(&ag->last_activity, rtpe_now.tv_sec);
+	atomic64_set_na(&ag->last_activity, rtpe_now.tv_sec);
 
 	__DBG("running checks, call "STR_FORMAT" tag "STR_FORMAT"", STR_FMT(&ag->call->callid),
 			STR_FMT(&ag->media->monologue->tag));
@@ -903,7 +898,7 @@ static void __do_ice_checks(struct ice_agent *ag) {
 	}
 
 	/* triggered checks are preferred */
-	pair = g_queue_pop_head(&ag->triggered);
+	pair = t_queue_pop_head(&ag->triggered);
 	if (pair) {
 		__DBG("running triggered check on " PAIR_FORMAT, PAIR_FMT(pair));
 		PAIR_CLEAR(pair, TRIGGERED);
@@ -912,7 +907,7 @@ static void __do_ice_checks(struct ice_agent *ag) {
 	}
 
 	/* find the highest-priority non-frozen non-in-progress pair */
-	for (l = ag->all_pairs_list.head; l; l = l->next) {
+	for (__auto_type l = ag->all_pairs_list.head; l; l = l->next) {
 		pair = l->data;
 
 		__DBG("considering checking " PAIR_FORMAT, PAIR_FMT(pair));
@@ -1004,7 +999,7 @@ static struct ice_candidate *__cand_lookup(struct ice_agent *ag, const endpoint_
 
 	d.endpoint = *sin;
 	d.component_id = component;
-	return g_hash_table_lookup(ag->candidate_hash, &d);
+	return t_hash_table_lookup(ag->candidate_hash, &d);
 }
 static struct ice_candidate *__foundation_lookup(struct ice_agent *ag, const str *foundation,
 		unsigned int component)
@@ -1013,7 +1008,7 @@ static struct ice_candidate *__foundation_lookup(struct ice_agent *ag, const str
 
 	d.foundation = *foundation;
 	d.component_id = component;
-	return g_hash_table_lookup(ag->foundation_hash, &d);
+	return t_hash_table_lookup(ag->foundation_hash, &d);
 }
 static struct ice_candidate_pair *__pair_lookup(struct ice_agent *ag, struct ice_candidate *cand,
 		const struct local_intf *ifa)
@@ -1022,10 +1017,10 @@ static struct ice_candidate_pair *__pair_lookup(struct ice_agent *ag, struct ice
 
 	p.local_intf = ifa;
 	p.remote_candidate = cand;
-	return g_hash_table_lookup(ag->pair_hash, &p);
+	return t_hash_table_lookup(ag->pair_hash, &p);
 }
 
-static void __cand_ice_foundation(struct call *call, struct ice_candidate *cand) {
+static void __cand_ice_foundation(call_t *call, struct ice_candidate *cand) {
 	char buf[64];
 	int len;
 
@@ -1035,12 +1030,12 @@ static void __cand_ice_foundation(struct call *call, struct ice_candidate *cand)
 }
 
 /* agent must be locked */
-static struct ice_candidate_pair *__learned_candidate(struct ice_agent *ag, struct stream_fd *sfd,
+static struct ice_candidate_pair *__learned_candidate(struct ice_agent *ag, stream_fd *sfd,
 		const endpoint_t *src, unsigned long priority)
 {
 	struct ice_candidate *cand, *old_cand;
 	struct ice_candidate_pair *pair;
-	struct call *call = ag->call;
+	call_t *call = ag->call;
 	struct packet_stream *ps = sfd->stream;
 
 	cand = g_slice_alloc0(sizeof(*cand));
@@ -1058,7 +1053,7 @@ static struct ice_candidate_pair *__learned_candidate(struct ice_agent *ag, stru
 		if (comp == ps->component)
 			continue;
 		unsigned long prio = prio_base - comp;
-		known_cand = g_hash_table_lookup(ag->cand_prio_hash, GUINT_TO_POINTER(prio));
+		known_cand = t_hash_table_lookup(ag->cand_prio_hash, GUINT_TO_POINTER(prio));
 		if (known_cand)
 			break;
 	}
@@ -1084,10 +1079,10 @@ static struct ice_candidate_pair *__learned_candidate(struct ice_agent *ag, stru
 		goto pair;
 	}
 
-	g_queue_push_tail(&ag->remote_candidates, cand);
-	g_hash_table_insert(ag->candidate_hash, cand, cand);
-	g_hash_table_insert(ag->cand_prio_hash, GUINT_TO_POINTER(cand->priority), cand);
-	g_hash_table_insert(ag->foundation_hash, cand, cand);
+	t_queue_push_tail(&ag->remote_candidates, cand);
+	t_hash_table_insert(ag->candidate_hash, cand, cand);
+	t_hash_table_insert(ag->cand_prio_hash, GUINT_TO_POINTER(cand->priority), cand);
+	t_hash_table_insert(ag->foundation_hash, cand, cand);
 
 pair:
 	pair = __pair_candidate(sfd, ag, cand);
@@ -1109,7 +1104,7 @@ static void __trigger_check(struct ice_candidate_pair *pair) {
 	if (PAIR_CLEAR(pair, FAILED))
 		PAIR_CLEAR(pair, IN_PROGRESS);
 	if (ag->triggered.length < 4 * MAX_ICE_CANDIDATES && !PAIR_SET(pair, TRIGGERED))
-		g_queue_push_tail(&ag->triggered, pair);
+		t_queue_push_tail(&ag->triggered, pair);
 	mutex_unlock(&ag->lock);
 
 	__agent_schedule(ag, 0);
@@ -1119,7 +1114,6 @@ static void __trigger_check(struct ice_candidate_pair *pair) {
 /* also regenerates all_pairs_list */
 static void __recalc_pair_prios(struct ice_agent *ag) {
 	struct ice_candidate_pair *pair;
-	GList *l;
 	GQueue nominated, valid, succ, all;
 
 	ilogs(ice, LOG_DEBUG, "Recalculating all ICE pair priorities");
@@ -1129,7 +1123,7 @@ static void __recalc_pair_prios(struct ice_agent *ag) {
 	g_tree_find_remove_all(&valid, ag->valid_pairs);
 	g_tree_find_remove_all(&all, ag->all_pairs);
 
-	for (l = ag->candidate_pairs.head; l; l = l->next) {
+	for (__auto_type l = ag->candidate_pairs.head; l; l = l->next) {
 		pair = l->data;
 		__do_ice_pair_priority(pair);
 		/* this changes the packets, so we must keep these from being seen as retransmits */
@@ -1162,22 +1156,21 @@ static void __role_change(struct ice_agent *ag, int new_controlling) {
 }
 
 /* initializes "out" */
-static void __get_complete_components(GQueue *out, struct ice_agent *ag, GTree *t, unsigned int flag) {
-	GQueue compo1 = G_QUEUE_INIT;
-	GList *l;
+static void __get_complete_components(candidate_pair_q *out, struct ice_agent *ag, GTree *t, unsigned int flag) {
+	candidate_pair_q compo1 = TYPED_GQUEUE_INIT;
 	struct ice_candidate_pair *pair1, *pairX;
 	struct ice_candidate *cand;
 	unsigned int i;
 
 	__get_pairs_by_component(&compo1, t, 1);
 
-	g_queue_init(out);
+	t_queue_init(out);
 
-	for (l = compo1.head; l; l = l->next) {
+	for (__auto_type l = compo1.head; l; l = l->next) {
 		pair1 = l->data;
 
-		g_queue_clear(out);
-		g_queue_push_tail(out, pair1);
+		t_queue_clear(out);
+		t_queue_push_tail(out, pair1);
 
 		for (i = 2; i <= ag->active_components; i++) {
 			cand = __foundation_lookup(ag, &pair1->remote_candidate->foundation, i);
@@ -1188,7 +1181,7 @@ static void __get_complete_components(GQueue *out, struct ice_agent *ag, GTree *
 				goto next_foundation;
 			if (!bf_isset(&pairX->pair_flags, flag))
 				goto next_foundation;
-			g_queue_push_tail(out, pairX);
+			t_queue_push_tail(out, pairX);
 		}
 		goto found;
 
@@ -1197,21 +1190,22 @@ next_foundation:
 	}
 
 	/* nothing found */
-	g_queue_clear(out);
+	t_queue_clear(out);
 
 found:
-	g_queue_clear(&compo1);
+	t_queue_clear(&compo1);
 }
 
 /* call(W) or call(R)+agent must be locked - no in_lock or out_lock must be held */
 static int __check_valid(struct ice_agent *ag) {
 	struct call_media *media;
 	struct packet_stream *ps;
-	GList *l, *k, *m;
-	GQueue all_compos;
+	packet_stream_list *l;
+	candidate_pair_list *k;
+	candidate_pair_q all_compos;
 	struct ice_candidate_pair *pair;
 //	const struct local_intf *ifa;
-	struct stream_fd *sfd;
+	stream_fd *sfd;
 	int is_complete = 1;
 
 	if (!ag) {
@@ -1258,7 +1252,7 @@ static int __check_valid(struct ice_agent *ag) {
 					FMT_M(endpoint_print_buf(&pair->remote_candidate->endpoint)));
 		mutex_unlock(&ps->out_lock);
 
-		for (m = ps->sfds.head; m; m = m->next) {
+		for (__auto_type m = ps->sfds.head; m; m = m->next) {
 			sfd = m->data;
 			if (sfd->local_intf != pair->local_intf)
 				continue;
@@ -1272,7 +1266,7 @@ static int __check_valid(struct ice_agent *ag) {
 
 	call_media_unkernelize(media, "ICE negotiation event");
 
-	g_queue_clear(&all_compos);
+	t_queue_clear(&all_compos);
 	return 1;
 }
 
@@ -1284,7 +1278,7 @@ static int __check_valid(struct ice_agent *ag) {
  * -1 = generic error, process packet as normal
  * -2 = role conflict
  */
-int ice_request(struct stream_fd *sfd, const endpoint_t *src,
+int ice_request(stream_fd *sfd, const endpoint_t *src,
 		struct stun_attrs *attrs)
 {
 	struct packet_stream *ps = sfd->stream;
@@ -1302,7 +1296,7 @@ int ice_request(struct stream_fd *sfd, const endpoint_t *src,
 	if (!ag)
 		return -1;
 
-	atomic64_set(&ag->last_activity, rtpe_now.tv_sec);
+	atomic64_set_na(&ag->last_activity, rtpe_now.tv_sec);
 
 	/* determine candidate pair */
 	{
@@ -1349,7 +1343,7 @@ int ice_request(struct stream_fd *sfd, const endpoint_t *src,
 	if (attrs->use && !PAIR_SET(pair, NOMINATED)) {
 		ilogs(ice, LOG_DEBUG, "ICE pair "PAIR_FORMAT" has been nominated by peer", PAIR_FMT(pair));
 
-		mutex_lock(&ag->lock);
+		LOCK(&ag->lock);
 
 		// coverity[use : FALSE]
 		g_tree_insert_coll(ag->nominated_pairs, pair, pair, __tree_coll_callback);
@@ -1361,8 +1355,6 @@ int ice_request(struct stream_fd *sfd, const endpoint_t *src,
 
 		if (!AGENT_ISSET(ag, CONTROLLING))
 			ret = __check_valid(ag);
-
-		mutex_unlock(&ag->lock);
 	}
 
 	return ret;
@@ -1375,7 +1367,7 @@ err:
 
 
 static int __check_succeeded_complete(struct ice_agent *ag) {
-	GQueue complete;
+	candidate_pair_q complete;
 	int ret;
 
 	__get_complete_succeeded_pairs(&complete, ag);
@@ -1388,12 +1380,12 @@ static int __check_succeeded_complete(struct ice_agent *ag) {
 		ilogs(ice, LOG_DEBUG, "No succeeded ICE pairs with all components yet");
 		ret = 0;
 	}
-	g_queue_clear(&complete);
+	t_queue_clear(&complete);
 	return ret;
 }
 
 /* call is locked in R */
-int ice_response(struct stream_fd *sfd, const endpoint_t *src,
+int ice_response(stream_fd *sfd, const endpoint_t *src,
 		struct stun_attrs *attrs, void *transaction)
 {
 	struct ice_candidate_pair *pair, *opair;
@@ -1413,12 +1405,12 @@ int ice_response(struct stream_fd *sfd, const endpoint_t *src,
 	if (!ag)
 		return -1;
 
-	atomic64_set(&ag->last_activity, rtpe_now.tv_sec);
+	atomic64_set_na(&ag->last_activity, rtpe_now.tv_sec);
 
 	{
 		LOCK(&ag->lock);
 
-		pair = g_hash_table_lookup(ag->transaction_hash, transaction);
+		pair = t_hash_table_lookup(ag->transaction_hash, transaction);
 		err = "ICE/STUN response with unknown transaction received";
 		if (!pair)
 			goto err;
@@ -1527,12 +1519,12 @@ err:
 
 
 
-void ice_thread_run(void *p) {
-	timerthread_run(&ice_agents_timer_thread);
+void ice_thread_launch(void) {
+	timerthread_launch(&ice_agents_timer_thread, NULL, 0, "ICE");
 }
 static void ice_agents_timer_run(void *ptr) {
 	struct ice_agent *ag = ptr;
-	struct call *call;
+	call_t *call;
 
 	call = ag->call;
 	log_info_ice_agent(ag);
@@ -1551,7 +1543,7 @@ static void random_ice_string(char *buf, int len) {
 		*buf++ = ice_chars[ssl_random() % strlen(ice_chars)];
 }
 
-static void create_random_ice_string(struct call *call, str *s, int len) {
+static void create_random_ice_string(call_t *call, str *s, int len) {
 	char buf[30];
 
 	assert(len < sizeof(buf));
@@ -1567,23 +1559,22 @@ void ice_foundation(str *s) {
 	random_ice_string(s->s, ICE_FOUNDATION_LENGTH);
 }
 
-void ice_remote_candidates(GQueue *out, struct ice_agent *ag) {
-	GQueue all_compos;
-	GList *l;
+void ice_remote_candidates(candidate_q *out, struct ice_agent *ag) {
+	candidate_pair_q all_compos;
 	struct ice_candidate_pair *pair;
 
-	g_queue_init(out);
+	t_queue_init(out);
 
 	mutex_lock(&ag->lock);
 	__get_complete_valid_pairs(&all_compos, ag);
 	mutex_unlock(&ag->lock);
 
-	for (l = all_compos.head; l; l = l->next) {
+	for (__auto_type l = all_compos.head; l; l = l->next) {
 		pair = l->data;
-		g_queue_push_tail(out, pair->remote_candidate);
+		t_queue_push_tail(out, pair->remote_candidate);
 	}
 
-	g_queue_clear(&all_compos);
+	t_queue_clear(&all_compos);
 }
 
 bool ice_peer_address_known(struct ice_agent *ag, const endpoint_t *sin, struct packet_stream *ps,
