@@ -1,17 +1,19 @@
+#include "statistics.h"
+
 #include <math.h>
 #include <stdarg.h>
+
 #include "call.h"
-#include "statistics.h"
 #include "graphite.h"
 #include "main.h"
 #include "control_ng.h"
-
+#include "bufferpool.h"
 
 struct timeval rtpe_started;
 
 
 mutex_t rtpe_codec_stats_lock;
-GHashTable *rtpe_codec_stats;
+codec_stats_ht rtpe_codec_stats;
 
 
 struct global_stats_gauge rtpe_stats_gauge;			// master values
@@ -20,14 +22,14 @@ struct global_gauge_min_max rtpe_gauge_min_max;			// master lifetime min/max
 struct global_stats_sampled rtpe_stats_sampled;			// master cumulative values
 struct global_sampled_min_max rtpe_sampled_min_max;		// master lifetime min/max
 
-struct global_stats_counter rtpe_stats;				// total, cumulative, master
+struct global_stats_counter *rtpe_stats;			// total, cumulative, master
 struct global_stats_counter rtpe_stats_rate;			// per-second, calculated once per timer run
 struct global_stats_counter rtpe_stats_intv;			// calculated once per sec by `call_rate_stats_updater()`
 
 
 // op can be CMC_INCREMENT or CMC_DECREMENT
 // check not to multiple decrement or increment
-void statistics_update_ip46_inc_dec(struct call* c, int op) {
+void statistics_update_ip46_inc_dec(call_t * c, int op) {
 	// already incremented
 	if (op == CMC_INCREMENT && CALL_ISSET(c, MEDIA_COUNTED)) {
 		return ;
@@ -103,13 +105,13 @@ void statistics_update_ip46_inc_dec(struct call* c, int op) {
 	bf_set_clear(&c->call_flags, CALL_FLAG_MEDIA_COUNTED, op == CMC_INCREMENT);
 }
 
-void statistics_update_foreignown_dec(struct call* c) {
+void statistics_update_foreignown_dec(call_t * c) {
 	if (IS_FOREIGN_CALL(c)) {
 		RTPE_GAUGE_DEC(foreign_sessions);
 	}
 }
 
-void statistics_update_foreignown_inc(struct call* c) {
+void statistics_update_foreignown_inc(call_t * c) {
 	if (IS_FOREIGN_CALL(c)) { /* foreign call*/
 		RTPE_GAUGE_INC(foreign_sessions);
 		RTPE_STATS_INC(foreign_sess);
@@ -117,18 +119,16 @@ void statistics_update_foreignown_inc(struct call* c) {
 
 }
 
-void statistics_update_oneway(struct call* c) {
+void statistics_update_oneway(call_t * c) {
 	struct call_monologue *ml;
 	struct call_media *md;
-	GList *o;
-	GList *l;
 
 	if (IS_OWN_CALL(c)) {
 		// --- for statistics getting one way stream or no relay at all
 		unsigned int total_nopacket_relayed_sess = 0;
 		struct packet_stream *ps, *ps2;
 
-		for (l = c->monologues.head; l; l = l->next) {
+		for (__auto_type l = c->monologues.head; l; l = l->next) {
 			ml = l->data;
 
 			// --- go through partner ml and search the RTP
@@ -137,7 +137,7 @@ void statistics_update_oneway(struct call* c) {
 				if (!md)
 					continue;
 
-				for (o = md->streams.head; o; o = o->next) {
+				for (__auto_type o = md->streams.head; o; o = o->next) {
 					ps = o->data;
 					if (PS_ISSET(ps, RTP)) {
 						// --- only RTP is interesting
@@ -149,13 +149,13 @@ void statistics_update_oneway(struct call* c) {
 			continue;
 
 found:;
-			struct sink_handler *sh = g_queue_peek_head(&ps->rtp_sinks);
+			struct sink_handler *sh = t_queue_peek_head(&ps->rtp_sinks);
 			ps2 = sh ? sh->sink : NULL;
 			if (!ps2)
 				continue;
 
-			if (atomic64_get(&ps2->stats_in.packets)==0) {
-				if (atomic64_get(&ps->stats_in.packets)!=0)
+			if (atomic64_get_na(&ps2->stats_in->packets)==0) {
+				if (atomic64_get_na(&ps->stats_in->packets)!=0)
 					RTPE_STATS_INC(oneway_stream_sess);
 				else
 					total_nopacket_relayed_sess++;
@@ -197,26 +197,26 @@ found:;
 }
 
 
-INLINE void prom_metric(GQueue *ret, const char *name, const char *type) {
-	struct stats_metric *last = g_queue_peek_tail(ret);
+INLINE void prom_metric(stats_metric_q *ret, const char *name, const char *type) {
+	stats_metric *last = t_queue_peek_tail(ret);
 	last->prom_name = name;
 	last->prom_type = type;
 }
-static void prom_label(GQueue *ret, const char *fmt, ...) {
+static void prom_label(stats_metric_q *ret, const char *fmt, ...) {
 	if (!fmt)
 		return;
 	va_list ap;
 	va_start(ap, fmt);
-	struct stats_metric *last = g_queue_peek_tail(ret);
+	stats_metric *last = t_queue_peek_tail(ret);
 	last->prom_label = g_strdup_vprintf(fmt, ap);
 	va_end(ap);
 }
 #define PROM(name, type) prom_metric(ret, name, type)
 #define PROMLAB(fmt, ...) prom_label(ret, fmt, ##__VA_ARGS__)
 
-INLINE void metric_push(GQueue *ret, struct stats_metric *m) {
-	struct stats_metric *last = NULL;
-	for (GList *l_last = ret->tail; l_last; l_last = l_last->prev) {
+INLINE void metric_push(stats_metric_q *ret, stats_metric *m) {
+	stats_metric *last = NULL;
+	for (__auto_type l_last = ret->tail; l_last; l_last = l_last->prev) {
 		last = l_last->data;
 		if (last->label)
 			break;
@@ -228,12 +228,12 @@ INLINE void metric_push(GQueue *ret, struct stats_metric *m) {
 	}
 	else if (m->is_bracket && !m->is_close_bracket && last && last->is_close_bracket)
 		m->is_follow_up = 1;
-	g_queue_push_tail(ret, m);
+	t_queue_push_tail(ret, m);
 }
-static void add_metric(GQueue *ret, const char *label, const char *desc, const char *fmt1, const char *fmt2, ...) {
+static void add_metric(stats_metric_q *ret, const char *label, const char *desc, const char *fmt1, const char *fmt2, ...) {
 	va_list ap;
 
-	struct stats_metric *m = g_slice_alloc0(sizeof(*m));
+	stats_metric *m = g_slice_alloc0(sizeof(*m));
 	if (label)
 		m->label = g_strdup(label);
 	if (desc)
@@ -273,10 +273,10 @@ static void add_metric(GQueue *ret, const char *label, const char *desc, const c
 	}
 	metric_push(ret, m);
 }
-static void add_header(GQueue *ret, const char *fmt1, const char *fmt2, ...) {
+static void add_header(stats_metric_q *ret, const char *fmt1, const char *fmt2, ...) {
 	va_list ap;
 
-	struct stats_metric *m = g_slice_alloc0(sizeof(*m));
+	stats_metric *m = g_slice_alloc0(sizeof(*m));
 	if (fmt1) {
 		va_start(ap, fmt2); // coverity[copy_paste_error : FALSE]
 		m->label = g_strdup_vprintf(fmt1, ap);
@@ -313,8 +313,8 @@ static void add_header(GQueue *ret, const char *fmt1, const char *fmt2, ...) {
 #define HEADERl(fmt2, ...) add_header(ret, NULL, fmt2, ##__VA_ARGS__)
 
 
-GQueue *statistics_gather_metrics(struct interface_sampled_rate_stats *interface_rate_stats) {
-	GQueue *ret = g_queue_new();
+stats_metric_q *statistics_gather_metrics(struct interface_sampled_rate_stats *interface_rate_stats) {
+	stats_metric_q *ret = stats_metric_q_new();
 
 	double calls_dur_iv;
 	uint64_t cur_sessions, num_sessions, min_sess_iv, max_sess_iv;
@@ -324,59 +324,59 @@ GQueue *statistics_gather_metrics(struct interface_sampled_rate_stats *interface
 	HEADER("{", "");
 
 	rwlock_lock_r(&rtpe_callhash_lock);
-	cur_sessions = g_hash_table_size(rtpe_callhash);
+	cur_sessions = t_hash_table_size(rtpe_callhash);
 	rwlock_unlock_r(&rtpe_callhash_lock);
 
-	METRIC("sessionsown", "Owned sessions", UINT64F, UINT64F, cur_sessions - atomic64_get(&rtpe_stats_gauge.foreign_sessions));
+	METRIC("sessionsown", "Owned sessions", UINT64F, UINT64F, cur_sessions - atomic64_get_na(&rtpe_stats_gauge.foreign_sessions));
 	PROM("sessions", "gauge");
 	PROMLAB("type=\"own\"");
-	METRIC("sessionsforeign", "Foreign sessions", UINT64F, UINT64F, atomic64_get(&rtpe_stats_gauge.foreign_sessions));
+	METRIC("sessionsforeign", "Foreign sessions", UINT64F, UINT64F, atomic64_get_na(&rtpe_stats_gauge.foreign_sessions));
 	PROM("sessions", "gauge");
 	PROMLAB("type=\"foreign\"");
 
 	METRIC("sessionstotal", "Total sessions", UINT64F, UINT64F, cur_sessions);
-	METRIC("transcodedmedia", "Transcoded media", UINT64F, UINT64F, atomic64_get(&rtpe_stats_gauge.transcoded_media));
+	METRIC("transcodedmedia", "Transcoded media", UINT64F, UINT64F, atomic64_get_na(&rtpe_stats_gauge.transcoded_media));
 	PROM("transcoded_media", "gauge");
 
 	METRIC("packetrate_user", "Packets per second (userspace)", UINT64F, UINT64F,
-			atomic64_get(&rtpe_stats_rate.packets_user));
+			atomic64_get_na(&rtpe_stats_rate.packets_user));
 	METRIC("byterate_user", "Bytes per second (userspace)", UINT64F, UINT64F,
-			atomic64_get(&rtpe_stats_rate.bytes_user));
+			atomic64_get_na(&rtpe_stats_rate.bytes_user));
 	METRIC("errorrate_user", "Errors per second (userspace)", UINT64F, UINT64F,
-			atomic64_get(&rtpe_stats_rate.errors_user));
+			atomic64_get_na(&rtpe_stats_rate.errors_user));
 	METRIC("packetrate_kernel", "Packets per second (kernel)", UINT64F, UINT64F,
-			atomic64_get(&rtpe_stats_rate.packets_kernel));
+			atomic64_get_na(&rtpe_stats_rate.packets_kernel));
 	METRIC("byterate_kernel", "Bytes per second (kernel)", UINT64F, UINT64F,
-			atomic64_get(&rtpe_stats_rate.bytes_kernel));
+			atomic64_get_na(&rtpe_stats_rate.bytes_kernel));
 	METRIC("errorrate_kernel", "Errors per second (kernel)", UINT64F, UINT64F,
-			atomic64_get(&rtpe_stats_rate.errors_kernel));
+			atomic64_get_na(&rtpe_stats_rate.errors_kernel));
 	METRIC("packetrate", "Packets per second (total)", UINT64F, UINT64F,
-			atomic64_get(&rtpe_stats_rate.packets_user) +
-			atomic64_get(&rtpe_stats_rate.packets_kernel));
+			atomic64_get_na(&rtpe_stats_rate.packets_user) +
+			atomic64_get_na(&rtpe_stats_rate.packets_kernel));
 	METRIC("byterate", "Bytes per second (total)", UINT64F, UINT64F,
-			atomic64_get(&rtpe_stats_rate.bytes_user) +
-			atomic64_get(&rtpe_stats_rate.bytes_kernel));
+			atomic64_get_na(&rtpe_stats_rate.bytes_user) +
+			atomic64_get_na(&rtpe_stats_rate.bytes_kernel));
 	METRIC("errorrate", "Errors per second (total)", UINT64F, UINT64F,
-			atomic64_get(&rtpe_stats_rate.errors_user) +
-			atomic64_get(&rtpe_stats_rate.errors_kernel));
+			atomic64_get_na(&rtpe_stats_rate.errors_user) +
+			atomic64_get_na(&rtpe_stats_rate.errors_kernel));
 
 	METRIC("media_userspace", "Userspace-only media streams", UINT64F, UINT64F,
-			atomic64_get(&rtpe_stats_gauge.userspace_streams));
+			atomic64_get_na(&rtpe_stats_gauge.userspace_streams));
 	PROM("mediastreams", "gauge");
 	PROMLAB("type=\"userspace\"");
 
 	METRIC("media_kernel", "Kernel-only media streams", UINT64F, UINT64F,
-			atomic64_get(&rtpe_stats_gauge.kernel_only_streams));
+			atomic64_get_na(&rtpe_stats_gauge.kernel_only_streams));
 	PROM("mediastreams", "gauge");
 	PROMLAB("type=\"kernel\"");
 
 	METRIC("media_mixed", "Mixed kernel/userspace media streams", UINT64F, UINT64F,
-			atomic64_get(&rtpe_stats_gauge.kernel_user_streams));
+			atomic64_get_na(&rtpe_stats_gauge.kernel_user_streams));
 	PROM("mediastreams", "gauge");
 	PROMLAB("type=\"mixed\"");
 
-	num_sessions = atomic64_get(&rtpe_stats.managed_sess);
-	uint64_t total_duration = atomic64_get(&rtpe_stats.call_duration);
+	num_sessions = atomic64_get_na(&rtpe_stats->managed_sess);
+	uint64_t total_duration = atomic64_get_na(&rtpe_stats->call_duration);
 	uint64_t avg_us = num_sessions ? total_duration / num_sessions : 0;
 
 	HEADER("}", "");
@@ -388,67 +388,67 @@ GQueue *statistics_gather_metrics(struct interface_sampled_rate_stats *interface
 
 	METRIC("managedsessions", "Total managed sessions", UINT64F, UINT64F, num_sessions);
 	PROM("sessions_total", "counter");
-	METRIC("rejectedsessions", "Total rejected sessions", UINT64F, UINT64F, atomic64_get(&rtpe_stats.rejected_sess));
+	METRIC("rejectedsessions", "Total rejected sessions", UINT64F, UINT64F, atomic64_get_na(&rtpe_stats->rejected_sess));
 	PROM("closed_sessions_total", "counter");
 	PROMLAB("reason=\"rejected\"");
-	METRIC("timeoutsessions", "Total timed-out sessions via TIMEOUT", UINT64F, UINT64F, atomic64_get(&rtpe_stats.timeout_sess));
+	METRIC("timeoutsessions", "Total timed-out sessions via TIMEOUT", UINT64F, UINT64F, atomic64_get_na(&rtpe_stats->timeout_sess));
 	PROM("closed_sessions_total", "counter");
 	PROMLAB("reason=\"timeout\"");
-	METRIC("silenttimeoutsessions", "Total timed-out sessions via SILENT_TIMEOUT", UINT64F, UINT64F,atomic64_get(&rtpe_stats.silent_timeout_sess));
+	METRIC("silenttimeoutsessions", "Total timed-out sessions via SILENT_TIMEOUT", UINT64F, UINT64F,atomic64_get_na(&rtpe_stats->silent_timeout_sess));
 	PROM("closed_sessions_total", "counter");
 	PROMLAB("reason=\"silent_timeout\"");
-	METRIC("finaltimeoutsessions", "Total timed-out sessions via FINAL_TIMEOUT", UINT64F, UINT64F,atomic64_get(&rtpe_stats.final_timeout_sess));
+	METRIC("finaltimeoutsessions", "Total timed-out sessions via FINAL_TIMEOUT", UINT64F, UINT64F,atomic64_get_na(&rtpe_stats->final_timeout_sess));
 	PROM("closed_sessions_total", "counter");
 	PROMLAB("reason=\"final_timeout\"");
-	METRIC("offertimeoutsessions", "Total timed-out sessions via OFFER_TIMEOUT", UINT64F, UINT64F,atomic64_get(&rtpe_stats.offer_timeout_sess));
+	METRIC("offertimeoutsessions", "Total timed-out sessions via OFFER_TIMEOUT", UINT64F, UINT64F,atomic64_get_na(&rtpe_stats->offer_timeout_sess));
 	PROM("closed_sessions_total", "counter");
 	PROMLAB("reason=\"offer_timeout\"");
-	METRIC("regularterminatedsessions", "Total regular terminated sessions", UINT64F, UINT64F, atomic64_get(&rtpe_stats.regular_term_sess));
+	METRIC("regularterminatedsessions", "Total regular terminated sessions", UINT64F, UINT64F, atomic64_get_na(&rtpe_stats->regular_term_sess));
 	PROM("closed_sessions_total", "counter");
 	PROMLAB("reason=\"terminated\"");
-	METRIC("forcedterminatedsessions", "Total forced terminated sessions", UINT64F, UINT64F, atomic64_get(&rtpe_stats.forced_term_sess));
+	METRIC("forcedterminatedsessions", "Total forced terminated sessions", UINT64F, UINT64F, atomic64_get_na(&rtpe_stats->forced_term_sess));
 	PROM("closed_sessions_total", "counter");
 	PROMLAB("reason=\"force_terminated\"");
 
 	METRIC("relayedpackets_user", "Total relayed packets (userspace)", UINT64F, UINT64F,
-			atomic64_get(&rtpe_stats.packets_user));
+			atomic64_get_na(&rtpe_stats->packets_user));
 	PROM("packets_total", "counter");
 	PROMLAB("type=\"userspace\"");
 	METRIC("relayedpacketerrors_user", "Total relayed packet errors (userspace)", UINT64F, UINT64F,
-			atomic64_get(&rtpe_stats.errors_user));
+			atomic64_get_na(&rtpe_stats->errors_user));
 	PROM("packet_errors_total", "counter");
 	PROMLAB("type=\"userspace\"");
 	METRIC("relayedbytes_user", "Total relayed bytes (userspace)", UINT64F, UINT64F,
-			atomic64_get(&rtpe_stats.bytes_user));
+			atomic64_get_na(&rtpe_stats->bytes_user));
 	PROM("bytes_total", "counter");
 	PROMLAB("type=\"userspace\"");
 
 	METRIC("relayedpackets_kernel", "Total relayed packets (kernel)", UINT64F, UINT64F,
-			atomic64_get(&rtpe_stats.packets_kernel));
+			atomic64_get_na(&rtpe_stats->packets_kernel));
 	PROM("packets_total", "counter");
 	PROMLAB("type=\"kernel\"");
 	METRIC("relayedpacketerrors_kernel", "Total relayed packet errors (kernel)", UINT64F, UINT64F,
-			atomic64_get(&rtpe_stats.errors_kernel));
+			atomic64_get_na(&rtpe_stats->errors_kernel));
 	PROM("packet_errors_total", "counter");
 	PROMLAB("type=\"kernel\"");
 	METRIC("relayedbytes_kernel", "Total relayed bytes (kernel)", UINT64F, UINT64F,
-			atomic64_get(&rtpe_stats.bytes_kernel));
+			atomic64_get_na(&rtpe_stats->bytes_kernel));
 	PROM("bytes_total", "counter");
 	PROMLAB("type=\"kernel\"");
 
 	METRIC("relayedpackets", "Total relayed packets", UINT64F, UINT64F,
-			atomic64_get(&rtpe_stats.packets_kernel) +
-			atomic64_get(&rtpe_stats.packets_user));
+			atomic64_get_na(&rtpe_stats->packets_kernel) +
+			atomic64_get_na(&rtpe_stats->packets_user));
 	METRIC("relayedpacketerrors", "Total relayed packet errors", UINT64F, UINT64F,
-			atomic64_get(&rtpe_stats.errors_kernel) +
-			atomic64_get(&rtpe_stats.errors_user));
+			atomic64_get_na(&rtpe_stats->errors_kernel) +
+			atomic64_get_na(&rtpe_stats->errors_user));
 	METRIC("relayedbytes", "Total relayed bytes", UINT64F, UINT64F,
-			atomic64_get(&rtpe_stats.bytes_kernel) +
-			atomic64_get(&rtpe_stats.bytes_user));
+			atomic64_get_na(&rtpe_stats->bytes_kernel) +
+			atomic64_get_na(&rtpe_stats->bytes_user));
 
-	METRIC("zerowaystreams", "Total number of streams with no relayed packets", UINT64F, UINT64F, atomic64_get(&rtpe_stats.nopacket_relayed_sess));
+	METRIC("zerowaystreams", "Total number of streams with no relayed packets", UINT64F, UINT64F, atomic64_get_na(&rtpe_stats->nopacket_relayed_sess));
 	PROM("zero_packet_streams_total", "counter");
-	METRIC("onewaystreams", "Total number of 1-way streams", UINT64F, UINT64F,atomic64_get(&rtpe_stats.oneway_stream_sess));
+	METRIC("onewaystreams", "Total number of 1-way streams", UINT64F, UINT64F,atomic64_get_na(&rtpe_stats->oneway_stream_sess));
 	PROM("one_way_sessions_total", "counter");
 	METRICva("avgcallduration", "Average call duration", "%.6f", "%.6f seconds", (double) avg_us / 1000000.0);
 	PROM("call_duration_avg", "gauge");
@@ -456,7 +456,7 @@ GQueue *statistics_gather_metrics(struct interface_sampled_rate_stats *interface
 	METRICva("totalcallsduration", "Total calls duration", "%.6f", "%.6f seconds", (double) total_duration / 1000000.0);
 	PROM("call_duration_total", "counter");
 
-	total_duration = atomic64_get(&rtpe_stats.call_duration2);
+	total_duration = atomic64_get_na(&rtpe_stats->call_duration2);
 	METRICva("totalcallsduration2", "Total calls duration squared", "%.6f", "%.6f seconds squared", (double) total_duration / 1000000.0);
 	PROM("call_duration2_total", "counter");
 
@@ -468,8 +468,8 @@ GQueue *statistics_gather_metrics(struct interface_sampled_rate_stats *interface
 
 	if (graphite_is_enabled()) {
 		calls_dur_iv = (double) atomic64_get_na(&rtpe_stats_graphite_diff.total_calls_duration_intv) / 1000000.0;
-		min_sess_iv = atomic64_get(&rtpe_gauge_graphite_min_max_sampled.min.total_sessions);
-		max_sess_iv = atomic64_get(&rtpe_gauge_graphite_min_max_sampled.max.total_sessions);
+		min_sess_iv = atomic64_get_na(&rtpe_gauge_graphite_min_max_sampled.min.total_sessions);
+		max_sess_iv = atomic64_get_na(&rtpe_gauge_graphite_min_max_sampled.max.total_sessions);
 
 		HEADER("intervalstatistics", "Graphite interval statistics (last reported values to graphite):");
 		HEADER("{", NULL);
@@ -481,13 +481,13 @@ GQueue *statistics_gather_metrics(struct interface_sampled_rate_stats *interface
 		METRIC("maxmanagedsessions", "Max managed sessions", UINT64F, UINT64F, max_sess_iv);
 
 		for (int i = 0; i < NGC_COUNT; i++) {
-			double min = (double) atomic64_get(&rtpe_sampled_graphite_min_max_sampled.min.ng_command_times[i]) / 1000000.0;
-			double max = (double) atomic64_get(&rtpe_sampled_graphite_min_max_sampled.max.ng_command_times[i]) / 1000000.0;
-			double avg = (double) atomic64_get(&rtpe_sampled_graphite_avg.avg.ng_command_times[i]) / 1000000.0;
-			AUTO_CLEANUP(char *min_label, free_gbuf) = g_strdup_printf("min%sdelay", ng_command_strings[i]);
-			AUTO_CLEANUP(char *max_label, free_gbuf) = g_strdup_printf("max%sdelay", ng_command_strings[i]);
-			AUTO_CLEANUP(char *avg_label, free_gbuf) = g_strdup_printf("avg%sdelay", ng_command_strings[i]);
-			AUTO_CLEANUP(char *long_label, free_gbuf) = g_strdup_printf("Min/Max/Avg %s processing delay", ng_command_strings[i]);
+			double min = (double) atomic64_get_na(&rtpe_sampled_graphite_min_max_sampled.min.ng_command_times[i]) / 1000000.0;
+			double max = (double) atomic64_get_na(&rtpe_sampled_graphite_min_max_sampled.max.ng_command_times[i]) / 1000000.0;
+			double avg = (double) atomic64_get_na(&rtpe_sampled_graphite_avg.avg.ng_command_times[i]) / 1000000.0;
+			g_autoptr(char) min_label = g_strdup_printf("min%sdelay", ng_command_strings[i]);
+			g_autoptr(char) max_label = g_strdup_printf("max%sdelay", ng_command_strings[i]);
+			g_autoptr(char) avg_label = g_strdup_printf("avg%sdelay", ng_command_strings[i]);
+			g_autoptr(char) long_label = g_strdup_printf("Min/Max/Avg %s processing delay", ng_command_strings[i]);
 			METRICl(long_label, "%.6f/%.6f/%.6f sec", min, max, avg);
 			METRICsva(min_label, "%.6f", min);
 			METRICsva(max_label, "%.6f", max);
@@ -495,13 +495,13 @@ GQueue *statistics_gather_metrics(struct interface_sampled_rate_stats *interface
 		}
 
 		for (int i = 0; i < NGC_COUNT; i++) {
-			uint64_t min = atomic64_get(&rtpe_rate_graphite_min_max_avg_sampled.min.ng_commands[i]);
-			uint64_t max = atomic64_get(&rtpe_rate_graphite_min_max_avg_sampled.max.ng_commands[i]);
-			uint64_t avg = atomic64_get(&rtpe_rate_graphite_min_max_avg_sampled.avg.ng_commands[i]);
-			AUTO_CLEANUP(char *min_label, free_gbuf) = g_strdup_printf("min%srequestrate", ng_command_strings[i]);
-			AUTO_CLEANUP(char *max_label, free_gbuf) = g_strdup_printf("max%srequestrate", ng_command_strings[i]);
-			AUTO_CLEANUP(char *avg_label, free_gbuf) = g_strdup_printf("avg%srequestrate", ng_command_strings[i]);
-			AUTO_CLEANUP(char *long_label, free_gbuf) = g_strdup_printf("Min/Max/Avg %s requests per second", ng_command_strings[i]);
+			uint64_t min = atomic64_get_na(&rtpe_rate_graphite_min_max_avg_sampled.min.ng_commands[i]);
+			uint64_t max = atomic64_get_na(&rtpe_rate_graphite_min_max_avg_sampled.max.ng_commands[i]);
+			uint64_t avg = atomic64_get_na(&rtpe_rate_graphite_min_max_avg_sampled.avg.ng_commands[i]);
+			g_autoptr(char) min_label = g_strdup_printf("min%srequestrate", ng_command_strings[i]);
+			g_autoptr(char) max_label = g_strdup_printf("max%srequestrate", ng_command_strings[i]);
+			g_autoptr(char) avg_label = g_strdup_printf("avg%srequestrate", ng_command_strings[i]);
+			g_autoptr(char) long_label = g_strdup_printf("Min/Max/Avg %s requests per second", ng_command_strings[i]);
 			METRICl(long_label, "%" PRIu64 "/%" PRIu64 "/%" PRIu64 " per sec", min, max, avg);
 			METRICsva(min_label, "%" PRIu64 "", min);
 			METRICsva(max_label, "%" PRIu64 "", max);
@@ -517,21 +517,21 @@ GQueue *statistics_gather_metrics(struct interface_sampled_rate_stats *interface
 
 #define STAT_GET_PRINT_GEN(source, avgs, stat_name, name, divisor, prefix, label...) \
 	METRIC(#stat_name "_total", "Sum of all " name " values sampled", "%.6f", "%.6f", \
-			(double) atomic64_get(&(source)->sums.stat_name) / (divisor)); \
+			(double) atomic64_get_na(&(source)->sums.stat_name) / (divisor)); \
 	PROM(prefix #stat_name "_total", "counter"); \
 	PROMLAB(label); \
 	METRIC(#stat_name "2_total", "Sum of all " name " square values sampled", "%.6f", "%.6f", \
-			(double) atomic64_get(&(source)->sums_squared.stat_name) / (divisor * divisor)); \
+			(double) atomic64_get_na(&(source)->sums_squared.stat_name) / (divisor * divisor)); \
 	PROM(prefix #stat_name "2_total", "counter"); \
 	PROMLAB(label); \
 	METRIC(#stat_name "_samples_total", "Total number of " name " samples", UINT64F, UINT64F, \
-			atomic64_get(&(source)->counts.stat_name)); \
+			atomic64_get_na(&(source)->counts.stat_name)); \
 	PROM(prefix #stat_name "_samples_total", "counter"); \
 	PROMLAB(label); \
 	METRIC(#stat_name "_average", "Average " name, "%.6f", "%.6f", \
-			(double) atomic64_get(&(avgs)->avg.stat_name) / (divisor)); \
+			(double) atomic64_get_na(&(avgs)->avg.stat_name) / (divisor)); \
 	METRIC(#stat_name "_stddev", name " standard deviation", "%.6f", "%.6f", \
-			(double) atomic64_get(&(avgs)->stddev.stat_name) / (divisor * divisor));
+			(double) atomic64_get_na(&(avgs)->stddev.stat_name) / (divisor * divisor));
 
 #define STAT_GET_PRINT(stat_name, name, divisor) \
 	STAT_GET_PRINT_GEN(&rtpe_stats_sampled, &sampled_avgs, stat_name, name, divisor, "", NULL)
@@ -550,19 +550,19 @@ GQueue *statistics_gather_metrics(struct interface_sampled_rate_stats *interface
 	STAT_GET_PRINT(packetloss, "packet loss", 1.0);
 	STAT_GET_PRINT(jitter_measured, "jitter (measured)", 1.0);
 	METRIC("packets_lost", "Packets lost", UINT64F, UINT64F,
-			atomic64_get(&rtpe_stats.packets_lost));
+			atomic64_get_na(&rtpe_stats->packets_lost));
 	PROM("packets_lost", "counter");
 	METRIC("rtp_duplicates", "Duplicate RTP packets", UINT64F, UINT64F,
-			atomic64_get(&rtpe_stats.rtp_duplicates));
+			atomic64_get_na(&rtpe_stats->rtp_duplicates));
 	PROM("rtp_duplicates", "counter");
 	METRIC("rtp_skips", "RTP sequence skips", UINT64F, UINT64F,
-			atomic64_get(&rtpe_stats.rtp_skips));
+			atomic64_get_na(&rtpe_stats->rtp_skips));
 	PROM("rtp_skips", "counter");
 	METRIC("rtp_seq_resets", "RTP sequence resets", UINT64F, UINT64F,
-			atomic64_get(&rtpe_stats.rtp_seq_resets));
+			atomic64_get_na(&rtpe_stats->rtp_seq_resets));
 	PROM("rtp_seq_resets", "counter");
 	METRIC("rtp_reordered", "Out-of-order RTP packets", UINT64F, UINT64F,
-			atomic64_get(&rtpe_stats.rtp_reordered));
+			atomic64_get_na(&rtpe_stats->rtp_reordered));
 	PROM("rtp_reordered", "counter");
 	HEADER(NULL, "");
 	HEADER("}", "");
@@ -685,7 +685,7 @@ GQueue *statistics_gather_metrics(struct interface_sampled_rate_stats *interface
 		HEADER("}", NULL);
 
 #define F(f) \
-		METRICs(#f, UINT64F, atomic64_get(&lif->stats.s.f)); \
+		METRICs(#f, UINT64F, atomic64_get_na(&lif->stats->s.f)); \
 		PROM("interface_" #f, "counter"); \
 		PROMLAB("name=\"%s\",address=\"%s\"", lif->logical->name.s, \
 				sockaddr_print_buf(&lif->spec->local_address.addr));
@@ -702,9 +702,9 @@ GQueue *statistics_gather_metrics(struct interface_sampled_rate_stats *interface
 			HEADER("{", NULL);
 
 			struct interface_counter_stats diff;
-			interface_counter_calc_diff(&lif->stats.s, &intv_stats->s, &diff);
+			interface_counter_calc_diff(&lif->stats->s, &intv_stats->s, &diff);
 
-#define F(f) METRICs(#f, UINT64F, atomic64_get(&diff.f));
+#define F(f) METRICs(#f, UINT64F, atomic64_get_na(&diff.f));
 #include "interface_counter_stats_fields.inc"
 #undef F
 
@@ -717,7 +717,7 @@ GQueue *statistics_gather_metrics(struct interface_sampled_rate_stats *interface
 				struct interface_counter_stats rate;
 				interface_counter_calc_rate_from_diff(time_diff_us, &diff, &rate);
 
-#define F(f) METRICs(#f, UINT64F, atomic64_get(&rate.f));
+#define F(f) METRICs(#f, UINT64F, atomic64_get_na(&rate.f));
 #include "interface_counter_stats_fields.inc"
 #undef F
 
@@ -729,10 +729,10 @@ GQueue *statistics_gather_metrics(struct interface_sampled_rate_stats *interface
 		HEADER("{", NULL);
 
 		struct interface_sampled_stats_avg stat_avg;
-		interface_sampled_avg(&stat_avg, &lif->stats.sampled);
+		interface_sampled_avg(&stat_avg, &lif->stats->sampled);
 
 #define INTF_SAMPLED_STAT(stat_name, name, divisor, prefix, label...) \
-	STAT_GET_PRINT_GEN(&lif->stats.sampled, &stat_avg, stat_name, name, divisor, prefix, label)
+	STAT_GET_PRINT_GEN(&lif->stats->sampled, &stat_avg, stat_name, name, divisor, prefix, label)
 
 		INTF_SAMPLED_STAT(mos, "MOS", 10.0, "interface_",
 				"name=\"%s\",address=\"%s\"", lif->logical->name.s,
@@ -761,34 +761,34 @@ GQueue *statistics_gather_metrics(struct interface_sampled_rate_stats *interface
 			HEADER("{", NULL);
 
 			struct interface_sampled_stats diff;
-			interface_sampled_calc_diff(&lif->stats.sampled, &intv_stats->sampled, &diff);
+			interface_sampled_calc_diff(&lif->stats->sampled, &intv_stats->sampled, &diff);
 			struct interface_sampled_stats_avg avg;
 			interface_sampled_avg(&avg, &diff);
 
 			METRIC("mos", "Average interval MOS", "%.6f", "%.6f", \
-					(double) atomic64_get(&avg.avg.mos) / 10.0); \
+					(double) atomic64_get_na(&avg.avg.mos) / 10.0); \
 			METRIC("mos_stddev", "Standard deviation interval MOS", "%.6f", "%.6f", \
-					(double) atomic64_get(&avg.stddev.mos) / 100.0); \
+					(double) atomic64_get_na(&avg.stddev.mos) / 100.0); \
 			METRIC("jitter", "Average interval jitter (reported)", "%.6f", "%.6f", \
-					(double) atomic64_get(&avg.avg.jitter)); \
+					(double) atomic64_get_na(&avg.avg.jitter)); \
 			METRIC("jitter_stddev", "Standard deviation interval jitter (reported)", "%.6f", "%.6f", \
-					(double) atomic64_get(&avg.stddev.jitter)); \
+					(double) atomic64_get_na(&avg.stddev.jitter)); \
 			METRIC("rtt_e2e", "Average interval end-to-end round-trip time", "%.6f", "%.6f", \
-					(double) atomic64_get(&avg.avg.rtt_e2e)); \
+					(double) atomic64_get_na(&avg.avg.rtt_e2e)); \
 			METRIC("rtt_e2e_stddev", "Standard deviation interval end-to-end round-trip time", "%.6f", "%.6f", \
-					(double) atomic64_get(&avg.stddev.rtt_e2e)); \
+					(double) atomic64_get_na(&avg.stddev.rtt_e2e)); \
 			METRIC("rtt_dsct", "Average interval discrete round-trip time", "%.6f", "%.6f", \
-					(double) atomic64_get(&avg.avg.rtt_dsct)); \
+					(double) atomic64_get_na(&avg.avg.rtt_dsct)); \
 			METRIC("rtt_dsct_stddev", "Standard deviation interval discrete round-trip time", "%.6f", "%.6f", \
-					(double) atomic64_get(&avg.stddev.rtt_dsct)); \
+					(double) atomic64_get_na(&avg.stddev.rtt_dsct)); \
 			METRIC("packetloss", "Average interval packet loss", "%.6f", "%.6f", \
-					(double) atomic64_get(&avg.avg.packetloss)); \
+					(double) atomic64_get_na(&avg.avg.packetloss)); \
 			METRIC("packetloss_stddev", "Standard deviation interval packet loss", "%.6f", "%.6f", \
-					(double) atomic64_get(&avg.stddev.packetloss)); \
+					(double) atomic64_get_na(&avg.stddev.packetloss)); \
 			METRIC("jitter_measured", "Average interval jitter (measured)", "%.6f", "%.6f", \
-					(double) atomic64_get(&avg.avg.jitter_measured)); \
+					(double) atomic64_get_na(&avg.avg.jitter_measured)); \
 			METRIC("jitter_measured_stddev", "Standard deviation interval jitter (measured)", "%.6f", "%.6f", \
-					(double) atomic64_get(&avg.stddev.jitter_measured)); \
+					(double) atomic64_get_na(&avg.stddev.jitter_measured)); \
 
 			HEADER("}", NULL);
 		}
@@ -796,7 +796,7 @@ GQueue *statistics_gather_metrics(struct interface_sampled_rate_stats *interface
 		HEADER("ingress", NULL);
 		HEADER("{", NULL);
 #define F(f) \
-		METRICs(#f, UINT64F, atomic64_get(&lif->stats.in.f)); \
+		METRICs(#f, UINT64F, atomic64_get_na(&lif->stats->in.f)); \
 		PROM("interface_" #f, "gauge"); \
 		PROMLAB("name=\"%s\",address=\"%s\",direction=\"ingress\"", lif->logical->name.s, \
 				sockaddr_print_buf(&lif->spec->local_address.addr));
@@ -807,7 +807,7 @@ GQueue *statistics_gather_metrics(struct interface_sampled_rate_stats *interface
 		HEADER("egress", NULL);
 		HEADER("{", NULL);
 #define F(f) \
-		METRICs(#f, UINT64F, atomic64_get(&lif->stats.out.f)); \
+		METRICs(#f, UINT64F, atomic64_get_na(&lif->stats->out.f)); \
 		PROM("interface_" #f, "gauge"); \
 		PROMLAB("name=\"%s\",address=\"%s\",direction=\"egress\"", lif->logical->name.s, \
 				sockaddr_print_buf(&lif->spec->local_address.addr));
@@ -820,9 +820,9 @@ GQueue *statistics_gather_metrics(struct interface_sampled_rate_stats *interface
 			HEADER("{", NULL);
 
 			struct interface_counter_stats_dir diff_in;
-			interface_counter_calc_diff_dir(&lif->stats.in, &intv_stats->in, &diff_in);
+			interface_counter_calc_diff_dir(&lif->stats->in, &intv_stats->in, &diff_in);
 
-#define F(f) METRICs(#f, UINT64F, atomic64_get(&diff_in.f));
+#define F(f) METRICs(#f, UINT64F, atomic64_get_na(&diff_in.f));
 #include "interface_counter_stats_fields_dir.inc"
 #undef F
 
@@ -832,9 +832,9 @@ GQueue *statistics_gather_metrics(struct interface_sampled_rate_stats *interface
 			HEADER("{", NULL);
 
 			struct interface_counter_stats_dir diff_out;
-			interface_counter_calc_diff_dir(&lif->stats.out, &intv_stats->out, &diff_out);
+			interface_counter_calc_diff_dir(&lif->stats->out, &intv_stats->out, &diff_out);
 
-#define F(f) METRICs(#f, UINT64F, atomic64_get(&diff_out.f));
+#define F(f) METRICs(#f, UINT64F, atomic64_get_na(&diff_out.f));
 #include "interface_counter_stats_fields_dir.inc"
 #undef F
 
@@ -848,7 +848,7 @@ GQueue *statistics_gather_metrics(struct interface_sampled_rate_stats *interface
 				interface_counter_calc_rate_from_diff_dir(time_diff_us, &diff_in,
 						&rate);
 
-#define F(f) METRICs(#f, UINT64F, atomic64_get(&rate.f));
+#define F(f) METRICs(#f, UINT64F, atomic64_get_na(&rate.f));
 #include "interface_counter_stats_fields_dir.inc"
 #undef F
 
@@ -860,7 +860,7 @@ GQueue *statistics_gather_metrics(struct interface_sampled_rate_stats *interface
 				interface_counter_calc_rate_from_diff_dir(time_diff_us, &diff_out,
 						&rate);
 
-#define F(f) METRICs(#f, UINT64F, atomic64_get(&rate.f));
+#define F(f) METRICs(#f, UINT64F, atomic64_get_na(&rate.f));
 #include "interface_counter_stats_fields_dir.inc"
 #undef F
 
@@ -875,13 +875,15 @@ GQueue *statistics_gather_metrics(struct interface_sampled_rate_stats *interface
 	mutex_lock(&rtpe_codec_stats_lock);
 	HEADER("transcoders", NULL);
 	HEADER("[", "");
-	GList *chains = g_hash_table_get_keys(rtpe_codec_stats);
 
 	int last_tv_sec = rtpe_now.tv_sec - 1;
 	unsigned int idx = last_tv_sec & 1;
-	for (GList *l = chains; l; l = l->next) {
-		char *chain = l->data;
-		struct codec_stats *stats_entry = g_hash_table_lookup(rtpe_codec_stats, chain);
+
+	codec_stats_ht_iter iter;
+	t_hash_table_iter_init(&iter, rtpe_codec_stats);
+	char *chain;
+	struct codec_stats *stats_entry;
+	while (t_hash_table_iter_next(&iter, &chain, &stats_entry)) {
 		HEADER("{", "");
 		METRICsva("chain", "\"%s\"", chain);
 		METRICs("num", "%i", g_atomic_int_get(&stats_entry->num_transcoders));
@@ -905,7 +907,6 @@ GQueue *statistics_gather_metrics(struct interface_sampled_rate_stats *interface
 	}
 
 	mutex_unlock(&rtpe_codec_stats_lock);
-	g_list_free(chains);
 	HEADER("]", "");
 
 	HEADER("}", NULL);
@@ -914,8 +915,7 @@ GQueue *statistics_gather_metrics(struct interface_sampled_rate_stats *interface
 }
 #pragma GCC diagnostic warning "-Wformat-zero-length"
 
-static void free_stats_metric(void *p) {
-	struct stats_metric *m = p;
+static void free_stats_metric(stats_metric *m) {
 	g_free(m->descr);
 	g_free(m->label);
 	g_free(m->value_long);
@@ -925,42 +925,43 @@ static void free_stats_metric(void *p) {
 	g_slice_free1(sizeof(*m), m);
 }
 
-void statistics_free_metrics(GQueue **q) {
-	g_queue_free_full(*q, free_stats_metric);
-	*q = NULL;
+void statistics_free_metrics(stats_metric_q *q) {
+	t_queue_free_full(q, free_stats_metric);
 }
 
-void statistics_free() {
+void statistics_free(void) {
 	mutex_destroy(&rtpe_codec_stats_lock);
-	g_hash_table_destroy(rtpe_codec_stats);
+	t_hash_table_destroy(rtpe_codec_stats);
+	bufferpool_unref(rtpe_stats);
+	rtpe_stats = NULL;
 }
 
-static void codec_stats_free(void *p) {
-	struct codec_stats *stats_entry = p;
+static void codec_stats_free(struct codec_stats *stats_entry) {
 	free(stats_entry->chain);
 	g_free(stats_entry->chain_brief);
 	g_slice_free1(sizeof(*stats_entry), stats_entry);
 }
 
-void statistics_init() {
+TYPED_GHASHTABLE_IMPL(codec_stats_ht, c_str_hash, c_str_equal, NULL, codec_stats_free)
+
+void statistics_init(void) {
 	gettimeofday(&rtpe_started, NULL);
-	//rtpe_totalstats_interval.managed_sess_min = 0; // already zeroed
-	//rtpe_totalstats_interval.managed_sess_max = 0;
+	rtpe_stats = bufferpool_alloc0(shm_bufferpool, sizeof(*rtpe_stats));
 
 	mutex_init(&rtpe_codec_stats_lock);
-	rtpe_codec_stats = g_hash_table_new_full(g_str_hash, g_str_equal, NULL, codec_stats_free);
+	rtpe_codec_stats = codec_stats_ht_new();
 }
 
 const char *statistics_ng(bencode_item_t *input, bencode_item_t *output) {
-	AUTO_CLEANUP_INIT(GQueue *metrics, statistics_free_metrics, statistics_gather_metrics(NULL));
-	AUTO_CLEANUP_INIT(GQueue bstack, g_queue_clear, G_QUEUE_INIT);
+	g_autoptr(stats_metric_q) metrics = statistics_gather_metrics(NULL);
+	g_auto(GQueue) bstack = G_QUEUE_INIT;
 
 	bencode_item_t *dict = output;
 	const char *sub_label = "statistics"; // top level
 	bencode_buffer_t *buf = output->buffer;
 
-	for (GList *l = metrics->head; l; l = l->next) {
-		struct stats_metric *m = l->data;
+	for (__auto_type l = metrics->head; l; l = l->next) {
+		stats_metric *m = l->data;
 		if (!m->label)
 			continue;
 
@@ -1022,14 +1023,14 @@ const char *statistics_ng(bencode_item_t *input, bencode_item_t *output) {
 /**
  * Separate thread for update of running min/max call counters.
  */
-enum thread_looper_action call_rate_stats_updater() {
+enum thread_looper_action call_rate_stats_updater(void) {
 	static struct timeval last_run;
 
 	stats_rate_min_max(&rtpe_rate_graphite_min_max, &rtpe_stats_rate);
 
-	if (!last_run.tv_sec) { /* `stats_counters_calc_rate()` shouldn't be called on the very first cycle */
+	if (last_run.tv_sec) { /* `stats_counters_calc_rate()` shouldn't be called on the very first cycle */
 		long long run_diff_us = timeval_diff(&rtpe_now, &last_run);
-		stats_counters_calc_rate(&rtpe_stats, run_diff_us, &rtpe_stats_intv, &rtpe_stats_rate);
+		stats_counters_calc_rate(rtpe_stats, run_diff_us, &rtpe_stats_intv, &rtpe_stats_rate);
 	}
 
 	last_run = rtpe_now;
