@@ -14,6 +14,7 @@
 #include "obj.h"
 #include "socket.h"
 #include "log_funcs.h"
+#include "uring.h"
 
 struct udp_listener_callback {
 	struct obj obj;
@@ -22,11 +23,12 @@ struct udp_listener_callback {
 	struct obj *p;
 };
 
-static void udp_listener_closed(int fd, void *p, uintptr_t x) {
-	abort();
+static void udp_listener_closed(int fd, void *p) {
+	if (!rtpe_shutdown)
+		abort();
 }
 
-static void udp_listener_incoming(int fd, void *p, uintptr_t x) {
+static void udp_listener_incoming(int fd, void *p) {
 	struct udp_listener_callback *cb = p;
 	int len;
 	struct udp_buffer *udp_buf = NULL;
@@ -95,7 +97,7 @@ int udp_listener_init(socket_t *sock, const endpoint_t *ep,
 	i.closed = udp_listener_closed;
 	i.readable = udp_listener_incoming;
 	i.obj = &cb->obj;
-	if (poller_add_item(rtpe_poller, &i))
+	if (!rtpe_poller_add_item(rtpe_control_poller, &i))
 		goto fail;
 
 	obj_put(cb);
