@@ -1,7 +1,9 @@
 #include "websocket.h"
+
 #include <libwebsockets.h>
 #include <assert.h>
 #include <json-glib/json-glib.h>
+
 #include "log.h"
 #include "main.h"
 #include "str.h"
@@ -24,6 +26,12 @@ struct websocket_output {
 	ssize_t content_length;
 };
 
+TYPED_DIRECT_FUNCS(janus_session_hash, janus_session_eq, struct janus_session)
+TYPED_GHASHTABLE(janus_sessions_ht, struct janus_session, struct janus_session,
+		janus_session_hash, janus_session_eq, NULL, NULL)
+TYPED_GQUEUE(websocket_message, struct websocket_message)
+TYPED_GQUEUE(websocket_output, struct websocket_output)
+
 struct websocket_conn {
 	// used in the single threaded libwebsockets context
 	struct lws *wsi;
@@ -34,12 +42,12 @@ struct websocket_conn {
 	// multithreaded message processing
 	mutex_t lock;
 	unsigned int jobs;
-	GQueue messages;
+	websocket_message_q messages;
 	cond_t cond;
-	GHashTable *janus_sessions;
+	janus_sessions_ht janus_sessions;
 
 	// output buffer - also protected by lock
-	GQueue output_q;
+	websocket_output_q output_q;
 };
 
 struct websocket_ng_buf {
@@ -82,8 +90,7 @@ static struct websocket_output *websocket_output_new(void) {
 	return wo;
 }
 
-static void websocket_output_free(void *p) {
-	struct websocket_output *wo = p;
+static void websocket_output_free(struct websocket_output *wo) {
 	if (wo->str)
 		g_string_free(wo->str, TRUE);
 	g_slice_free1(sizeof(*wo), wo);
@@ -92,7 +99,7 @@ static void websocket_output_free(void *p) {
 
 // appends to output buffer without triggering a response - unlocked
 static void __websocket_queue_raw(struct websocket_conn *wc, const char *msg, size_t len) {
-	struct websocket_output *wo = g_queue_peek_tail(&wc->output_q);
+	struct websocket_output *wo = t_queue_peek_tail(&wc->output_q);
 
 	if (!wo->str) {
 		wo->str = g_string_new("");
@@ -118,7 +125,7 @@ size_t websocket_queue_len(struct websocket_conn *wc) {
 	LOCK(&wc->lock);
 
 	size_t ret = 0;
-	for (GList *l = wc->output_q.head; l; l = l->next) {
+	for (__auto_type l = wc->output_q.head; l; l = l->next) {
 		struct websocket_output *wo = l->data;
 		ret += (wo->str->len - LWS_PRE);
 	}
@@ -133,9 +140,9 @@ void websocket_write_raw(struct websocket_conn *wc, const char *msg, size_t len,
 {
 	mutex_lock(&wc->lock);
 	__websocket_queue_raw(wc, msg, len);
-	struct websocket_output *wo = g_queue_peek_tail(&wc->output_q);
+	struct websocket_output *wo = t_queue_peek_tail(&wc->output_q);
 	wo->protocol = protocol;
-	g_queue_push_tail(&wc->output_q, websocket_output_new());
+	t_queue_push_tail(&wc->output_q, websocket_output_new());
 
 	mutex_unlock(&wc->lock);
 
@@ -193,7 +200,7 @@ void websocket_write_binary(struct websocket_conn *wc, const char *msg, size_t l
 
 void websocket_write_next(struct websocket_conn *wc) {
 	LOCK(&wc->lock);
-	g_queue_push_tail(&wc->output_q, websocket_output_new());
+	t_queue_push_tail(&wc->output_q, websocket_output_new());
 }
 
 
@@ -214,7 +221,7 @@ static void websocket_message_push(struct websocket_conn *wc, websocket_message_
 	assert(wm != NULL);
 	wm->func = func;
 
-	g_queue_push_tail(&wc->messages, wm);
+	t_queue_push_tail(&wc->messages, wm);
 	wc->jobs++;
 	g_thread_pool_push(websocket_threads, wc, NULL);
 
@@ -226,7 +233,7 @@ static void websocket_process(void *p, void *up) {
 	struct websocket_conn *wc = p;
 
 	mutex_lock(&wc->lock);
-	struct websocket_message *wm = g_queue_pop_head(&wc->messages);
+	struct websocket_message *wm = t_queue_pop_head(&wc->messages);
 	mutex_unlock(&wc->lock);
 
 	assert(wm != NULL);
@@ -301,7 +308,7 @@ static int websocket_dequeue(struct websocket_conn *wc) {
 	mutex_lock(&wc->lock);
 	struct websocket_output *wo;
 	struct lws *wsi = wc->wsi;
-	while ((wo = g_queue_pop_head(&wc->output_q))) {
+	while ((wo = t_queue_pop_head(&wc->output_q))) {
 		// used buffer slot?
 		if (!wo->str)
 			goto next;
@@ -339,7 +346,7 @@ static int websocket_dequeue(struct websocket_conn *wc) {
 next:
 		websocket_output_free(wo);
 	}
-	g_queue_push_tail(&wc->output_q, websocket_output_new());
+	t_queue_push_tail(&wc->output_q, websocket_output_new());
 
 	mutex_unlock(&wc->lock);
 
@@ -356,7 +363,7 @@ void websocket_http_response(struct websocket_conn *wc, int status, const char *
 {
 	LOCK(&wc->lock);
 
-	struct websocket_output *wo = g_queue_peek_tail(&wc->output_q);
+	struct websocket_output *wo = t_queue_peek_tail(&wc->output_q);
 
 	wo->http_status = status;
 	wo->content_type = content_type;
@@ -377,16 +384,17 @@ static const char *websocket_http_ping(struct websocket_message *wm) {
 }
 
 
+TYPED_GHASHTABLE(metric_types_ht, char, void, c_str_hash, c_str_equal, NULL, NULL)
+
 static const char *websocket_http_metrics(struct websocket_message *wm) {
 	ilogs(http, LOG_DEBUG, "Respoding to GET /metrics");
 
-	AUTO_CLEANUP_INIT(GQueue *metrics, statistics_free_metrics, statistics_gather_metrics(NULL));
-	AUTO_CLEANUP_INIT(GString *outp, __g_string_free, g_string_new(""));
-	AUTO_CLEANUP_INIT(GHashTable *metric_types, __g_hash_table_destroy,
-			g_hash_table_new(g_str_hash, g_str_equal));
+	g_autoptr(stats_metric_q) metrics = statistics_gather_metrics(NULL);
+	g_autoptr(GString) outp = g_string_new("");
+	g_auto(metric_types_ht) metric_types = metric_types_ht_new();
 
-	for (GList *l = metrics->head; l; l = l->next) {
-		struct stats_metric *m = l->data;
+	for (__auto_type l = metrics->head; l; l = l->next) {
+		stats_metric *m = l->data;
 		if (!m->label)
 			continue;
 		if (!m->value_short)
@@ -394,14 +402,14 @@ static const char *websocket_http_metrics(struct websocket_message *wm) {
 		if (!m->prom_name)
 			continue;
 
-		if (!g_hash_table_lookup(metric_types, m->prom_name)) {
+		if (!t_hash_table_lookup(metric_types, m->prom_name)) {
 			if (m->descr)
 				g_string_append_printf(outp, "# HELP rtpengine_%s %s\n",
 						m->prom_name, m->descr);
 			if (m->prom_type)
 				g_string_append_printf(outp, "# TYPE rtpengine_%s %s\n",
 						m->prom_name, m->prom_type);
-			g_hash_table_insert(metric_types, (void *) m->prom_name, (void *) 0x1);
+			t_hash_table_insert(metric_types, (void *) m->prom_name, (void *) 0x1);
 		}
 
 		g_string_append_printf(outp, "rtpengine_%s", m->prom_name);
@@ -467,8 +475,10 @@ static void websocket_ng_send_ws(str *cookie, str *body, const endpoint_t *sin,
 		void *p1)
 {
 	struct websocket_conn *wc = p1;
-	websocket_queue_raw(wc, cookie->s, cookie->len);
-	websocket_queue_raw(wc, " ", 1);
+	if (cookie) {
+		websocket_queue_raw(wc, cookie->s, cookie->len);
+		websocket_queue_raw(wc, " ", 1);
+	}
 	websocket_queue_raw(wc, body->s, body->len);
 	websocket_write_binary(wc, NULL, 0, true);
 }
@@ -476,9 +486,12 @@ static void websocket_ng_send_http(str *cookie, str *body, const endpoint_t *sin
 		void *p1)
 {
 	struct websocket_conn *wc = p1;
-	websocket_http_response(wc, 200, "application/x-rtpengine-ng", cookie->len + 1 + body->len);
-	websocket_queue_raw(wc, cookie->s, cookie->len);
-	websocket_queue_raw(wc, " ", 1);
+	websocket_http_response(wc, 200, "application/x-rtpengine-ng",
+			(cookie ? (cookie->len + 1) : 0) + body->len);
+	if (cookie) {
+		websocket_queue_raw(wc, cookie->s, cookie->len);
+		websocket_queue_raw(wc, " ", 1);
+	}
 	websocket_queue_raw(wc, body->s, body->len);
 	websocket_write_http(wc, NULL, true);
 }
@@ -488,7 +501,9 @@ static void __ng_buf_free(void *p) {
 	g_string_free(buf->body, TRUE);
 }
 
-static const char *websocket_ng_process(struct websocket_message *wm) {
+static const char *websocket_ng_process_generic(struct websocket_message *wm,
+		__typeof__(control_ng_process) cb)
+{
 	struct websocket_ng_buf *buf = obj_alloc0("websocket_ng_buf", sizeof(*buf), __ng_buf_free);
 
 	endpoint_print(&wm->wc->endpoint, buf->addr, sizeof(buf->addr));
@@ -501,13 +516,21 @@ static const char *websocket_ng_process(struct websocket_message *wm) {
 	str_init_len(&buf->cmd, buf->body->str, buf->body->len);
 	buf->endpoint = wm->wc->endpoint;
 
-	control_ng_process(&buf->cmd, &buf->endpoint, buf->addr, NULL, websocket_ng_send_ws, wm->wc, &buf->obj);
+	cb(&buf->cmd, &buf->endpoint, buf->addr, NULL, websocket_ng_send_ws, wm->wc, &buf->obj);
 
 	obj_put(buf);
 
 	return NULL;
 }
-static const char *websocket_http_ng(struct websocket_message *wm) {
+static const char *websocket_ng_process(struct websocket_message *wm) {
+	return websocket_ng_process_generic(wm, control_ng_process);
+}
+static const char *websocket_ng_plain_process(struct websocket_message *wm) {
+	return websocket_ng_process_generic(wm, control_ng_process_plain);
+}
+static const char *websocket_http_ng_generic(struct websocket_message *wm,
+		__typeof__(control_ng_process) cb)
+{
 	struct websocket_ng_buf *buf = obj_alloc0("websocket_ng_buf", sizeof(*buf), __ng_buf_free);
 
 	endpoint_print(&wm->wc->endpoint, buf->addr, sizeof(buf->addr));
@@ -520,7 +543,7 @@ static const char *websocket_http_ng(struct websocket_message *wm) {
 	str_init_len(&buf->cmd, buf->body->str, buf->body->len);
 	buf->endpoint = wm->wc->endpoint;
 
-	if (control_ng_process(&buf->cmd, &buf->endpoint, buf->addr, NULL, websocket_ng_send_http, wm->wc,
+	if (cb(&buf->cmd, &buf->endpoint, buf->addr, NULL, websocket_ng_send_http, wm->wc,
 				&buf->obj))
 		websocket_http_complete(wm->wc, 600, "text/plain", 6, "error\n");
 
@@ -528,6 +551,12 @@ static const char *websocket_http_ng(struct websocket_message *wm) {
 
 	return NULL;
 }
+static const char *websocket_http_ng(struct websocket_message *wm) {
+	return websocket_http_ng_generic(wm, control_ng_process);
+}
+static const char *websocket_http_ng_plain(struct websocket_message *wm) {
+	return websocket_http_ng_generic(wm, control_ng_process_plain);
+}
 
 
 
@@ -638,6 +667,9 @@ static int websocket_http_body(struct websocket_conn *wc, const char *body, size
 
 	if (!strcmp(uri, "/ng") && wm->method == M_POST && wm->content_type == CT_NG)
 		handler = websocket_http_ng;
+	else if (!strcmp(uri, "/ng-plain") && wm->method == M_POST
+			&& (wm->content_type == CT_NG || wm->content_type == CT_JSON))
+		handler = websocket_http_ng_plain;
 	else if (!strcmp(uri, "/admin") && wm->method == M_POST && wm->content_type == CT_JSON)
 		handler = websocket_janus_process;
 	else if (!strcmp(uri, "/janus") && wm->method == M_POST && wm->content_type == CT_JSON)
@@ -667,21 +699,21 @@ static void websocket_conn_cleanup(struct websocket_conn *wc) {
 	// therefore, remove janus_sessions list from wc, then unlock, then iterate the
 	// list, as janus_detach_websocket locks the session
 
-	GHashTable *janus_sessions = wc->janus_sessions;
-	wc->janus_sessions = NULL;
+	janus_sessions_ht janus_sessions = wc->janus_sessions;
+	wc->janus_sessions = janus_sessions_ht_null();
 
 	mutex_unlock(&wc->lock);
 
 	// detach all Janus sessions
-	if (janus_sessions) {
-		GHashTableIter iter;
-		g_hash_table_iter_init(&iter, janus_sessions);
-		gpointer key;
-		while (g_hash_table_iter_next(&iter, &key, NULL)) {
-			janus_detach_websocket(key, wc);
-			obj_put_o(key);
+	if (t_hash_table_is_set(janus_sessions)) {
+		janus_sessions_ht_iter iter;
+		t_hash_table_iter_init(&iter, janus_sessions);
+		struct janus_session *session;
+		while (t_hash_table_iter_next(&iter, &session, NULL)) {
+			janus_detach_websocket(session, wc);
+			obj_put_o((void *) session);
 		}
-		g_hash_table_destroy(janus_sessions);
+		t_hash_table_destroy(janus_sessions);
 	}
 
 
@@ -692,7 +724,7 @@ static void websocket_conn_cleanup(struct websocket_conn *wc) {
 		free(wc->wm->uri);
 	g_slice_free1(sizeof(*wc->wm), wc->wm);
 	wc->wm = NULL;
-	g_queue_clear_full(&wc->output_q, websocket_output_free);
+	t_queue_clear_full(&wc->output_q, websocket_output_free);
 	if (wc->uri)
 		free(wc->uri);
 
@@ -739,10 +771,10 @@ static int websocket_conn_init(struct lws *wsi, void *p) {
 	wc->wsi = wsi;
 	mutex_init(&wc->lock);
 	cond_init(&wc->cond);
-	g_queue_init(&wc->messages);
-	g_queue_push_tail(&wc->output_q, websocket_output_new());
+	t_queue_init(&wc->messages);
+	t_queue_push_tail(&wc->output_q, websocket_output_new());
 	wc->wm = websocket_message_new(wc);
-	wc->janus_sessions = g_hash_table_new(g_direct_hash, g_direct_equal);
+	wc->janus_sessions = janus_sessions_ht_new();
 
 	return 0;
 }
@@ -750,9 +782,9 @@ static int websocket_conn_init(struct lws *wsi, void *p) {
 
 void websocket_conn_add_session(struct websocket_conn *wc, struct janus_session *s) {
 	mutex_lock(&wc->lock);
-	if (wc->janus_sessions) {
-		assert(g_hash_table_lookup(wc->janus_sessions, s) == NULL);
-		g_hash_table_insert(wc->janus_sessions, s, s);
+	if (t_hash_table_is_set(wc->janus_sessions)) {
+		assert(t_hash_table_lookup(wc->janus_sessions, s) == NULL);
+		t_hash_table_insert(wc->janus_sessions, s, s);
 	}
 	mutex_unlock(&wc->lock);
 }
@@ -938,6 +970,11 @@ static int websocket_rtpengine_ng(struct lws *wsi, enum lws_callback_reasons rea
 {
 	return websocket_protocol(wsi, reason, user, in, len, websocket_ng_process, "rtpengine-ng");
 }
+static int websocket_rtpengine_ng_plain(struct lws *wsi, enum lws_callback_reasons reason, void *user, void *in,
+		size_t len)
+{
+	return websocket_protocol(wsi, reason, user, in, len, websocket_ng_plain_process, "rtpengine-ng-plain");
+}
 
 
 static const struct lws_protocols websocket_protocols[] = {
@@ -966,6 +1003,11 @@ static const struct lws_protocols websocket_protocols[] = {
 		.callback = websocket_rtpengine_ng,
 		.per_session_data_size = sizeof(struct websocket_conn),
 	},
+	{
+		.name = "ng-plain.rtpengine.com",
+		.callback = websocket_rtpengine_ng_plain,
+		.per_session_data_size = sizeof(struct websocket_conn),
+	},
 	{ 0, }
 };
 
