@@ -7,6 +7,7 @@
 #include <libavutil/frame.h>
 #include <stdint.h>
 
+#include "types.h"
 
 /*
  * Similar to the existing media_player, but instead of simply producing
@@ -18,9 +19,8 @@
 
 struct audio_player;
 struct call_media;
-struct rtp_payload_type;
 
-bool audio_player_setup(struct call_media *, const struct rtp_payload_type *,
+bool audio_player_setup(struct call_media *, const rtp_payload_type *,
 		unsigned int size_ms, unsigned int delay_ms);
 void audio_player_activate(struct call_media *);
 void audio_player_free(struct call_media *);
@@ -28,7 +28,7 @@ void audio_player_free(struct call_media *);
 void audio_player_start(struct call_media *);
 void audio_player_stop(struct call_media *);
 bool audio_player_is_active(struct call_media *);
-bool audio_player_pt_match(struct call_media *, const struct rtp_payload_type *);
+bool audio_player_pt_match(struct call_media *, const rtp_payload_type *);
 
 void audio_player_add_frame(struct audio_player *, uint32_t ssrc, AVFrame *);
 
