@@ -89,6 +89,9 @@ INLINE bencode_buffer_t *bencode_item_buffer(bencode_item_t *);
 /* like strdup() but uses the bencode buffer to store the string */
 INLINE char *bencode_strdup(bencode_buffer_t *, const char *);
 
+/* ditto but inits a str object */
+INLINE void bencode_strdup_str(bencode_buffer_t *, str *, const char *);
+
 
 
 
@@ -378,6 +381,12 @@ INLINE char *bencode_strdup(bencode_buffer_t *buf, const char *s) {
 	return ret;
 }
 
+INLINE void bencode_strdup_str(bencode_buffer_t *buf, str *o, const char *s) {
+	o->len = strlen(s);
+	o->s = bencode_buffer_alloc(buf, o->len);
+	memcpy(o->s, s, o->len);
+}
+
 INLINE bencode_item_t *bencode_dictionary_add(bencode_item_t *dict, const char *key, bencode_item_t *val) {
 	if (!key)
 		return NULL;
@@ -481,11 +490,11 @@ INLINE char *bencode_dictionary_get_string(bencode_item_t *dict, const char *key
 	return val->iov[1].iov_base;
 }
 
-INLINE char *bencode_dictionary_get_str(bencode_item_t *dict, const char *key, str *str) {
-	str->s = bencode_dictionary_get_string(dict, key, &str->len);
-	if (!str->s)
-		str->len = 0;
-	return str->s;
+INLINE char *bencode_dictionary_get_str(bencode_item_t *dict, const char *key, str *s) {
+	s->s = bencode_dictionary_get_string(dict, key, &s->len);
+	if (!s->s)
+		s->len = 0;
+	return s->s;
 }
 
 INLINE char *bencode_dictionary_get_string_dup(bencode_item_t *dict, const char *key, size_t *len) {
@@ -501,9 +510,9 @@ INLINE char *bencode_dictionary_get_string_dup(bencode_item_t *dict, const char
 	return ret;
 }
 
-INLINE char *bencode_dictionary_get_str_dup(bencode_item_t *dict, const char *key, str *str) {
-	str->s = bencode_dictionary_get_string_dup(dict, key, &str->len);
-	return str->s;
+INLINE char *bencode_dictionary_get_str_dup(bencode_item_t *dict, const char *key, str *s) {
+	s->s = bencode_dictionary_get_string_dup(dict, key, &s->len);
+	return s->s;
 }
 
 INLINE long long int bencode_dictionary_get_integer(bencode_item_t *dict, const char *key, long long int defval) {
@@ -576,12 +585,12 @@ INLINE int bencode_strcmp(bencode_item_t *a, const char *b) {
 		return 1;
 	return memcmp(a->iov[1].iov_base, b, len);
 }
-INLINE int bencode_dictionary_get_strcmp(bencode_item_t *dict, const char *key, const char *str) {
+INLINE int bencode_dictionary_get_strcmp(bencode_item_t *dict, const char *key, const char *s) {
 	bencode_item_t *i;
 	i = bencode_dictionary_get(dict, key);
 	if (!i)
 		return 2;
-	return bencode_strcmp(i, str);
+	return bencode_strcmp(i, s);
 }
 
 INLINE str *bencode_get_str(bencode_item_t *in, str *out) {
