@@ -9,6 +9,7 @@
 #include <errno.h>
 #include <netinet/tcp.h>
 #include <stdbool.h>
+#include "containers.h"
 
 
 
@@ -34,6 +35,8 @@ typedef struct socket socket_t;
 typedef const struct socket_type socktype_t;
 typedef const struct socket_family sockfamily_t;
 
+TYPED_GQUEUE(socket, socket_t)
+
 
 #include "str.h"
 
@@ -205,6 +208,31 @@ INLINE ssize_t socket_sendto_from(socket_t *s, const void *b, size_t l, const en
 	return socket_sendiov(s, &(struct iovec) { .iov_base = (void *) b, .iov_len = l }, l, dst, src);
 }
 
+#define socket_recvfrom_parse_cmsg(tv, to, parse_to, msgh, firsthdr, nexthdr) do { \
+	if ((*tv) || (*to)) { \
+		struct cmsghdr *cm; \
+		for (cm = firsthdr; cm; cm = nexthdr) { \
+			if (cm->cmsg_level == SOL_SOCKET && cm->cmsg_type == SO_TIMESTAMP && (*tv)) { \
+				*(*tv) = *((struct timeval *) CMSG_DATA(cm)); \
+				(*tv) = NULL; \
+			} \
+			if (parse && (*to) && parse_to(cm, (*to))) \
+				(*to) = NULL; \
+		} \
+		if (G_UNLIKELY((*tv))) { \
+			ilog(LOG_WARNING, "No receive timestamp received from kernel"); \
+			ZERO(*(*tv)); \
+		} \
+		if (G_UNLIKELY((*to))) { \
+			ilog(LOG_WARNING, "No local address received from kernel"); \
+			ZERO(*(*to)); \
+		} \
+	} \
+	if (G_UNLIKELY(((msgh)->msg_flags & MSG_TRUNC))) \
+		ilog(LOG_WARNING, "Kernel indicates that data was truncated"); \
+	if (G_UNLIKELY(((msgh)->msg_flags & MSG_CTRUNC))) \
+		ilog(LOG_WARNING, "Kernel indicates that ancillary data was truncated"); \
+} while (0)
 
 
 /* XXX obsolete these? */
@@ -244,6 +272,7 @@ int connect_socket(socket_t *r, int type, const endpoint_t *ep);
 int connect_socket_nb(socket_t *r, int type, const endpoint_t *ep); // 1 == in progress
 int connect_socket_retry(socket_t *r); // retries connect() while in progress
 int close_socket(socket_t *r);
+int reset_socket(socket_t *r);
 void move_socket(socket_t *dst, socket_t *src);
 void dummy_socket(socket_t *r, const sockaddr_t *);
 
@@ -265,9 +294,7 @@ guint sockaddr_t_hash(gconstpointer); // for glib
 gint sockaddr_t_eq(gconstpointer, gconstpointer); // true/false, for glib
 
 unsigned int endpoint_hash(const endpoint_t *);
-bool endpoint_eq(const endpoint_t *, const endpoint_t *); /* true/false */
-guint endpoint_t_hash(gconstpointer); // for glib
-gint endpoint_t_eq(gconstpointer, gconstpointer); // true/false, for glib
+gboolean endpoint_eq(const endpoint_t *, const endpoint_t *); /* true/false */
 
 INLINE sockfamily_t *get_socket_family_enum(enum socket_families i) {
 	if (i >= __SF_LAST)
