@@ -41,6 +41,7 @@ extern int c_mysql_port;
 extern char *forward_to;
 extern endpoint_t tls_send_to_ep;
 extern int tls_resample;
+extern bool tls_disable;
 extern char *notify_uri;
 extern gboolean notify_post;
 extern gboolean notify_nverify;
@@ -48,7 +49,7 @@ extern int notify_threads;
 extern int notify_retries;
 extern gboolean notify_record;
 extern gboolean notify_purge;
-
+extern gboolean mix_output_per_media;
 extern volatile int shutdown_flag;
 
 
