@@ -18,19 +18,19 @@ out:
 	return ret;
 }
 
-void tag_name(metafile_t *mf, unsigned long t, const char *str) {
+void tag_name(metafile_t *mf, unsigned long t, const char *s) {
 	tag_t *tag = tag_get(mf, t);
-	tag->name = g_string_chunk_insert(mf->gsc, str);
+	tag->name = g_string_chunk_insert(mf->gsc, s);
 }
 
-void tag_label(metafile_t *mf, unsigned long t, const char *str) {
+void tag_label(metafile_t *mf, unsigned long t, const char *s) {
 	tag_t *tag = tag_get(mf, t);
-	tag->label = g_string_chunk_insert(mf->gsc, str);
+	tag->label = g_string_chunk_insert(mf->gsc, s);
 }
 
-void tag_metadata(metafile_t *mf, unsigned long t, const char *str) {
+void tag_metadata(metafile_t *mf, unsigned long t, const char *s) {
 	tag_t *tag = tag_get(mf, t);
-	tag->metadata = g_string_chunk_insert(mf->gsc, str);
+	tag->metadata = g_string_chunk_insert(mf->gsc, s);
 }
 
 void tag_free(tag_t *tag) {
