@@ -37,17 +37,29 @@ v=0
 o=- 1545997027 1 IN IP4 198.51.100.1
 s=tester
 t=0 0
+a=s-dummy
+a=s-foo:bar
+a=s-baz:quux blah
 m=audio 6150 RTP/AVP 0
 c=IN IP4 198.51.100.14
 a=sendrecv
+a=m-dummy
+a=m-foo:bar
+a=m-baz:quux blah
 ----------------------------------
 v=0
 o=- 1545997027 1 IN IP4 198.51.100.1
 s=tester
 t=0 0
+a=s-dummy
+a=s-foo:bar
+a=s-baz:quux blah
 m=audio PORT RTP/AVP 0
 c=IN IP4 203.0.113.1
 a=rtpmap:0 PCMU/8000
+a=m-dummy
+a=m-foo:bar
+a=m-baz:quux blah
 a=sendrecv
 a=rtcp:PORT
 SDP
@@ -55,20 +67,32 @@ SDP
 ($port_b) = answer('SIPREC pause/resume',
 	{ }, <<SDP);
 v=0
-o=- 1545997027 1 IN IP4 198.51.100.1
+o=test 1545997028 2 IN IP4 198.51.100.1
 s=tester
 t=0 0
+a=a-dummy
+a=a-foo:bar
+a=a-baz:quux blah
 m=audio 6152 RTP/AVP 0
 c=IN IP4 198.51.100.14
 a=sendrecv
+a=b-dummy
+a=b-foo:bar
+a=b-baz:quux blah
 ----------------------------------
 v=0
-o=- 1545997027 1 IN IP4 198.51.100.1
+o=test 1545997028 2 IN IP4 198.51.100.1
 s=tester
 t=0 0
+a=a-dummy
+a=a-foo:bar
+a=a-baz:quux blah
 m=audio PORT RTP/AVP 0
 c=IN IP4 203.0.113.1
 a=rtpmap:0 PCMU/8000
+a=b-dummy
+a=b-foo:bar
+a=b-baz:quux blah
 a=sendrecv
 a=rtcp:PORT
 SDP
@@ -81,15 +105,21 @@ snd($sock_a, $port_b, rtp(0, 4000, 7000, 0x6543, "\x00" x 160));
 rcv_no($sock_c);
 
 (undef, $ttr, undef, undef, undef, $port_c) = subscribe_request('SIPREC pause/resume',
-	{ 'from-tag' => ft(), flags => ['SIPREC'] }, <<SDP);
+	{ 'from-tag' => ft(), flags => ['SIPREC', 'replace-origin'] }, <<SDP);
 v=0
-o=- 1545997027 1 IN IP4 198.51.100.1
+o=- 1545997027 1 IN IP4 203.0.113.1
 s=tester
 t=0 0
+a=s-dummy
+a=s-foo:bar
+a=s-baz:quux blah
 m=audio PORT RTP/AVP 0
 c=IN IP4 203.0.113.1
 a=label:1
 a=rtpmap:0 PCMU/8000
+a=m-dummy
+a=m-foo:bar
+a=m-baz:quux blah
 a=sendonly
 a=rtcp:PORT
 SDP
@@ -119,10 +149,16 @@ v=0
 o=- 1545997027 1 IN IP4 198.51.100.1
 s=tester
 t=0 0
+a=s-dummy
+a=s-foo:bar
+a=s-baz:quux blah
 m=audio PORT RTP/AVP 0
 c=IN IP4 203.0.113.1
 a=label:1
 a=rtpmap:0 PCMU/8000
+a=m-dummy
+a=m-foo:bar
+a=m-baz:quux blah
 a=sendonly
 a=rtcp:PORT
 SDP
@@ -152,10 +188,16 @@ v=0
 o=- 1545997027 1 IN IP4 198.51.100.1
 s=tester
 t=0 0
+a=s-dummy
+a=s-foo:bar
+a=s-baz:quux blah
 m=audio PORT RTP/AVP 0
 c=IN IP4 203.0.113.1
 a=label:1
 a=rtpmap:0 PCMU/8000
+a=m-dummy
+a=m-foo:bar
+a=m-baz:quux blah
 a=sendonly
 a=rtcp:PORT
 SDP
@@ -186,10 +228,16 @@ v=0
 o=- 1545997027 1 IN IP4 198.51.100.1
 s=tester
 t=0 0
+a=s-dummy
+a=s-foo:bar
+a=s-baz:quux blah
 m=audio PORT RTP/AVP 0
 c=IN IP4 203.0.113.1
 a=label:1
 a=rtpmap:0 PCMU/8000
+a=m-dummy
+a=m-foo:bar
+a=m-baz:quux blah
 a=inactive
 a=rtcp:PORT
 SDP
@@ -219,10 +267,16 @@ v=0
 o=- 1545997027 1 IN IP4 198.51.100.1
 s=tester
 t=0 0
+a=s-dummy
+a=s-foo:bar
+a=s-baz:quux blah
 m=audio PORT RTP/AVP 0
 c=IN IP4 203.0.113.1
 a=label:1
 a=rtpmap:0 PCMU/8000
+a=m-dummy
+a=m-foo:bar
+a=m-baz:quux blah
 a=sendonly
 a=rtcp:PORT
 SDP
@@ -568,6 +622,35 @@ rcv_no($sock_ax);
 rcv_no($sock_bx);
 rcv_no($sock_cx);
 
+$resp = rtpe_req('unsubscribe', 'simple sub, no RTCP mirror', { 'from-tag' => ft(), 'to-tag' => $ttr });
+
+snd($sock_b, $port_a, rtp(0, 2002, 4320, 0x3456, "\x00" x 160));
+rcv($sock_a, $port_b, rtpm(0, 2002, 4320, 0x3456, "\x00" x 160));
+rcv_no($sock_b);
+rcv_no($sock_c);
+snd($sock_a, $port_b, rtp(0, 4002, 7320, 0x6543, "\x00" x 160));
+rcv($sock_b, $port_a, rtpm(0, 4002, 7320, 0x6543, "\x00" x 160));
+rcv_no($sock_c);
+rcv_no($sock_a);
+snd($sock_c, $port_c, rtp(0, 5001, 8160, 0x9876, "\x00" x 160));
+rcv_no($sock_a);
+rcv_no($sock_b);
+rcv_no($sock_c);
+
+snd($sock_bx, $port_ax, "\x81\xc8\x00\x0c\x00\x00\x34\x56xxxxyyyy\x00\x00\x22\xd8\x00\x00\x00\x25\x00\x00\x18\xdc\x00\x00\x65\x43\x06\x00\x00\x01\x00\x00\x07\xf5\x00\x00\x00\x001234qwer\x81\xca\x00\x05\x00\x00\x34\x56\x01\x0cabcdefghijkl\x00\x00");
+rcv($sock_ax, $port_bx, qr/^\x81\xc8\x00\x0c\x00\x00\x34\x56xxxxyyyy\x00\x00\x22\xd8\x00\x00\x00\x25\x00\x00\x18\xdc\x00\x00\x65\x43\x06\x00\x00\x01\x00\x00\x07\xf5\x00\x00\x00\x001234qwer\x81\xca\x00\x05\x00\x00\x34\x56\x01\x0cabcdefghijkl\x00\x00$/);
+rcv_no($sock_bx);
+rcv_no($sock_cx);
+
+snd($sock_ax, $port_bx, "\x81\xc8\x00\x0c\x00\x00\x65\x43aaaabbbb\x00\x00\x22\xd8\x00\x00\x00\x25\x00\x00\x18\xdc\x00\x00\x34\x56\x06\x00\x00\x01\x00\x00\x07\xf5\x00\x00\x00\x00poiuytre\x81\xca\x00\x05\x00\x00\x65\x43\x01\x0cqwertyuiopqw\x00\x00");
+rcv($sock_bx, $port_ax, qr/^\x81\xc8\x00\x0c\x00\x00\x65\x43aaaabbbb\x00\x00\x22\xd8\x00\x00\x00\x25\x00\x00\x18\xdc\x00\x00\x34\x56\x06\x00\x00\x01\x00\x00\x07\xf5\x00\x00\x00\x00poiuytre\x81\xca\x00\x05\x00\x00\x65\x43\x01\x0cqwertyuiopqw\x00\x00$/);
+rcv_no($sock_cx);
+rcv_no($sock_ax);
+
+snd($sock_cx, $port_cx, "\x81\xc8\x00\x0c\x00\x00\x98\x76aaaabbbb\x00\x00\x22\xd8\x00\x00\x00\x25\x00\x00\x18\xdc\x00\x00\x65\x43\x06\x00\x00\x01\x00\x00\x07\xf5\x00\x00\x00\x00poiuytre\x81\xca\x00\x05\x00\x00\x98\x76\x01\x0cqwertyuiopqw\x00\x00");
+rcv_no($sock_ax);
+rcv_no($sock_bx);
+rcv_no($sock_cx);
 
 
 
@@ -692,6 +775,37 @@ rcv($sock_ax, $port_bx, qr/^\x81\xc8\x00\x0c\x00\x00\x98\x76aaaabbbb\x00\x00\x22
 rcv_no($sock_bx);
 rcv_no($sock_cx);
 
+$resp = rtpe_req('unsubscribe', 'simple sub, RTCP mirror', { 'from-tag' => ft(), 'to-tag' => $ttr });
+
+snd($sock_b, $port_a, rtp(0, 2002, 4320, 0x3456, "\x00" x 160));
+rcv($sock_a, $port_b, rtpm(0, 2002, 4320, 0x3456, "\x00" x 160));
+rcv_no($sock_b);
+rcv_no($sock_c);
+snd($sock_a, $port_b, rtp(0, 4002, 7320, 0x6543, "\x00" x 160));
+rcv($sock_b, $port_a, rtpm(0, 4002, 7320, 0x6543, "\x00" x 160));
+rcv_no($sock_c);
+rcv_no($sock_a);
+snd($sock_c, $port_c, rtp(0, 5001, 8320, 0x9876, "\x00" x 160));
+rcv_no($sock_a);
+rcv_no($sock_b);
+rcv_no($sock_c);
+
+snd($sock_bx, $port_ax, "\x81\xc8\x00\x0c\x00\x00\x34\x56xxxxyyyy\x00\x00\x22\xd8\x00\x00\x00\x25\x00\x00\x18\xdc\x00\x00\x65\x43\x06\x00\x00\x01\x00\x00\x07\xf5\x00\x00\x00\x001234qwer\x81\xca\x00\x05\x00\x00\x34\x56\x01\x0cabcdefghijkl\x00\x00");
+rcv($sock_ax, $port_bx, qr/^\x81\xc8\x00\x0c\x00\x00\x34\x56xxxxyyyy\x00\x00\x22\xd8\x00\x00\x00\x25\x00\x00\x18\xdc\x00\x00\x65\x43\x06\x00\x00\x01\x00\x00\x07\xf5\x00\x00\x00\x001234qwer\x81\xca\x00\x05\x00\x00\x34\x56\x01\x0cabcdefghijkl\x00\x00$/);
+rcv_no($sock_bx);
+rcv_no($sock_cx);
+
+snd($sock_ax, $port_bx, "\x81\xc8\x00\x0c\x00\x00\x65\x43aaaabbbb\x00\x00\x22\xd8\x00\x00\x00\x25\x00\x00\x18\xdc\x00\x00\x34\x56\x06\x00\x00\x01\x00\x00\x07\xf5\x00\x00\x00\x00poiuytre\x81\xca\x00\x05\x00\x00\x65\x43\x01\x0cqwertyuiopqw\x00\x00");
+rcv($sock_bx, $port_ax, qr/^\x81\xc8\x00\x0c\x00\x00\x65\x43aaaabbbb\x00\x00\x22\xd8\x00\x00\x00\x25\x00\x00\x18\xdc\x00\x00\x34\x56\x06\x00\x00\x01\x00\x00\x07\xf5\x00\x00\x00\x00poiuytre\x81\xca\x00\x05\x00\x00\x65\x43\x01\x0cqwertyuiopqw\x00\x00$/);
+rcv_no($sock_cx);
+rcv_no($sock_ax);
+
+snd($sock_cx, $port_cx, "\x81\xc8\x00\x0c\x00\x00\x98\x76aaaabbbb\x00\x00\x22\xd8\x00\x00\x00\x25\x00\x00\x18\xdc\x00\x00\x65\x43\x06\x00\x00\x01\x00\x00\x07\xf5\x00\x00\x00\x00poiuytre\x81\xca\x00\x05\x00\x00\x98\x76\x01\x0cqwertyuiopqw\x00\x00");
+rcv($sock_ax, $port_bx, qr/^\x81\xc8\x00\x0c\x00\x00\x98\x76aaaabbbb\x00\x00\x22\xd8\x00\x00\x00\x25\x00\x00\x18\xdc\x00\x00\x65\x43\x06\x00\x00\x01\x00\x00\x07\xf5\x00\x00\x00\x00poiuytre\x81\xca\x00\x05\x00\x00\x98\x76\x01\x0cqwertyuiopqw\x00\x00$/);
+rcv_no($sock_bx);
+rcv_no($sock_cx);
+
+
 
 
 
@@ -748,39 +862,28 @@ SDP
 
 
 ($ftr, $ttr, $fts, $tag_medias, $media_labels) = subscribe_request('SIPREC sub',
-	{ flags => ['all', 'SIPREC'] }, <<SDP);
+	{ flags => ['all', 'SIPREC', 'replace-origin'] }, <<SDP);
 v=0
 o=- SDP_VERSION IN IP4 203.0.113.1
-s=RTPE_VERSION
+s=tester
 t=0 0
 m=audio PORT RTP/AVP 0
 c=IN IP4 203.0.113.1
-a=label:1
-a=rtpmap:0 PCMU/8000
-a=sendonly
-a=rtcp:PORT
-m=audio PORT RTP/AVP 0
-c=IN IP4 203.0.113.1
 a=label:0
 a=rtpmap:0 PCMU/8000
 a=sendonly
 a=rtcp:PORT
+m=audio PORT RTP/AVP 0
+c=IN IP4 203.0.113.1
+a=label:1
+a=rtpmap:0 PCMU/8000
+a=sendonly
+a=rtcp:PORT
 SDP
 
 is $ftr, undef, 'from-tag matches';
-is_deeply $fts, [ft(), tt()], 'from-tags match';
+is_deeply $fts, [tt(), ft()], 'from-tags match';
 is_deeply $tag_medias, [
-	{
-		tag => ft(),
-		medias => [
-			{
-				index => 1,
-				type => 'audio',
-				label => '1',
-				mode => 'sendrecv',
-			},
-		],
-	},
 	{
 		tag => tt(),
 		medias => [
@@ -792,6 +895,17 @@ is_deeply $tag_medias, [
 			},
 		],
 	},
+	{
+		tag => ft(),
+		medias => [
+			{
+				index => 1,
+				type => 'audio',
+				label => '1',
+				mode => 'sendrecv',
+			},
+		],
+	},
 ], 'tag-medias match';
 is_deeply $media_labels, {
 	'1' => {
@@ -858,38 +972,26 @@ SDP
 ($ftr, $ttr, $fts, $tag_medias, $media_labels) = subscribe_request('SIPREC sub',
 	{ flags => ['all', 'SIPREC'] }, <<SDP);
 v=0
-o=- SDP_VERSION IN IP4 203.0.113.1
-s=RTPE_VERSION
+o=- SDP_VERSION IN IP4 198.51.100.2
+s=tester
 t=0 0
 m=audio PORT RTP/AVP 0
 c=IN IP4 203.0.113.1
-a=label:1
-a=rtpmap:0 PCMU/8000
-a=sendonly
-a=rtcp:PORT
-m=audio PORT RTP/AVP 0
-c=IN IP4 203.0.113.1
 a=label:0
 a=rtpmap:0 PCMU/8000
 a=sendonly
 a=rtcp:PORT
+m=audio PORT RTP/AVP 0
+c=IN IP4 203.0.113.1
+a=label:1
+a=rtpmap:0 PCMU/8000
+a=sendonly
+a=rtcp:PORT
 SDP
 
 is $ftr, undef, 'from-tag matches';
-is_deeply $fts, [ft(), tt()], 'from-tags match';
+is_deeply $fts, [tt(), ft()], 'from-tags match';
 is_deeply $tag_medias, [
-	{
-		tag => ft(),
-		label => 'caller',
-		medias => [
-			{
-				index => 1,
-				type => 'audio',
-				label => '1',
-				mode => 'sendrecv',
-			},
-		],
-	},
 	{
 		tag => tt(),
 		label => 'called',
@@ -902,6 +1004,18 @@ is_deeply $tag_medias, [
 			},
 		],
 	},
+	{
+		tag => ft(),
+		label => 'caller',
+		medias => [
+			{
+				index => 1,
+				type => 'audio',
+				label => '1',
+				mode => 'sendrecv',
+			},
+		],
+	},
 ], 'tag-medias match';
 is_deeply $media_labels, {
 	'1' => {
@@ -976,11 +1090,11 @@ rcv($sock_a, $port_b, rtpm(0, 2000, 4000, 0x3456, "\x00" x 160));
 snd($sock_a, $port_b, rtp(0, 4000, 7000, 0x6543, "\x00" x 160));
 rcv($sock_b, $port_a, rtpm(0, 4000, 7000, 0x6543, "\x00" x 160));
 
-($ftr, $ttr, $fts, undef, undef, $port_c, undef, $port_d) = subscribe_request('"all" sub',
+($ftr, $ttr, $fts, undef, undef, $port_d, undef, $port_c) = subscribe_request('"all" sub',
 	{ 'flags' => ['all'] }, <<SDP);
 v=0
-o=- SDP_VERSION IN IP4 203.0.113.1
-s=RTPE_VERSION
+o=- SDP_VERSION IN IP4 198.51.100.1
+s=tester
 t=0 0
 m=audio PORT RTP/AVP 0 8
 c=IN IP4 203.0.113.1
@@ -997,7 +1111,7 @@ a=rtcp:PORT
 SDP
 
 is $ftr, undef, 'from-tag matches';
-is_deeply $fts, [ft(), tt()], 'from-tags match';
+is_deeply $fts, [tt(), ft()], 'from-tags match';
 
 subscribe_answer('"all" sub',
 	{ 'to-tag' => $ttr, flags => ['allow transcoding'] }, <<SDP);
@@ -1005,10 +1119,10 @@ v=0
 o=- 1545997027 1 IN IP4 198.51.100.1
 s=tester
 t=0 0
-m=audio 6084 RTP/AVP 8
+m=audio 6086 RTP/AVP 8
 c=IN IP4 198.51.100.14
 a=recvonly
-m=audio 6086 RTP/AVP 8
+m=audio 6084 RTP/AVP 8
 c=IN IP4 198.51.100.14
 a=recvonly
 SDP
@@ -1041,18 +1155,30 @@ v=0
 o=- 1545997027 1 IN IP4 198.51.100.1
 s=tester
 t=0 0
+a=s-dummy
+a=s-foo:bar
+a=s-baz:quux blah
 m=audio 6088 RTP/AVP 0 8
 c=IN IP4 198.51.100.14
 a=sendrecv
+a=m-dummy
+a=m-foo:bar
+a=m-baz:quux blah
 ----------------------------------
 v=0
 o=- 1545997027 1 IN IP4 198.51.100.1
 s=tester
 t=0 0
+a=s-dummy
+a=s-foo:bar
+a=s-baz:quux blah
 m=audio PORT RTP/AVP 0 8
 c=IN IP4 203.0.113.1
 a=rtpmap:0 PCMU/8000
 a=rtpmap:8 PCMA/8000
+a=m-dummy
+a=m-foo:bar
+a=m-baz:quux blah
 a=sendrecv
 a=rtcp:PORT
 SDP
@@ -1063,18 +1189,30 @@ v=0
 o=- 1545997027 1 IN IP4 198.51.100.1
 s=tester
 t=0 0
+a=a-dummy
+a=a-foo:bar
+a=a-baz:quux blah
 m=audio 6090 RTP/AVP 0 8
 c=IN IP4 198.51.100.14
 a=sendrecv
+a=b-dummy
+a=b-foo:bar
+a=b-baz:quux blah
 ----------------------------------
 v=0
 o=- 1545997027 1 IN IP4 198.51.100.1
 s=tester
 t=0 0
+a=a-dummy
+a=a-foo:bar
+a=a-baz:quux blah
 m=audio PORT RTP/AVP 0 8
 c=IN IP4 203.0.113.1
 a=rtpmap:0 PCMU/8000
 a=rtpmap:8 PCMA/8000
+a=b-dummy
+a=b-foo:bar
+a=b-baz:quux blah
 a=sendrecv
 a=rtcp:PORT
 SDP
@@ -1088,19 +1226,25 @@ rcv($sock_b, $port_a, rtpm(0, 4000, 7000, 0x6543, "\x00" x 160));
 ($ftr, $ttr, $fts, undef, undef, $port_c, undef, $port_d) = subscribe_request('sub to multiple tags',
 	{ 'from-tags' => [ft(), tt()] }, <<SDP);
 v=0
-o=- SDP_VERSION IN IP4 203.0.113.1
-s=RTPE_VERSION
+o=- SDP_VERSION IN IP4 198.51.100.1
+s=tester
 t=0 0
 m=audio PORT RTP/AVP 0 8
 c=IN IP4 203.0.113.1
 a=rtpmap:0 PCMU/8000
 a=rtpmap:8 PCMA/8000
+a=m-dummy
+a=m-foo:bar
+a=m-baz:quux blah
 a=sendonly
 a=rtcp:PORT
 m=audio PORT RTP/AVP 0 8
 c=IN IP4 203.0.113.1
 a=rtpmap:0 PCMU/8000
 a=rtpmap:8 PCMA/8000
+a=b-dummy
+a=b-foo:bar
+a=b-baz:quux blah
 a=sendonly
 a=rtcp:PORT
 SDP
@@ -1188,8 +1332,8 @@ rcv($sock_b, $port_a, rtpm(0, 4000, 7000, 0x6543, "\x00" x 160));
 ($ftr, $ttr, $fts, undef, undef, $port_c, undef, $port_d) = subscribe_request('sub to multiple tags via flags',
 	{ flags => ['from-tags-' . ft(), 'from-tags-' . tt()] }, <<SDP);
 v=0
-o=- SDP_VERSION IN IP4 203.0.113.1
-s=RTPE_VERSION
+o=- SDP_VERSION IN IP4 198.51.100.1
+s=tester
 t=0 0
 m=audio PORT RTP/AVP 0 8
 c=IN IP4 203.0.113.1
@@ -1288,8 +1432,8 @@ rcv($sock_b, $port_a, rtpm(0, 4000, 7000, 0x6543, "\x00" x 160));
 ($ftr, $ttr, $fts, undef, undef, $port_c, undef, $port_d) = subscribe_request('sub to multiple tags - reverse',
 	{ 'from-tags' => [tt(), ft()] }, <<SDP);
 v=0
-o=- SDP_VERSION IN IP4 203.0.113.1
-s=RTPE_VERSION
+o=- SDP_VERSION IN IP4 198.51.100.1
+s=tester
 t=0 0
 m=audio PORT RTP/AVP 0 8
 c=IN IP4 203.0.113.1
@@ -1616,21 +1760,21 @@ rcv($sock_a, $port_b, rtpm(0, 2000, 4000, 0x3456, "\x00" x 160));
 snd($sock_a, $port_b, rtp(0, 4000, 7000, 0x6543, "\x00" x 160));
 rcv($sock_b, $port_a, rtpm(0, 4000, 7000, 0x6543, "\x00" x 160));
 
-(undef, $ttr, undef, undef, undef, $port_c, undef, $port_d) = subscribe_request('SIPREC sub pause/resume',
+(undef, $ttr, undef, undef, undef, $port_d, undef, $port_c) = subscribe_request('SIPREC sub pause/resume',
 	{ flags => ['all', 'SIPREC'] }, <<SDP);
 v=0
-o=- SDP_VERSION IN IP4 203.0.113.1
-s=RTPE_VERSION
+o=- SDP_VERSION IN IP4 198.51.100.1
+s=tester
 t=0 0
 m=audio PORT RTP/AVP 0
 c=IN IP4 203.0.113.1
-a=label:1
+a=label:0
 a=rtpmap:0 PCMU/8000
 a=sendonly
 a=rtcp:PORT
 m=audio PORT RTP/AVP 0
 c=IN IP4 203.0.113.1
-a=label:0
+a=label:1
 a=rtpmap:0 PCMU/8000
 a=sendonly
 a=rtcp:PORT
@@ -1642,14 +1786,14 @@ v=0
 o=- 1545997027 1 IN IP4 198.51.100.1
 s=tester
 t=0 0
-m=audio 6122 RTP/AVP 0
-c=IN IP4 198.51.100.14
-a=label:1
-a=recvonly
 m=audio 6124 RTP/AVP 0
 c=IN IP4 198.51.100.14
 a=label:0
 a=recvonly
+m=audio 6122 RTP/AVP 0
+c=IN IP4 198.51.100.14
+a=label:1
+a=recvonly
 SDP
 
 snd($sock_b, $port_a, rtp(0, 2001, 4160, 0x3456, "\x00" x 160));
@@ -1706,38 +1850,27 @@ a=recvonly
 a=rtcp:PORT
 SDP
 
-(undef, undef, undef, $tag_medias, $media_labels, $port_c, undef, $port_d) = subscribe_request('SIPREC sub pause/resume',
+(undef, undef, undef, $tag_medias, $media_labels, $port_d, undef, $port_c) = subscribe_request('SIPREC sub pause/resume',
 	{ flags => ['all', 'SIPREC'], 'to-tag' => $ttr }, <<SDP);
 v=0
-o=- SDP_VERSION IN IP4 203.0.113.1
-s=RTPE_VERSION
+o=- SDP_VERSION IN IP4 198.51.100.1
+s=tester
 t=0 0
 m=audio PORT RTP/AVP 0
 c=IN IP4 203.0.113.1
-a=label:1
-a=rtpmap:0 PCMU/8000
-a=inactive
-a=rtcp:PORT
-m=audio PORT RTP/AVP 0
-c=IN IP4 203.0.113.1
 a=label:0
 a=rtpmap:0 PCMU/8000
 a=sendonly
 a=rtcp:PORT
+m=audio PORT RTP/AVP 0
+c=IN IP4 203.0.113.1
+a=label:1
+a=rtpmap:0 PCMU/8000
+a=inactive
+a=rtcp:PORT
 SDP
 
 is_deeply $tag_medias, [
-	{
-		tag => ft(),
-		medias => [
-			{
-				index => 1,
-				type => 'audio',
-				label => '1',
-				mode => 'sendonly',
-			},
-		],
-	},
 	{
 		tag => tt(),
 		medias => [
@@ -1749,6 +1882,17 @@ is_deeply $tag_medias, [
 			},
 		],
 	},
+	{
+		tag => ft(),
+		medias => [
+			{
+				index => 1,
+				type => 'audio',
+				label => '1',
+				mode => 'sendonly',
+			},
+		],
+	},
 ], 'SIPREC sub pause/resume - tag-medias match';
 is_deeply $media_labels, {
 	'1' => {
@@ -1771,14 +1915,14 @@ v=0
 o=- 1545997027 1 IN IP4 198.51.100.1
 s=tester
 t=0 0
-m=audio 6122 RTP/AVP 0
-c=IN IP4 198.51.100.14
-a=label:1
-a=inactive
 m=audio 6124 RTP/AVP 0
 c=IN IP4 198.51.100.14
 a=label:0
 a=recvonly
+m=audio 6122 RTP/AVP 0
+c=IN IP4 198.51.100.14
+a=label:1
+a=inactive
 SDP
 
 snd($sock_b, $port_a, rtp(0, 2001, 4320, 0x3456, "\x00" x 160));
@@ -1827,38 +1971,27 @@ a=sendrecv
 a=rtcp:PORT
 SDP
 
-(undef, $ttr, undef, $tag_medias, $media_labels, $port_c, undef, $port_d) = subscribe_request('SIPREC sub pause/resume',
+(undef, $ttr, undef, $tag_medias, $media_labels, $port_d, undef, $port_c) = subscribe_request('SIPREC sub pause/resume',
 	{ flags => ['all', 'SIPREC'], 'to-tag' => $ttr }, <<SDP);
 v=0
-o=- SDP_VERSION IN IP4 203.0.113.1
-s=RTPE_VERSION
+o=- SDP_VERSION IN IP4 198.51.100.1
+s=tester
 t=0 0
 m=audio PORT RTP/AVP 0
 c=IN IP4 203.0.113.1
-a=label:1
+a=label:0
 a=rtpmap:0 PCMU/8000
 a=sendonly
 a=rtcp:PORT
 m=audio PORT RTP/AVP 0
 c=IN IP4 203.0.113.1
-a=label:0
+a=label:1
 a=rtpmap:0 PCMU/8000
 a=sendonly
 a=rtcp:PORT
 SDP
 
 is_deeply $tag_medias, [
-	{
-		tag => ft(),
-		medias => [
-			{
-				index => 1,
-				type => 'audio',
-				label => '1',
-				mode => 'sendrecv',
-			},
-		],
-	},
 	{
 		tag => tt(),
 		medias => [
@@ -1870,6 +2003,17 @@ is_deeply $tag_medias, [
 			},
 		],
 	},
+	{
+		tag => ft(),
+		medias => [
+			{
+				index => 1,
+				type => 'audio',
+				label => '1',
+				mode => 'sendrecv',
+			},
+		],
+	},
 ], 'SIPREC sub pause/resume - tag-medias match';
 is_deeply $media_labels, {
 	'1' => {
@@ -1892,14 +2036,14 @@ v=0
 o=- 1545997027 1 IN IP4 198.51.100.1
 s=tester
 t=0 0
-m=audio 6122 RTP/AVP 0
-c=IN IP4 198.51.100.14
-a=label:1
-a=recvonly
 m=audio 6124 RTP/AVP 0
 c=IN IP4 198.51.100.14
 a=label:0
 a=recvonly
+m=audio 6122 RTP/AVP 0
+c=IN IP4 198.51.100.14
+a=label:1
+a=recvonly
 SDP
 
 snd($sock_b, $port_a, rtp(0, 2001, 4480, 0x3456, "\x00" x 160));
@@ -1952,38 +2096,27 @@ a=recvonly
 a=rtcp:PORT
 SDP
 
-(undef, undef, undef, $tag_medias, $media_labels, $port_c, undef, $port_d) = subscribe_request('SIPREC sub pause/resume',
+(undef, undef, undef, $tag_medias, $media_labels, $port_d, undef, $port_c) = subscribe_request('SIPREC sub pause/resume',
 	{ flags => ['all', 'SIPREC'], 'to-tag' => $ttr }, <<SDP);
 v=0
-o=- SDP_VERSION IN IP4 203.0.113.1
-s=RTPE_VERSION
+o=- SDP_VERSION IN IP4 198.51.100.1
+s=tester
 t=0 0
 m=audio PORT RTP/AVP 0
 c=IN IP4 203.0.113.1
-a=label:1
-a=rtpmap:0 PCMU/8000
-a=sendonly
-a=rtcp:PORT
-m=audio PORT RTP/AVP 0
-c=IN IP4 203.0.113.1
 a=label:0
 a=rtpmap:0 PCMU/8000
 a=inactive
 a=rtcp:PORT
+m=audio PORT RTP/AVP 0
+c=IN IP4 203.0.113.1
+a=label:1
+a=rtpmap:0 PCMU/8000
+a=sendonly
+a=rtcp:PORT
 SDP
 
 is_deeply $tag_medias, [
-	{
-		tag => ft(),
-		medias => [
-			{
-				index => 1,
-				type => 'audio',
-				label => '1',
-				mode => 'recvonly',
-			},
-		],
-	},
 	{
 		tag => tt(),
 		medias => [
@@ -1995,6 +2128,17 @@ is_deeply $tag_medias, [
 			},
 		],
 	},
+	{
+		tag => ft(),
+		medias => [
+			{
+				index => 1,
+				type => 'audio',
+				label => '1',
+				mode => 'recvonly',
+			},
+		],
+	},
 ], 'tag-medias match';
 is_deeply $media_labels, {
 	'1' => {
@@ -2017,14 +2161,14 @@ v=0
 o=- 1545997027 1 IN IP4 198.51.100.1
 s=tester
 t=0 0
-m=audio 6122 RTP/AVP 0
-c=IN IP4 198.51.100.14
-a=label:1
-a=recvonly
 m=audio 6124 RTP/AVP 0
 c=IN IP4 198.51.100.14
 a=label:0
 a=inactive
+m=audio 6122 RTP/AVP 0
+c=IN IP4 198.51.100.14
+a=label:1
+a=recvonly
 SDP
 
 snd($sock_a, $port_b, rtp(0, 4001, 7480, 0x6543, "\x00" x 160));
@@ -3166,9 +3310,15 @@ v=0
 o=- 1545997027 1 IN IP4 198.51.100.1
 s=tester
 t=0 0
+a=s-dummy
+a=s-foo:bar
+a=s-baz:quux blah
 m=audio 6042 RTP/AVP 0 8 9
 c=IN IP4 198.51.100.14
 a=sendonly
+a=m-dummy
+a=m-foo:bar
+a=m-baz:quux blah
 ----------------------------------
 v=0
 o=- SDP_VERSION IN IP4 203.0.113.1
@@ -3189,9 +3339,15 @@ v=0
 o=- 1545997027 1 IN IP4 198.51.100.1
 s=tester
 t=0 0
+a=s-dummy
+a=s-foo:bar
+a=s-baz:quux blah
 m=audio PORT RTP/AVP 0
 c=IN IP4 203.0.113.1
 a=rtpmap:0 PCMU/8000
+a=m-dummy
+a=m-foo:bar
+a=m-baz:quux blah
 a=sendonly
 a=rtcp:PORT
 SDP
@@ -3218,9 +3374,15 @@ v=0
 o=- 1545997027 1 IN IP4 198.51.100.1
 s=tester
 t=0 0
+a=s-dummy
+a=s-foo:bar
+a=s-baz:quux blah
 m=audio PORT RTP/AVP 0
 c=IN IP4 203.0.113.1
 a=rtpmap:0 PCMU/8000
+a=m-dummy
+a=m-foo:bar
+a=m-baz:quux blah
 a=sendonly
 a=rtcp:PORT
 SDP
@@ -3424,5 +3586,92 @@ rcv($sock_c, $port_c, rtpm(0, 2002, 4320, -1, "\x29" x 160));
 
 
 
+new_call;
+
+publish('multi publish/subscribe',
+	{ }, <<SDP);
+v=0
+o=- 1545997027 1 IN IP4 198.51.100.1
+s=tester
+t=0 0
+a=s-dummy
+a=s-foo:bar
+a=s-baz:quux blah
+m=audio 6078 RTP/AVP 0 8 9
+c=IN IP4 198.51.100.14
+a=sendonly
+a=m-dummy
+a=m-foo:bar
+a=m-baz:quux blah
+----------------------------------
+v=0
+o=- SDP_VERSION IN IP4 203.0.113.1
+s=RTPE_VERSION
+t=0 0
+m=audio PORT RTP/AVP 0
+c=IN IP4 203.0.113.1
+a=rtpmap:0 PCMU/8000
+a=recvonly
+a=rtcp:PORT
+SDP
+
+$ftr = ft();
+new_ft;
+
+publish('multi publish/subscribe',
+	{ }, <<SDP);
+v=0
+o=- 1545997027 1 IN IP4 198.51.100.1
+s=tester
+t=0 0
+a=a-dummy
+a=a-foo:bar
+a=a-baz:quux blah
+m=audio 6126 RTP/AVP 0 8 9
+c=IN IP4 198.51.100.14
+a=sendonly
+a=b-dummy
+a=b-foo:bar
+a=b-baz:quux blah
+----------------------------------
+v=0
+o=- SDP_VERSION IN IP4 203.0.113.1
+s=RTPE_VERSION
+t=0 0
+m=audio PORT RTP/AVP 0
+c=IN IP4 203.0.113.1
+a=rtpmap:0 PCMU/8000
+a=recvonly
+a=rtcp:PORT
+SDP
+
+$fts = ft();
+
+subscribe_request('multi publish/subscribe',
+	{ 'from-tags' => [ $ftr, $fts ] }, <<SDP);
+v=0
+o=- SDP_VERSION IN IP4 198.51.100.1
+s=tester
+t=0 0
+m=audio PORT RTP/AVP 0
+c=IN IP4 203.0.113.1
+a=rtpmap:0 PCMU/8000
+a=m-dummy
+a=m-foo:bar
+a=m-baz:quux blah
+a=sendonly
+a=rtcp:PORT
+m=audio PORT RTP/AVP 0
+c=IN IP4 203.0.113.1
+a=rtpmap:0 PCMU/8000
+a=b-dummy
+a=b-foo:bar
+a=b-baz:quux blah
+a=sendonly
+a=rtcp:PORT
+SDP
+
+
+
 done_testing();
 #done_testing;NGCP::Rtpengine::AutoTest::terminate('f00');exit;
