@@ -5,26 +5,31 @@
 #include "main.h"
 #include "ssrc.h"
 #include "helpers.h"
+#include "bufferpool.h"
 
 int _log_facility_rtcp;
 int _log_facility_cdr;
 int _log_facility_dtmf;
 struct rtpengine_config rtpe_config;
 struct rtpengine_config initial_rtpe_config;
-struct poller *rtpe_poller;
-struct poller_map *rtpe_poller_map;
+struct poller **rtpe_pollers;
+struct poller *rtpe_control_poller;
+struct poller *uring_poller;
+unsigned int num_media_pollers;
+unsigned int rtpe_poller_rr_iter;
 GString *dtmf_logs;
 GQueue rtpe_control_ng = G_QUEUE_INIT;
+struct bufferpool *shm_bufferpool;
 
 static str *sdup(char *s) {
 	str r = STR_INIT(s);
 	return str_dup(&r);
 }
-static void queue_dump(GString *s, GQueue *q) {
-	for (GList *l = q->head; l; l = l->next) {
+static void queue_dump(GString *s, rtp_pt_q *q) {
+	for (__auto_type l = q->head; l; l = l->next) {
 		if (s->len)
 			g_string_append(s, " ");
-		struct rtp_payload_type *pt = l->data;
+		rtp_payload_type *pt = l->data;
 		g_string_append_printf(s, "%i/%s", pt->payload_type, pt->encoding_with_params.s);
 		if (pt->format_parameters.len)
 			g_string_append_printf(s, "/%s", pt->format_parameters.s);
@@ -36,8 +41,8 @@ static GHashTable *rtp_ts_ht;
 static GHashTable *rtp_seq_ht;
 static uint32_t ssrc_A;
 static uint32_t ssrc_B;
-static struct call call;
-static struct sdp_ng_flags flags;
+static call_t call;
+static sdp_ng_flags flags;
 static struct call_media *media_A;
 static struct call_media *media_B;
 struct call_monologue *ml_A;
@@ -47,19 +52,16 @@ struct stream_params rtp_types_sp;
 #define start() __start(__FILE__, __LINE__)
 
 static void __cleanup(void) {
-	if (flags.codec_except)
-		g_hash_table_destroy(flags.codec_except);
-	if (flags.codec_set)
-		g_hash_table_destroy(flags.codec_set);
-	if (flags.sdes_no)
-		g_hash_table_destroy(flags.sdes_no);
-	g_queue_clear_full(&flags.codec_offer, free);
-	g_queue_clear_full(&flags.codec_transcode, free);
-	g_queue_clear_full(&flags.codec_strip, free);
-	g_queue_clear_full(&flags.codec_accept, free);
-	g_queue_clear_full(&flags.codec_consume, free);
-	g_queue_clear_full(&flags.codec_mask, free);
-	g_queue_clear(&call.monologues);
+	str_case_ht_destroy_ptr(&flags.codec_except);
+	str_case_value_ht_destroy_ptr(&flags.codec_set);
+	str_case_ht_destroy_ptr(&flags.sdes_no);
+	t_queue_clear_full(&flags.codec_offer, str_free);
+	t_queue_clear_full(&flags.codec_transcode, str_free);
+	t_queue_clear_full(&flags.codec_strip, str_free);
+	t_queue_clear_full(&flags.codec_accept, str_free);
+	t_queue_clear_full(&flags.codec_consume, str_free);
+	t_queue_clear_full(&flags.codec_mask, str_free);
+	t_queue_clear(&call.monologues);
 
 	codec_store_cleanup(&rtp_types_sp.codecs);
 	memset(&flags, 0, sizeof(flags));
@@ -68,10 +70,10 @@ static void __init(void) {
 	__cleanup();
 	codec_store_init(&rtp_types_sp.codecs, NULL);
 	rtp_types_sp.rtp_endpoint.port = 9;
-	flags.codec_except = g_hash_table_new_full(str_case_hash, str_case_equal, free, NULL);
-	flags.codec_set = g_hash_table_new_full(str_case_hash, str_case_equal, free, free);
+	flags.codec_except = str_case_ht_new();
+	flags.codec_set = str_case_value_ht_new();
 }
-static struct packet_stream *ps_new(struct call *c) {
+static struct packet_stream *ps_new(call_t *c) {
 	struct packet_stream *ps = malloc(sizeof(*ps));
 	assert(ps != NULL);
 	memset(ps, 0, sizeof(*ps));
@@ -86,28 +88,30 @@ static void __start(const char *file, int line) {
 	ssrc_B = 2345;
 	ZERO(call);
 	obj_hold(&call);
-	call.tags = g_hash_table_new(g_str_hash, g_str_equal);
+	call.tags = tags_ht_new();
 	str_init(&call.callid, "test-call");
 	bencode_buffer_init(&call.buffer);
 	ml_A = __monologue_create(&call);
 	ml_B = __monologue_create(&call);
 	media_A = call_media_new(&call); // originator
 	media_B = call_media_new(&call); // output destination
-	g_queue_push_tail(&media_A->streams, ps_new(&call));
-	g_queue_push_tail(&media_B->streams, ps_new(&call));
+	t_queue_push_tail(&media_A->streams, ps_new(&call));
+	t_queue_push_tail(&media_B->streams, ps_new(&call));
 	str_init(&ml_A->tag, "tag_A");
+	str_init(&ml_A->label, "label_A");
 	media_A->monologue = ml_A;
 	media_A->protocol = &transport_protocols[PROTO_RTP_AVP];
 	str_init(&ml_B->tag, "tag_B");
+	str_init(&ml_B->label, "label_B");
 	media_B->monologue = ml_B;
 	media_B->protocol = &transport_protocols[PROTO_RTP_AVP];
 	__init();
 }
 
-#define transcode(codec) g_queue_push_tail(&flags.codec_transcode, sdup(#codec))
-#define c_accept(codec) g_queue_push_tail(&flags.codec_accept, sdup(#codec))
-#define c_consume(codec) g_queue_push_tail(&flags.codec_consume, sdup(#codec))
-#define c_mask(codec) g_queue_push_tail(&flags.codec_mask, sdup(#codec))
+#define transcode(codec) t_queue_push_tail(&flags.codec_transcode, sdup(#codec))
+#define c_accept(codec) t_queue_push_tail(&flags.codec_accept, sdup(#codec))
+#define c_consume(codec) t_queue_push_tail(&flags.codec_consume, sdup(#codec))
+#define c_mask(codec) t_queue_push_tail(&flags.codec_mask, sdup(#codec))
 
 #ifdef WITH_AMR_TESTS
 static void codec_set(char *c) {
@@ -117,7 +121,7 @@ static void codec_set(char *c) {
 	str splitter = s;
 
 	while (1) {
-		g_hash_table_replace(flags.codec_set, str_dup(&splitter), str_dup(&s));
+		t_hash_table_replace(flags.codec_set, str_dup(&splitter), str_dup(&s));
 		char *cp = memrchr(splitter.s, '/', splitter.len);
 		if (!cp)
 			break;
@@ -142,7 +146,7 @@ static void codec_set(char *c) {
 
 static void __sdp_pt_fmt(int num, str codec, int clockrate, int channels, str full_codec, str full_full, str fmt) {
 	str *fmtdup = str_dup(&fmt);
-	struct rtp_payload_type pt = (struct rtp_payload_type) {
+	rtp_payload_type pt = (rtp_payload_type) {
 		.payload_type = num,
 		.encoding_with_params = full_codec,
 		.encoding_with_full_params = full_full,
@@ -167,25 +171,23 @@ static void __sdp_pt_fmt(int num, str codec, int clockrate, int channels, str fu
 static void offer(void) {
 	printf("offer\n");
 	flags.opmode = OP_OFFER;
-	struct call_subscription subs[2];
-	struct call_subscription *subs_p[2] = {&subs[0], &subs[1]};
-	codecs_offer_answer(media_B, media_A, &rtp_types_sp, &flags, subs_p);
+
+	codecs_offer_answer(media_B, media_A, &rtp_types_sp, &flags);
 	__init();
 }
 
 static void answer(void) {
 	printf("answer\n");
 	flags.opmode = OP_ANSWER;
-	struct call_subscription subs[2];
-	struct call_subscription *subs_p[2] = {&subs[0], &subs[1]};
-	codecs_offer_answer(media_A, media_B, &rtp_types_sp, &flags, subs_p);
+
+	codecs_offer_answer(media_A, media_B, &rtp_types_sp, &flags);
 	__init();
 }
 
 #define expect(side, exp_str) \
 	__expect(__FILE__, __LINE__, &media_ ## side->codecs.codec_prefs, exp_str)
 
-static void __expect(const char *file, int line, GQueue *dumper, const char *codecs) {
+static void __expect(const char *file, int line, rtp_pt_q *dumper, const char *codecs) {
 	printf("running test %s:%i\n", file, line);
 	GString *s = g_string_new("");
 	queue_dump(s, dumper);
@@ -248,8 +250,9 @@ static void __packet_seq_ts(const char *file, int line, struct call_media *media
 	str pl_exp = pload_exp;
 
 	// from media_packet_rtp()
-	struct local_intf lif = { };
-	struct stream_fd sfd = {
+	struct interface_stats_block sblock;
+	struct local_intf lif = { .stats = &sblock };
+	stream_fd sfd = {
 		.local_intf = &lif,
 	};
 	struct media_packet mp = {
@@ -260,7 +263,7 @@ static void __packet_seq_ts(const char *file, int line, struct call_media *media
 		.sfd = &sfd,
 	};
 	// from __stream_ssrc()
-	if (!ML_ISSET(media->monologue, TRANSCODING))
+	if (!MEDIA_ISSET(media, TRANSCODING))
 		mp.ssrc_in->ssrc_map_out = ntohl(ssrc);
 	mp.ssrc_out = get_ssrc_ctx(mp.ssrc_in->ssrc_map_out, other_media->monologue->ssrc_hash, SSRC_DIR_OUTPUT, NULL);
 	payload_tracker_add(&mp.ssrc_in->tracker, pt_in & 0x7f);
@@ -304,7 +307,7 @@ static void __packet_seq_ts(const char *file, int line, struct call_media *media
 			printf("no packet\n");
 			abort();
 		}
-		struct codec_packet *cp = g_queue_pop_head(&mp.packets_out);
+		struct codec_packet *cp = t_queue_pop_head(&mp.packets_out);
 		str cp_s = cp->s;
 		rtp = (void *) cp_s.s;
 		if (rtp->m_pt != (unsigned char) pt_out) {
@@ -369,13 +372,13 @@ static void __packet_seq_ts(const char *file, int line, struct call_media *media
 static void end(void) {
 	g_hash_table_destroy(rtp_ts_ht);
 	g_hash_table_destroy(rtp_seq_ht);
-	g_queue_clear_full(&media_A->streams, free);
-	g_queue_clear_full(&media_B->streams, free);
+	t_queue_clear_full(&media_A->streams, (void (*)(struct packet_stream *)) free);
+	t_queue_clear_full(&media_B->streams, (void (*)(struct packet_stream *)) free);
 	call_media_free(&media_A);
 	call_media_free(&media_B);
 	bencode_buffer_free(&call.buffer);
-	g_hash_table_destroy(call.tags);
-	g_queue_clear(&call.medias);
+	t_hash_table_destroy(call.tags);
+	t_queue_clear(&call.medias);
 	if (ml_A)
 		__monologue_free(ml_A);
 	if (ml_B)
@@ -412,6 +415,9 @@ static void dtmf(const char *s) {
 
 int main(void) {
 	rtpe_common_config_ptr = &rtpe_config.common;
+	bufferpool_init();
+	media_bufferpool = bufferpool_new(g_malloc, g_free, 4096);
+	shm_bufferpool = bufferpool_new(g_malloc, g_free, 4096);
 
 	unsigned long random_seed = 0;
 
@@ -788,7 +794,7 @@ int main(void) {
 	dtmf("");
 	// end
 	packet_seq(A, 101, "\x08\x8a\x03\x20", 1000160, 205, 101, "\x08\x8a\x03\x20");
-	dtmf("{\"callid\":\"test-call\",\"source_tag\":\"tag_A\",\"tags\":[],\"type\":\"DTMF\",\"timestamp\":0,\"source_ip\":\"\",\"event\":8,\"duration\":100,\"volume\":10}");
+	dtmf("{\"callid\":\"test-call\",\"source_tag\":\"tag_A\",\"source_label\":\"label_A\",\"tags\":[],\"type\":\"DTMF\",\"timestamp\":0,\"source_ip\":\"\",\"event\":8,\"duration\":100,\"volume\":10}");
 	packet_seq_exp(A, 101, "\x08\x8a\x03\x20", 1000160, 205, 101, "\x08\x8a\x03\x20", 0);
 	packet_seq_exp(A, 101, "\x08\x8a\x03\x20", 1000160, 205, 101, "\x08\x8a\x03\x20", 0);
 	dtmf("");
@@ -804,7 +810,7 @@ int main(void) {
 	dtmf("");
 	// end
 	packet_seq(A, 101, "\x05\x8a\x02\x80", 1001280, 211, 101, "\x05\x8a\x02\x80");
-	dtmf("{\"callid\":\"test-call\",\"source_tag\":\"tag_A\",\"tags\":[],\"type\":\"DTMF\",\"timestamp\":0,\"source_ip\":\"\",\"event\":5,\"duration\":80,\"volume\":10}");
+	dtmf("{\"callid\":\"test-call\",\"source_tag\":\"tag_A\",\"source_label\":\"label_A\",\"tags\":[],\"type\":\"DTMF\",\"timestamp\":0,\"source_ip\":\"\",\"event\":5,\"duration\":80,\"volume\":10}");
 	packet_seq_exp(A, 101, "\x05\x8a\x02\x80", 1001280, 211, 101, "\x05\x8a\x02\x80", 0);
 	packet_seq_exp(A, 101, "\x05\x8a\x02\x80", 1001280, 211, 101, "\x05\x8a\x02\x80", 0);
 	dtmf("");
@@ -837,7 +843,7 @@ int main(void) {
 	dtmf("");
 	// end
 	packet_seq(A, 101, "\x08\x8a\x03\x20", 1000160, 205, 101, "\x08\x8a\x03\x20");
-	dtmf("{\"callid\":\"test-call\",\"source_tag\":\"tag_A\",\"tags\":[],\"type\":\"DTMF\",\"timestamp\":0,\"source_ip\":\"\",\"event\":8,\"duration\":100,\"volume\":10}");
+	dtmf("{\"callid\":\"test-call\",\"source_tag\":\"tag_A\",\"source_label\":\"label_A\",\"tags\":[],\"type\":\"DTMF\",\"timestamp\":0,\"source_ip\":\"\",\"event\":8,\"duration\":100,\"volume\":10}");
 	packet_seq_exp(A, 101, "\x08\x8a\x03\x20", 1000160, 205, 101, "\x08\x8a\x03\x20", 0);
 	packet_seq_exp(A, 101, "\x08\x8a\x03\x20", 1000160, 205, 101, "\x08\x8a\x03\x20", 0);
 	dtmf("");
@@ -853,7 +859,7 @@ int main(void) {
 	dtmf("");
 	// end
 	packet_seq(A, 101, "\x05\x8a\x02\x80", 1001280, 211, 101, "\x05\x8a\x02\x80");
-	dtmf("{\"callid\":\"test-call\",\"source_tag\":\"tag_A\",\"tags\":[],\"type\":\"DTMF\",\"timestamp\":0,\"source_ip\":\"\",\"event\":5,\"duration\":80,\"volume\":10}");
+	dtmf("{\"callid\":\"test-call\",\"source_tag\":\"tag_A\",\"source_label\":\"label_A\",\"tags\":[],\"type\":\"DTMF\",\"timestamp\":0,\"source_ip\":\"\",\"event\":5,\"duration\":80,\"volume\":10}");
 	packet_seq_exp(A, 101, "\x05\x8a\x02\x80", 1001280, 211, 101, "\x05\x8a\x02\x80", 0);
 	packet_seq_exp(A, 101, "\x05\x8a\x02\x80", 1001280, 211, 101, "\x05\x8a\x02\x80", 0);
 	dtmf("");
@@ -885,7 +891,7 @@ int main(void) {
 	dtmf("");
 	// end
 	packet_seq(A, 101, "\x08\x8a\x03\x20", 1000160, 205, 101, "\x08\x8a\x03\x20");
-	dtmf("{\"callid\":\"test-call\",\"source_tag\":\"tag_A\",\"tags\":[],\"type\":\"DTMF\",\"timestamp\":0,\"source_ip\":\"\",\"event\":8,\"duration\":100,\"volume\":10}");
+	dtmf("{\"callid\":\"test-call\",\"source_tag\":\"tag_A\",\"source_label\":\"label_A\",\"tags\":[],\"type\":\"DTMF\",\"timestamp\":0,\"source_ip\":\"\",\"event\":8,\"duration\":100,\"volume\":10}");
 	packet_seq_exp(A, 101, "\x08\x8a\x03\x20", 1000160, 205, 101, "\x08\x8a\x03\x20", 0);
 	packet_seq_exp(A, 101, "\x08\x8a\x03\x20", 1000160, 205, 101, "\x08\x8a\x03\x20", 0);
 	dtmf("");
@@ -901,7 +907,7 @@ int main(void) {
 	dtmf("");
 	// end
 	packet_seq(A, 101, "\x05\x8a\x02\x80", 1001280, 211, 101, "\x05\x8a\x02\x80");
-	dtmf("{\"callid\":\"test-call\",\"source_tag\":\"tag_A\",\"tags\":[],\"type\":\"DTMF\",\"timestamp\":0,\"source_ip\":\"\",\"event\":5,\"duration\":80,\"volume\":10}");
+	dtmf("{\"callid\":\"test-call\",\"source_tag\":\"tag_A\",\"source_label\":\"label_A\",\"tags\":[],\"type\":\"DTMF\",\"timestamp\":0,\"source_ip\":\"\",\"event\":5,\"duration\":80,\"volume\":10}");
 	packet_seq_exp(A, 101, "\x05\x8a\x02\x80", 1001280, 211, 101, "\x05\x8a\x02\x80", 0);
 	packet_seq_exp(A, 101, "\x05\x8a\x02\x80", 1001280, 211, 101, "\x05\x8a\x02\x80", 0);
 	dtmf("");
@@ -933,7 +939,7 @@ int main(void) {
 	dtmf("");
 	// end
 	packet_seq(A, 101, "\x08\x8a\x03\x20", 1000160, 205, 101, "\x08\x8a\x03\x20");
-	dtmf("{\"callid\":\"test-call\",\"source_tag\":\"tag_A\",\"tags\":[],\"type\":\"DTMF\",\"timestamp\":0,\"source_ip\":\"\",\"event\":8,\"duration\":100,\"volume\":10}");
+	dtmf("{\"callid\":\"test-call\",\"source_tag\":\"tag_A\",\"source_label\":\"label_A\",\"tags\":[],\"type\":\"DTMF\",\"timestamp\":0,\"source_ip\":\"\",\"event\":8,\"duration\":100,\"volume\":10}");
 	packet_seq_exp(A, 101, "\x08\x8a\x03\x20", 1000160, 205, 101, "\x08\x8a\x03\x20", 0);
 	packet_seq_exp(A, 101, "\x08\x8a\x03\x20", 1000160, 205, 101, "\x08\x8a\x03\x20", 0);
 	dtmf("");
@@ -951,7 +957,7 @@ int main(void) {
 	dtmf("");
 	// end
 	packet_seq(A, 101, "\x05\x8a\x02\x80", 1001280, 211, -1, "");
-	dtmf("{\"callid\":\"test-call\",\"source_tag\":\"tag_A\",\"tags\":[],\"type\":\"DTMF\",\"timestamp\":0,\"source_ip\":\"\",\"event\":5,\"duration\":80,\"volume\":10}");
+	dtmf("{\"callid\":\"test-call\",\"source_tag\":\"tag_A\",\"source_label\":\"label_A\",\"tags\":[],\"type\":\"DTMF\",\"timestamp\":0,\"source_ip\":\"\",\"event\":5,\"duration\":80,\"volume\":10}");
 	packet_seq_exp(A, 101, "\x05\x8a\x02\x80", 1001280, 211, -1, "", 0);
 	packet_seq_exp(A, 101, "\x05\x8a\x02\x80", 1001280, 211, -1, "", 0);
 	dtmf("");
@@ -998,7 +1004,7 @@ int main(void) {
 	dtmf("");
 	// end
 	packet_seq(A, 101, "\x08\x8a\x03\x20", 1000160, 205, 101, "\x08\x8a\x03\x20");
-	dtmf("{\"callid\":\"test-call\",\"source_tag\":\"tag_A\",\"tags\":[],\"type\":\"DTMF\",\"timestamp\":0,\"source_ip\":\"\",\"event\":8,\"duration\":100,\"volume\":10}");
+	dtmf("{\"callid\":\"test-call\",\"source_tag\":\"tag_A\",\"source_label\":\"label_A\",\"tags\":[],\"type\":\"DTMF\",\"timestamp\":0,\"source_ip\":\"\",\"event\":8,\"duration\":100,\"volume\":10}");
 	packet_seq_exp(A, 101, "\x08\x8a\x03\x20", 1000160, 205, 101, "\x08\x8a\x03\x20", 0);
 	packet_seq_exp(A, 101, "\x08\x8a\x03\x20", 1000160, 205, 101, "\x08\x8a\x03\x20", 0);
 	dtmf("");
@@ -1016,7 +1022,7 @@ int main(void) {
 	dtmf("");
 	// end
 	packet_seq(A, 101, "\x05\x8a\x02\x80", 1001280, 211, -1, "");
-	dtmf("{\"callid\":\"test-call\",\"source_tag\":\"tag_A\",\"tags\":[],\"type\":\"DTMF\",\"timestamp\":0,\"source_ip\":\"\",\"event\":5,\"duration\":80,\"volume\":10}");
+	dtmf("{\"callid\":\"test-call\",\"source_tag\":\"tag_A\",\"source_label\":\"label_A\",\"tags\":[],\"type\":\"DTMF\",\"timestamp\":0,\"source_ip\":\"\",\"event\":5,\"duration\":80,\"volume\":10}");
 	packet_seq_exp(A, 101, "\x05\x8a\x02\x80", 1001280, 211, -1, "", 0);
 	packet_seq_exp(A, 101, "\x05\x8a\x02\x80", 1001280, 211, -1, "", 0);
 	dtmf("");
@@ -1062,7 +1068,7 @@ int main(void) {
 	dtmf("");
 	// end
 	packet_seq(A, 101, "\x08\x8a\x03\x20", 1000160, 205, 101, "\x08\x8a\x03\x20");
-	dtmf("{\"callid\":\"test-call\",\"source_tag\":\"tag_A\",\"tags\":[],\"type\":\"DTMF\",\"timestamp\":0,\"source_ip\":\"\",\"event\":8,\"duration\":100,\"volume\":10}");
+	dtmf("{\"callid\":\"test-call\",\"source_tag\":\"tag_A\",\"source_label\":\"label_A\",\"tags\":[],\"type\":\"DTMF\",\"timestamp\":0,\"source_ip\":\"\",\"event\":8,\"duration\":100,\"volume\":10}");
 	packet_seq_exp(A, 101, "\x08\x8a\x03\x20", 1000160, 205, 101, "\x08\x8a\x03\x20", 0);
 	packet_seq_exp(A, 101, "\x08\x8a\x03\x20", 1000160, 205, 101, "\x08\x8a\x03\x20", 0);
 	dtmf("");
@@ -1080,7 +1086,7 @@ int main(void) {
 	dtmf("");
 	// end
 	packet_seq(A, 101, "\x05\x8a\x02\x80", 1001280, 211, -1, "");
-	dtmf("{\"callid\":\"test-call\",\"source_tag\":\"tag_A\",\"tags\":[],\"type\":\"DTMF\",\"timestamp\":0,\"source_ip\":\"\",\"event\":5,\"duration\":80,\"volume\":10}");
+	dtmf("{\"callid\":\"test-call\",\"source_tag\":\"tag_A\",\"source_label\":\"label_A\",\"tags\":[],\"type\":\"DTMF\",\"timestamp\":0,\"source_ip\":\"\",\"event\":5,\"duration\":80,\"volume\":10}");
 	packet_seq_exp(A, 101, "\x05\x8a\x02\x80", 1001280, 211, -1, "", 0);
 	packet_seq_exp(A, 101, "\x05\x8a\x02\x80", 1001280, 211, -1, "", 0);
 	dtmf("");
@@ -1717,6 +1723,11 @@ int main(void) {
 	expect(B, "8/PCMA/8000");
 	end();
 
+	statistics_free();
+	bufferpool_destroy(media_bufferpool);
+	bufferpool_destroy(shm_bufferpool);
+	bufferpool_cleanup();
+
 	return 0;
 }
 
